/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
 
#ifndef UDGINPUT_CPP
#define UDGINPUT_CPP

#include "input.h"
#include "volume.h"

#include <iostream>
#include <vtkImageChangeInformation.h> // per portar a l'origen

namespace udg {


Input::Input()
{
    m_imageData = 0;
    m_reader = 0;
    m_volumeData = new Volume;
    m_namesGenerator = NamesGeneratorType::New();
    m_seriesReader = SeriesReaderType::New();
    m_gdcmIO = ImageIOType::New();

    
}


Input::~Input()
{
}

bool Input::openFile( const char * fileName )
{   
    bool ok = true;
    m_reader = ReaderType::New(); 
    m_reader->SetFileName( fileName );
    try
    {
        m_reader->Update();
    }
    catch ( itk::ExceptionObject & e )
    {
        std::cerr << "Exception in file reader " << std::endl;
        std::cerr << e << std::endl;
        ok = false;
    }
    if ( ok )
    {
        m_imageData = m_reader->GetOutput(); // aix no hauria de fer-se aqu o s?
        m_volumeData->setData( m_reader->GetOutput() );
    }
    
    return ok;
}

bool Input::readSeries( const char *dirPath )
{
    bool ok = true;

    m_namesGenerator->SetInputDirectory( dirPath );    
    
    const SeriesReaderType::FileNamesContainer &filenames = m_namesGenerator->GetInputFileNames();
    // aix s necessari per desprs poder demanar-li el diccionari de meta-dades i obtenir els tags del DICOM
    
    m_seriesReader->SetImageIO( m_gdcmIO );    
    m_seriesReader->SetFileNames( filenames );
    
    try
    {   
        m_seriesReader->Update();
    }
    catch ( itk::ExceptionObject & e )
    {        
        std::cerr << "Exception in series file reader " << std::endl;
        std::cerr << e << std::endl;
        ok = false;
    }
    if ( ok )
    {   
        m_volumeData->setData( m_seriesReader->GetOutput() ); 
        m_volumeData->getItkData()->SetMetaDataDictionary( m_gdcmIO->GetMetaDataDictionary() );

        vtkImageChangeInformation* changeFilter = vtkImageChangeInformation::New(); 
        changeFilter->SetInput( m_volumeData->getVtkData() );
        changeFilter->SetOutputOrigin( 0.0 , 0.0 , 0.0 ); 
        m_volumeData->setData( changeFilter->GetOutput() );
    }
    
    return ok;
}

}; // end namespace udg

#endif
