/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#ifndef UDGINPUT_H
#define UDGINPUT_H

#include <itkImage.h>
#include <itkImageFileReader.h>
#include <itkImageSeriesReader.h>
#include <itkGDCMImageIO.h>
#include <itkGDCMSeriesFileNames.h>

namespace udg {

/**
    Aquesta classe s'ocupa de les tasques d'obrir tot tipus de fitxer


@author Grup de Grfics de Girona ( GGG )
*/

class Volume;

class Input
{
public:
    
    //typedef unsigned int                     ImagePixelType;
    typedef signed int                       ImagePixelType;
    typedef itk::Image<ImagePixelType,3>     ImageType;
    
    typedef itk::ImageFileReader<ImageType>  ReaderType;
    typedef ReaderType::Pointer    ReaderTypePointer;
    
    typedef ImageType::Pointer      ItkImageTypePointer;
    
    typedef ImageType::RegionType   RegionType;
    typedef ImageType::SizeType     SizeType;
    typedef ImageType::IndexType    IndexType;
    
    Input();

    ~Input();
    
    /// Carrega un volum a partir del nom de fitxer que se li passi
    bool openFile(const char *fileName);

    /// Retorna un Volum
    Volume* getData() const { return m_volumeData; }; 

    /// Lector de sries dicom donat un directori que les cont
    bool readSeries( const char *dirPath );
    
private:
    
    ReaderTypePointer    m_reader;
    ItkImageTypePointer m_imageData;
    Volume* m_volumeData;

    typedef itk::ImageSeriesReader< ImageType >     SeriesReaderType;
    typedef itk::GDCMImageIO                        ImageIOType;
    typedef itk::GDCMSeriesFileNames                NamesGeneratorType;    

    /// El lector de sries dicom
    SeriesReaderType::Pointer m_seriesReader;
    
    /// el lector de DICOM
    ImageIOType::Pointer m_gdcmIO;

    /// el generador dels noms dels fitxers DICOM d'un directori
    NamesGeneratorType::Pointer m_namesGenerator;

};





};  

#endif
