#include "chooseseedwidget.h"
#include <qslider.h>
#include <qspinbox.h>

#include "volume.h"
#include <qlistview.h>

#include <qstring.h>

namespace udg {

ChooseSeedWidget::ChooseSeedWidget(QWidget *parent, const char *name)
 : ChooseSeedWidgetBase(parent, name)
{
    connect(m_xView,SIGNAL(signalSeed(int,double,double,double,double)),this,SLOT(slotAddSeed(int,double,double,double,double)));
    connect(m_yView,SIGNAL(signalSeed(int,double,double,double,double)),this,SLOT(slotAddSeed(int,double,double,double,double)));
    connect(m_zView,SIGNAL(signalSeed(int,double,double,double,double)),this,SLOT(slotAddSeed(int,double,double,double,double)));

    /*
    //Posem a les 3 vistes el mateix nmero de llesca
    connect(m_xView,SIGNAL(signalSlice(int)),m_yAxisSpinBox,SLOT(setValue(int)));
    connect(m_xView,SIGNAL(signalSlice(int)),m_zAxisSpinBox,SLOT(setValue(int)));
    connect(m_yView,SIGNAL(signalSlice(int)),m_xAxisSpinBox,SLOT(setValue(int)));
    connect(m_yView,SIGNAL(signalSlice(int)),m_zAxisSpinBox,SLOT(setValue(int)));
    connect(m_zView,SIGNAL(signalSlice(int)),m_xAxisSpinBox,SLOT(setValue(int)));
    connect(m_zView,SIGNAL(signalSlice(int)),m_yAxisSpinBox,SLOT(setValue(int)));
    */
    m_nAnt=-1;
    
    m_listViewSeed->setSorting(-1,TRUE);
}
  

ChooseSeedWidget::~ChooseSeedWidget()
{
}


void ChooseSeedWidget::setInputImage(Volume* newImageData)
{
    
    this->m_xView->setVolume( newImageData );
    this->m_yView->setVolume( newImageData );
    this->m_zView->setVolume( newImageData );
  

    this->m_xView->setView( QSeedSliceViewer::Axial );
    this->m_yView->setView( QSeedSliceViewer::Sagittal );
    this->m_zView->setView( QSeedSliceViewer::Coronal );
  
    int *size = newImageData->getVtkData()->GetDimensions();

    this->m_xAxisSlider->setMaxValue( size[2]-1 );
    this->m_yAxisSlider->setMaxValue( size[1]-1 );
    this->m_zAxisSlider->setMaxValue( size[0]-1 );
    
    this->m_xAxisSpinBox->setMaxValue( size[2]-1 );
    this->m_yAxisSpinBox->setMaxValue( size[1]-1 );
    this->m_zAxisSpinBox->setMaxValue( size[0]-1 );

    // Posem tots els sliders i valors spinboxs a z/2. Els sliders automaticament s'hi posaran
    // igual que les imatges tamb. (gracies a la propagacio de signals-slots)
    this->m_xAxisSpinBox->setValue( size[2]/2 );
    this->m_yAxisSpinBox->setValue( size[1]/2 );
    this->m_zAxisSpinBox->setValue( size[0]/2);

} 


void ChooseSeedWidget::slotAddSeed(int view,double x,double y,double z,double v)
{   
    QString strX, strY, strZ, strV;

    strX.sprintf("%g",x);
    strY.sprintf("%g",y);
    strZ.sprintf("%g",z);
    strV.sprintf("%g",v);

    (void)new QListViewItem(m_listViewSeed,strX,strY,strZ,strV);

    //Vista X
    if(view==0)
    {		
	m_yView->drawSeed(y,z,x,v);
	m_zView->drawSeed(x,z,y,v);
    }
    
    //Vista Y
    else if(view==1)
    {
	m_xView->drawSeed(z,x,y,v);	
	m_zView->drawSeed(z,y,x,v);
    }
    
    //Vista Z
    else
    {
	m_xView->drawSeed(x,z,y,v);
	m_yView->drawSeed(z,y,x,v);
    }


}


void ChooseSeedWidget::slotDelete()
{   
    if(m_listViewSeed->isSelected(m_listViewSeed->selectedItem()))
    {
        int nitems=m_listViewSeed->childCount();
        int n=nitems-(m_listViewSeed->selectedItem()->itemPos()/20)-1;

        m_xView->deleteSeed(n);
        m_yView->deleteSeed(n);
        m_zView->deleteSeed(n); 

        m_listViewSeed->takeItem(m_listViewSeed->selectedItem());
    }
}


void ChooseSeedWidget::slotDeleteRange(QListViewItem* item)
{
    int nitems=m_listViewSeed->childCount();
    int n=nitems-(m_listViewSeed->itemPos(item)/20)-1;

    m_xView->deleteSeed(n);
    m_yView->deleteSeed(n);
    m_zView->deleteSeed(n); 

    m_listViewSeed->takeItem(item);
}

void ChooseSeedWidget::slotOk()
{
    emit signalOk();
    close();
}

void ChooseSeedWidget::slotClose()
{
    close();
}

void ChooseSeedWidget::slotSelectItem()
{
    int nitems=m_listViewSeed->childCount();
    int n=nitems-(m_listViewSeed->selectedItem()->itemPos()/20)-1;
    
    m_xView->selectSeed(n,m_nAnt);
    m_yView->selectSeed(n,m_nAnt);
    m_zView->selectSeed(n,m_nAnt);

    m_nAnt=n;
}

QListView* ChooseSeedWidget::getSeedList()
{
    return m_listViewSeed;
}

}; // end namespace udg

