#include "confidenceconnecteddirector.h"
#include "confidenceconnectedparameters.h"

#include "volumerepository.h"

// per la finestra que obrim com a resultat
#include "qwidgetvolume.h"

// per tenir accs a l'aplication window 
#include "qapplicationmainwindow.h"

#include "viewresultswidget.h"

// per debug
#include <iostream.h>

#include <time.h>

namespace udg {
 
ConfidenceConnectedDirector::ConfidenceConnectedDirector(udg::QApplicationMainWindow* app, QObject *parent, const char *name):Director(app, parent, name)
{
    m_applicationWindow = app;
    m_parameters = 0;
    m_method = new MethodType;    
}


ConfidenceConnectedDirector::~ConfidenceConnectedDirector()
{
    delete m_method;    
}



void ConfidenceConnectedDirector::setParameters( ConfidenceConnectedParameters* parameters )
{
	m_parameters = parameters;
}


void ConfidenceConnectedDirector::execute()
{
 
    if( !m_parameters )
    {
        // Si no hi ha parmetres establerts no podem fer res
        // \TODO exception, cerr,... ?
        std::cerr << "ConfidenceConnectedDirector::No hi ha parametres establerts" << std::endl;
    }
    else
    {	
        // obtenim el repositori
        udg::VolumeRepository* repository = udg::VolumeRepository::getRepository();

        // obtenim el volum a treballar
        Volume* volum = repository->getVolume( m_parameters->getVolumeId() );

        //Ccul del temps d'execuci
        time_t start,end;
        double dif;
        time (&start);

        // li indiquem al mtode els parmetres necessaris
        m_method->setInputVolume( volum );        
        m_method->setSmoothingParameters();         
        m_method->addSeeds(m_parameters->getSeedList());
        m_method->setParameters( m_parameters->getMultiplierFactor(),m_parameters->getNumberIterations(),m_parameters->getRadius());

        if( m_method->applyMethod() )
        {   
            ViewResultsWidget* viewResults = new ViewResultsWidget(m_applicationWindow, "SegmentedVolumeWidget");            
            viewResults->setInputImage( this->getSegmentedVolume(),this->getBinaryVolume() );
            m_applicationWindow->addWorkingAreaWidget(viewResults, tr("Segmented Volume"));            
            viewResults->show();

            time (&end);
            dif = difftime (end,start);
            int minuts=dif/60;
            int segons=dif-(minuts*60);
            cout <<"----------------------------------------" << endl;
            cout <<"Temps Execucio Confidence Connected: " << minuts << " min. " << segons << " seg. " << endl;
            cout <<"----------------------------------------" << endl;
	}

        else
        {
            std::cerr << " No s'ha pogut aplicar el mtode de segmentaci" << std::endl;
        }
    }
}



udg::Volume* ConfidenceConnectedDirector::getSegmentedVolume()
{
	return m_method->getSegmentedVolume();
}

udg::Volume* ConfidenceConnectedDirector::getBinaryVolume()
{
	return m_method->getBinaryVolume();
}

}; //end namespace udg
