/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#ifndef UDGCONFIDENCECONNECTEDPARAMETERS_H
#define UDGCONFIDENCECONNECTEDPARAMETERS_H

#include "parameters.h"
#include "identifier.h"
#include "qlistview.h"

namespace udg {

/**
    Sub-classe de Parameters. En aquest cas encapsula els parmetres del mtode de segmentaci
    ConfidenceConnected.
    
    Tindrem un mtode set/get per tots i cadascun dels parmetres del mtode.
    Els mtodes set emetran el signal changed amb l'id de parmetre que hem creat amb el tipus
    enumerat
*/

class ConfidenceConnectedParameters : public Parameters
{
Q_OBJECT
public:
    
    /**
        Aquest tipus enumerat ens serveix per crear els identificadors de cada parmetre.
    */
     enum ConfidenceConnectedParametersNames { VolumeId, MultiplierFactor, NumberIterations, Radius, SeedList };

    ConfidenceConnectedParameters(QObject *parent = 0, const char *name = 0);

    ~ConfidenceConnectedParameters();

    
    void setVolumeId( udg::Identifier id)
    {   m_volumeId= id;	emit changed(VolumeId); }
    
    void setMultiplierFactor(double factor)
    {  	m_multiplierFactor=factor; emit changed(MultiplierFactor); }
    
    void setNumberIterations(int n)
    {	m_numberIterations=n; emit changed(NumberIterations); }

    void setRadius(double radius)
    {	m_radius=radius; emit changed(Radius); }

    void setSeedList(QListView* l)
    {	m_seedList=l; emit changed(SeedList); }

    udg::Identifier getVolumeId() const
    { 	return m_volumeId;}
    
    double getMultiplierFactor() const
    {  	return m_multiplierFactor; }
    
    int getNumberIterations() const
    { 	return m_numberIterations; }

    double getRadius() const
    {  	return m_radius; }

    QListView* getSeedList() const
    {	return m_seedList; }


 private:

    //Identificador del volum de dades amb el que treballem
    udg::Identifier m_volumeId;
    
    //Parmetres del mtode
    int m_numberIterations;
    double m_multiplierFactor,m_radius;
    QListView* m_seedList;
};

};

#endif
