/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#ifndef UDGFASTMARCHINGPARAMETERS_H
#define UDGFASTMARCHINGPARAMETERS_H

#include "parameters.h"
#include "identifier.h"
#include "qlistview.h"

namespace udg {

/**
    Sub-classe de Parameters. En aquest cas encapsula els parmetres del mtode de segmentaci
    FastMarching.
    
    Tindrem un mtode set/get per tots i cadascun dels parmetres del mtode.
    Els mtodes set emetran el signal changed amb l'id de parmetre que hem creat amb el tipus
    enumerat
*/

class FastMarchingParameters : public Parameters
{
Q_OBJECT
public:
    
    /**
        Aquest tipus enumerat ens serveix per crear els identificadors de cada parmetre.
    */
     enum FastMarchingParametersNames { VolumeId, Sigma, Alpha, Beta, Threshold, StopValue, SeedList };

    FastMarchingParameters(QObject *parent = 0, const char *name = 0);

    ~FastMarchingParameters();

    
    void setVolumeId( udg::Identifier id)
    {   m_volumeId= id;	emit changed(VolumeId); }
    
    void setSigma(double sigma)
    {  	m_sigma=sigma;	emit changed(Sigma); }
    
    void setAlpha(double alpha)
    {	m_alpha=alpha;	emit changed(Alpha); }

    void setBeta(double beta)
    {  	m_beta=beta;	emit changed(Beta); }
    
    void setThreshold(int threshold)
    {	m_threshold=threshold;	emit changed(Threshold); }

    void setStopValue(int stop)
    {  	m_stopValue=stop;  emit changed(StopValue); }

    void setSeedList(QListView* l)
    {	m_seedList=l; emit changed(SeedList); }

    udg::Identifier getVolumeId() const
    { 	return m_volumeId;}
    
    double getSigma() const
    {  	return m_sigma; }
    
    double getAlpha() const
    { 	return m_alpha; }
    
    double getBeta() const
    {  	return m_beta; }
    
    int getThreshold() const
    { 	return m_threshold; }

    int getStopValue() const
    {  	return m_stopValue; }

    QListView* getSeedList() const
    {	return m_seedList; }


 private:

    //Identificador del volum de dades amb el que treballem
    udg::Identifier m_volumeId;
    
    //Parmetres del mtode
    double m_sigma, m_alpha, m_beta;
    int m_threshold, m_stopValue;
    QListView* m_seedList;
};

};

#endif
