/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#include "mutualinformationdirector.h"
#include "mutualinformationparameters.h"

#include "volumerepository.h"

// per la finestra que obrim com a resultat
#include "qwidgetvolume.h"

// per tenir accs a l'aplication window 
#include "qapplicationmainwindow.h"

// per debug
#include <iostream.h>

namespace udg{

MutualInformationDirector::MutualInformationDirector( udg::QApplicationMainWindow* app,QObject *parent, const char *name )
 : Director(app, parent, name)
{
    m_applicationWindow = app; // aix no s del tot necessari perqu ja ho fa el pare
    
    m_method = new MethodType;    
    m_parameters = 0;
}

MutualInformationDirector::~MutualInformationDirector()
{
    delete m_method;
}

void MutualInformationDirector::setParameters( MutualInformationParameters* parameters )
{
    m_parameters = parameters;
}

void MutualInformationDirector::execute()
{
    bool ok = true; // \TODO Aix no es fa servir, cal?
    
    if( !m_parameters )
    {
        // Si no hi ha parmetres establerts no podem fer res
        // \TODO exception, cerr,... ?
        std::cerr << "MutualInformationDirector::No hi ha parametres establerts" << std::endl;
        ok = false;
    }
    else
    {
        // obtenim el repositori
        udg::VolumeRepository* repository = udg::VolumeRepository::getRepository();
                
        // li indiquem al mtode els parmetres
        
        m_method->SetInputImages( repository->getVolume( m_parameters->getFixedImageID() )->getItkData(), repository->getVolume( m_parameters->getMovingImageID() )->getItkData() );
        
    
        m_method->SetMetricParameters( m_parameters->getFixedImageStandardDeviation() ,
                                    m_parameters->getMovingImageStandardDeviation(),
                                    m_parameters->getSpatialSamples() );
        
        m_method->SetGaussianFilterParameters( m_parameters->getFixedImageVariance(), 
                                            m_parameters->getMovingImageVariance() );
                                            
        m_method->SetOptimizerParameters( m_parameters->getLearningRate(), 
                                        m_parameters->getNumberOfIterations() ); 
        
        if( m_method->applyMethod() )
        {
            // obrirem les finestres grcies al coneixement de l'application window
            // Primer es crea el tipus de finestra que volem mostrar
            // de mentres fem servir QWidgetVolume perqu ens serveixi d'exemple
            QWidgetVolume* registeredVolumeWindow = new QWidgetVolume(0, "RegisteredVolumeWidget");
            registeredVolumeWindow->setVolume( this->getRegisteredImage() );
            m_applicationWindow->addWorkingAreaWidget( registeredVolumeWindow , tr("Registered Volume") );
        }
        else
        {
            std::cerr << " No s'ha pogut aplicar el mtode " << std::endl;
        }
        
        
    }
}

udg::Volume* MutualInformationDirector::getRegisteredImage()
{
    udg::Volume* volume = new udg::Volume();
    
    //volume->setData( m_method->getRegisteredImage() );
     
    return volume; 
}

} // end namespace udg

