/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
 
#ifndef MUTUALINFORMATIONDIRECTOR_H
#define MUTUALINFORMATIONDIRECTOR_H

#include "director.h"
#include "volume.h"
#include "mutualinformationviolawells.h"


namespace udg{

//Fordware declaration
class MutualInformationParameters;

/**
    Sub-classe de Director. En aquest cas fa de pont amb un mtode de registre basat en mesures 
    d'informaci mtua.
    
    Aquesta inclou un punter a la classe de parmetres especfics del seu mtode, la classe que 
    encapsula el mtode de registre ( MutualInformationViolaWells )
    
    Totes les classes heredades de Director, inclouran un mtode setParameters on l'indicarem el 
    punter a la classe de parmetres corresponent.
    Tamb implementaran l'slot virtual apply, on es far la feina ms important de l'execuci d'un 
    mtode
    
*/

class MutualInformationDirector : public Director
{

 Q_OBJECT
 
public:

    // Com que el mtode treballa amb dades itk, hem d'extreure el tipus itk
    // per declarar el mtode que fa servir templates.
    typedef udg::Volume::ItkImageType ImageType;
    typedef udg::MutualInformationViolaWells< ImageType, ImageType > MethodType;    
    
    MutualInformationDirector( udg::QApplicationMainWindow* app, QObject *parent = 0, const char *name = 0 );

    ~MutualInformationDirector();

    
    void setParameters( MutualInformationParameters* parameters );
    
    
public slots:
    /**
        Aquest mtode aplicar el mtode de registre. Comprova primer si t els parmetres
        necessaris, sin retornar fals. Slot pblic heredat de Director.
    */
    void execute();

private:

    MethodType *m_method;
    MutualInformationParameters *m_parameters;
    
    udg::QApplicationMainWindow* m_applicationWindow;
    
    udg::Volume* getRegisteredImage(); 
};

}; // end namespace udg

#endif
