/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

 
// includes Qt indispensables
#include <qaction.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qsettings.h>
#include <qstatusbar.h>
#include <qdockwindow.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qmessagebox.h>

#include <qfileinfo.h> //Per m_workingDirectory

#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>

// els nostres widgets
#include "qapplicationmainwindow.h"
#include "qworkingarea.h"

#include "qmaintoolbox.h"
#include "qworkingareawindow.h" //Per el mtode addWorkingAreaWidget
#include "qworkingareawidget.h" //Per el mtode addWorkingAreaWidget
#include "qsimplesliceviewer.h" //Per mostrar model en 2D
#include "qwidgetvolume.h" // Per mostrar una vista 3D dels plans
#include "qsimple3dviewer.h" // vista 3D amb vol rendering ray casting

// elements de la plataforma
#include "volumerepository.h"
#include "identifier.h"
#include "volume.h"
#include "input.h"

// ------------------------------------------------------------------------------------
// Inici zona de codi editable [ #include's dels forward delcaration ]
// ------------------------------------------------------------------------------------

// include's afegits
#include "mutualinformationdirector.h"
#include "mutualinformationparameters.h"
#include "mutualinformationinputparametersform.h"

#include "connectedthresholddirector.h"
#include "connectedthresholdparameters.h"
#include "connectedthresholdinputparametersform.h"

#include "neighborhoodconnecteddirector.h"
#include "neighborhoodconnectedparameters.h"
#include "neighborhoodconnectedinputparametersform.h"

#include "confidenceconnecteddirector.h"
#include "confidenceconnectedparameters.h"
#include "confidenceconnectedinputparametersform.h"

#include "isolatedconnecteddirector.h"
#include "isolatedconnectedparameters.h"
#include "isolatedconnectedinputparametersform.h"

/*#include "watersheddirector.h"
#include "watershedparameters.h"
#include "watershedinputparametersform.h"
*/
#include "fastmarchingdirector.h"
#include "fastmarchingparameters.h"
#include "fastmarchinginputparametersform.h"

#include "shapedetectiondirector.h"
#include "shapedetectionparameters.h"
#include "shapedetectioninputparametersform.h"

#include "geodesicactivedirector.h"
#include "geodesicactiveparameters.h"
#include "geodesicactiveinputparametersform.h"

/*#include "thresholdlevelsetdirector.h"
#include "thresholdlevelsetparameters.h"
#include "thresholdlevelsetinputparametersform.h"

#include "cannyedgelevelsetdirector.h"
#include "cannyedgelevelsetparameters.h"
#include "cannyedgelevelsetinputparametersform.h"

#include "laplacianlevelsetdirector.h"
#include "laplacianlevelsetparameters.h"
#include "laplacianlevelsetinputparametersform.h"
*/

#include "raycastingdirector.h"
#include "raycastingparameters.h"
#include "raycastinginputparametersform.h"

#include "multiplanedirector.h"
#include "multiplaneinputparametersform.h"
// ------------------------------------------------------------------------------------
// Fi zona de codi editable
// ------------------------------------------------------------------------------------


namespace udg{


QApplicationMainWindow::QApplicationMainWindow(QWidget *parent, const char *name)
    : QMainWindow(parent, name, WDestructiveClose)
{
    m_workingArea = new QWorkingArea(this);
    setCentralWidget( m_workingArea );

// ------------------------------------------------------------------------------------
// Inici zona de codi editable
// ------------------------------------------------------------------------------------
    
    // ------------------------------------------------------------------------------------
    // Aqui creem els directors i parametres de cada mtode
    // ------------------------------------------------------------------------------------
    
    // ------------------------------------------------------------------------------------
    // Secci : Segmentaci
    // ------------------------------------------------------------------------------------
    
    // Connected Threshold
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_connectedThresholdDirector = new ConnectedThresholdDirector( this );
    m_connectedThresholdParameters = new ConnectedThresholdParameters( this );
    
    // Li assignem al director quins sn els seus parmetres
    m_connectedThresholdDirector->setParameters( m_connectedThresholdParameters );

    // Creem els widgets que aniran al toolbox
    m_connectedThresholdInputParametersForm = new ConnectedThresholdInputParametersForm( this, "Connected Threshold Page" );
    // li assignem els parmetres
    m_connectedThresholdInputParametersForm->setParameters( m_connectedThresholdParameters );

    // connectem l'acci amb el director
    m_connectedThresholdAction = new QAction( this );
    m_connectedThresholdAction->setMenuText(tr("&Connected Threshold Information"));
    connect( m_connectedThresholdAction , SIGNAL( activated() ),  m_connectedThresholdDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_connectedThresholdParameters, SIGNAL( changed(int) ), m_connectedThresholdInputParametersForm, SLOT( readParameter( int ) ) );
    

    // Neighborhood Connected
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_neighborhoodConnectedDirector = new NeighborhoodConnectedDirector( this );
    m_neighborhoodConnectedParameters = new NeighborhoodConnectedParameters( this );
    
    // Li assignem al director quins sn els seus parmetres
    m_neighborhoodConnectedDirector->setParameters( m_neighborhoodConnectedParameters );

    // Creem els widgets que aniran al toolbox
    m_neighborhoodConnectedInputParametersForm = new NeighborhoodConnectedInputParametersForm( this, "Neighborhood Connected Page" );
    // li assignem els parmetres
    m_neighborhoodConnectedInputParametersForm->setParameters( m_neighborhoodConnectedParameters );

    // connectem l'acci amb el director
    m_neighborhoodConnectedAction = new QAction( this );
    m_neighborhoodConnectedAction->setMenuText(tr("&Neighborhood Connected Information"));
    connect( m_neighborhoodConnectedAction , SIGNAL( activated() ),  m_neighborhoodConnectedDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_neighborhoodConnectedParameters, SIGNAL( changed(int) ), m_neighborhoodConnectedInputParametersForm, SLOT( readParameter( int ) ) );
    

    // Confidence Connected
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_confidenceConnectedDirector = new ConfidenceConnectedDirector( this );
    m_confidenceConnectedParameters = new ConfidenceConnectedParameters( this );
    
    // Li assignem al director quins sn els seus parmetres
    m_confidenceConnectedDirector->setParameters( m_confidenceConnectedParameters );

    // Creem els widgets que aniran al toolbox
    m_confidenceConnectedInputParametersForm = new ConfidenceConnectedInputParametersForm( this, "Confidence Connected Page" );
    // li assignem els parmetres
    m_confidenceConnectedInputParametersForm->setParameters( m_confidenceConnectedParameters );
    
   // connectem l'acci amb el director
    m_confidenceConnectedAction = new QAction( this );
    m_confidenceConnectedAction->setMenuText(tr("&Confidence Connected Information"));
    connect( m_confidenceConnectedAction , SIGNAL( activated() ),  m_confidenceConnectedDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_confidenceConnectedParameters, SIGNAL( changed(int) ), m_confidenceConnectedInputParametersForm, SLOT( readParameter( int ) ) );
    

    // Isolated Connected
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_isolatedConnectedDirector = new IsolatedConnectedDirector( this );
    m_isolatedConnectedParameters = new IsolatedConnectedParameters( this );
    
    // Li assignem al director quins sn els seus parmetres
    m_isolatedConnectedDirector->setParameters( m_isolatedConnectedParameters );

    // Creem els widgets que aniran al toolbox
    m_isolatedConnectedInputParametersForm = new IsolatedConnectedInputParametersForm( this, "Isolated Connected Page" );
    // li assignem els parmetres
    m_isolatedConnectedInputParametersForm->setParameters( m_isolatedConnectedParameters );
    
   // connectem l'acci amb el director
    m_isolatedConnectedAction = new QAction( this );
    m_isolatedConnectedAction->setMenuText(tr("&Isolated Connected Information"));
    connect( m_isolatedConnectedAction , SIGNAL( activated() ),  m_isolatedConnectedDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_isolatedConnectedParameters, SIGNAL( changed(int) ), m_isolatedConnectedInputParametersForm, SLOT( readParameter( int ) ) );
    

    /*
    // Watershed
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_watershedDirector = new WatershedDirector( this );
    m_watershedParameters = new WatershedParameters( this );
    
    // Li assignem al director quins sn els seus parmetres
    m_watershedDirector->setParameters( m_watershedParameters );

    // Creem els widgets que aniran al toolbox
    m_watershedInputParametersForm = new WatershedInputParametersForm( this, "Watershed Page" );
    // li assignem els parmetres
    m_watershedInputParametersForm->setParameters( m_watershedParameters );
    
    // connectem l'acci amb el director
    m_watershedAction = new QAction( this );
    m_watershedAction->setMenuText(tr("&Watershed Information"));
    connect( m_watershedAction , SIGNAL( activated() ),  m_watershedDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_watershedParameters, SIGNAL( changed(int) ), m_watershedInputParametersForm, SLOT( readParameter( int ) ) );
    */

    // Fast Marching
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_fastMarchingDirector = new FastMarchingDirector( this );
    m_fastMarchingParameters = new FastMarchingParameters( this );
    
    // Li assignem al director quins sn els seus parmetres
    m_fastMarchingDirector->setParameters( m_fastMarchingParameters );

    // Creem els widgets que aniran al toolbox
    m_fastMarchingInputParametersForm = new FastMarchingInputParametersForm( this, "Fast Marching Page" );
    // li assignem els parmetres
    m_fastMarchingInputParametersForm->setParameters( m_fastMarchingParameters );
    
    // connectem l'acci amb el director
    m_fastMarchingAction = new QAction( this );
    m_fastMarchingAction->setMenuText(tr("&Fast Marching Information"));
    connect( m_fastMarchingAction , SIGNAL( activated() ),  m_fastMarchingDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_fastMarchingParameters, SIGNAL( changed(int) ), m_fastMarchingInputParametersForm, SLOT( readParameter( int ) ) );
    
     
    // Shape Detection
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_shapeDetectionDirector = new ShapeDetectionDirector( this );
    m_shapeDetectionParameters = new ShapeDetectionParameters( this );
     
    // Li assignem al director quins sn els seus parmetres
    m_shapeDetectionDirector->setParameters( m_shapeDetectionParameters );
 
    // Creem els widgets que aniran al toolbox
    m_shapeDetectionInputParametersForm = new ShapeDetectionInputParametersForm( this, "Shape Detection Page" );
    // li assignem els parmetres
    m_shapeDetectionInputParametersForm->setParameters( m_shapeDetectionParameters );
 
    // connectem l'acci amb el director
    m_shapeDetectionAction = new QAction( this );
    m_shapeDetectionAction->setMenuText(tr("&Shape Detection Information"));
    connect( m_shapeDetectionAction , SIGNAL( activated() ),  m_shapeDetectionDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_shapeDetectionParameters, SIGNAL( changed(int) ), m_shapeDetectionInputParametersForm, SLOT( readParameter( int ) ) );



    //Geodesic Active 
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_geodesicActiveDirector = new GeodesicActiveDirector( this );
    m_geodesicActiveParameters = new GeodesicActiveParameters( this );
     
    // Li assignem al director quins sn els seus parmetres
    m_geodesicActiveDirector->setParameters( m_geodesicActiveParameters );
 
    // Creem els widgets que aniran al toolbox
    m_geodesicActiveInputParametersForm = new GeodesicActiveInputParametersForm( this, "Shape Detection Page" );
    // li assignem els parmetres
    m_geodesicActiveInputParametersForm->setParameters( m_geodesicActiveParameters );
 
    // connectem l'acci amb el director
    m_geodesicActiveAction = new QAction( this );
    m_geodesicActiveAction->setMenuText(tr("&Shape Detection Information"));
    connect( m_geodesicActiveAction , SIGNAL( activated() ),  m_geodesicActiveDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_geodesicActiveParameters, SIGNAL( changed(int) ), m_geodesicActiveInputParametersForm, SLOT( readParameter( int ) ) );


    /*
    // Threshold Level Set
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_thresholdLevelSetDirector = new ThresholdLevelSetDirector( this );
    m_thresholdLevelSetParameters = new ThresholdLevelSetParameters( this );
     
    // Li assignem al director quins sn els seus parmetres
    m_thresholdLevelSetDirector->setParameters( m_thresholdLevelSetParameters );
 
    // Creem els widgets que aniran al toolbox
    m_thresholdLevelSetInputParametersForm = new ThresholdLevelSetInputParametersForm( this, "Shape Detection Page" );
    // li assignem els parmetres
    m_thresholdLevelSetInputParametersForm->setParameters( m_thresholdLevelSetParameters );
 
    // connectem l'acci amb el director
    m_thresholdLevelSetAction = new QAction( this );
    m_thresholdLevelSetAction->setMenuText(tr("&Shape Detection Information"));
    connect( m_thresholdLevelSetAction , SIGNAL( activated() ),  m_thresholdLevelSetDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_thresholdLevelSetParameters, SIGNAL( changed(int) ), m_thresholdLevelSetInputParametersForm, SLOT( readParameter( int ) ) );



    // Canny Edge Level Set
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_cannyEdgeLevelSetDirector = new CannyEdgeLevelSetDirector( this );
    m_cannyEdgeLevelSetParameters = new CannyEdgeLevelSetParameters( this );
     
    // Li assignem al director quins sn els seus parmetres
    m_cannyEdgeLevelSetDirector->setParameters( m_cannyEdgeLevelSetParameters );
 
    // Creem els widgets que aniran al toolbox
    m_cannyEdgeLevelSetInputParametersForm = new CannyEdgeLevelSetInputParametersForm( this, "Shape Detection Page" );
    // li assignem els parmetres
    m_cannyEdgeLevelSetInputParametersForm->setParameters( m_cannyEdgeLevelSetParameters );
 
    // connectem l'acci amb el director
    m_cannyEdgeLevelSetAction = new QAction( this );
    m_cannyEdgeLevelSetAction->setMenuText(tr("&Shape Detection Information"));
    connect( m_cannyEdgeLevelSetAction , SIGNAL( activated() ),  m_cannyEdgeLevelSetDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_cannyEdgeLevelSetParameters, SIGNAL( changed(int) ), m_cannyEdgeLevelSetInputParametersForm, SLOT( readParameter( int ) ) );



    // Laplacian Level Set
    // ------------------------------------------------------------------------------------
     
    // Creem el director i els parmetres
    m_laplacianLevelSetDirector = new LaplacianLevelSetDirector( this );
    m_laplacianLevelSetParameters = new LaplacianLevelSetParameters( this );
     
    // Li assignem al director quins sn els seus parmetres
    m_laplacianLevelSetDirector->setParameters( m_laplacianLevelSetParameters );
 
    // Creem els widgets que aniran al toolbox
    m_laplacianLevelSetInputParametersForm = new LaplacianLevelSetInputParametersForm( this, "Shape Detection Page" );
    // li assignem els parmetres
    m_laplacianLevelSetInputParametersForm->setParameters( m_laplacianLevelSetParameters );
 
    // connectem l'acci amb el director
    m_laplacianLevelSetAction = new QAction( this );
    m_laplacianLevelSetAction->setMenuText(tr("&Shape Detection Information"));
    connect( m_laplacianLevelSetAction , SIGNAL( activated() ),  m_laplacianLevelSetDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_laplacianLevelSetParameters, SIGNAL( changed(int) ), m_laplacianLevelSetInputParametersForm, SLOT( readParameter( int ) ) );

*/

    // ------------------------------------------------------------------------------------
    // Secci : Registre
    // ------------------------------------------------------------------------------------
    
    // Informaci Mtua / MultiModal
    // ------------------------------------------------------------------------------------
    
    // Creem el director i els parmetres
    m_mutualInformationDirector = new MutualInformationDirector( this );
    m_mutualInformationParameters = new MutualInformationParameters( this );
     
    // Li assignem al director quins sn els seus parmetres
    m_mutualInformationDirector->setParameters( m_mutualInformationParameters );
     
    // Creem els widgets que aniran al toolbox
    m_mutualInformationInputParametersForm = new MutualInformationInputParametersForm( this, "Mutual Page" );     
    // li assignem els parmetres 
    m_mutualInformationInputParametersForm->setParameters( m_mutualInformationParameters );
   
    // connectem l'acci amb el director
    m_mutualInformationAction = new QAction( this );
    m_mutualInformationAction->setMenuText(tr("&Mutual Information"));
    connect( m_mutualInformationAction , SIGNAL( activated() ),  m_mutualInformationDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_mutualInformationParameters, SIGNAL( changed(int) ), m_mutualInformationInputParametersForm, SLOT( readParameter( int ) ) );
    

    // ------------------------------------------------------------------------------------
    // Secci : Visualitzaci
    // ------------------------------------------------------------------------------------
    // Ray Casting
    // ------------------------------------------------------------------------------------

    // Creem el director i els parmetres
    m_rayCastingDirector = new RayCastingDirector( this );
    m_rayCastingParameters = new RayCastingParameters( this );

    // Li assignem al director quins sn els seus parmetres
    m_rayCastingDirector->setParameters( m_rayCastingParameters );

    // Creem els widgets que aniran al toolbox
    m_rayCastingInputParametersForm = new RayCastingInputParametersForm( this, "Ray Casting Page" );
    // li assignem els parmetres
    m_rayCastingInputParametersForm->setParameters( m_rayCastingParameters );

    // connectem l'acci amb el director
    m_rayCastingAction = new QAction( this );
    m_rayCastingAction->setMenuText(tr("&Ray Casting Information"));
    connect( m_rayCastingAction , SIGNAL( activated() ),  m_rayCastingDirector, SLOT( execute() ) );
    // connectem els parmetres amb les interfcies
    // quan un parmetre s'actualitzi, s'actualitzar a tots els widgets que el tinguin com a input
    connect( m_rayCastingParameters, SIGNAL( changed(int) ), m_rayCastingInputParametersForm, SLOT( readParameter( int ) ) );


    // ------------------------------------------------------------------------------------
    // Multiplane
    // ------------------------------------------------------------------------------------

    // Creem el director
    m_multiplaneDirector = new MultiplaneDirector( this );
    // Creem els widgets que aniran al toolbox
    m_multiplaneInputParametersForm = new MultiplaneInputParametersForm( this, "Multiplane Page" );

    // connectem l'acci amb el director
    m_multiplaneAction = new QAction( this );
    m_multiplaneAction->setMenuText(tr("&Multiplane Information"));
    connect( m_multiplaneAction , SIGNAL( activated() ),  m_multiplaneDirector, SLOT( execute() ) );


// ------------------------------------------------------------------------------------
// Fi de zona de codi editable
// ------------------------------------------------------------------------------------    
    
    // ------------------------------------------------------------------------------------
    // A partir d'aqu no s'ha de tocar res ms
    // ------------------------------------------------------------------------------------
    
    // ------------------------------------------------------------------------------------
    // aqu creem el repositori de volums i l'objecte input per poder accedir als arxius
    // ------------------------------------------------------------------------------------
    
    m_volumeRepository = udg::VolumeRepository::getRepository();
    m_inputReader = new udg::Input;
    
    createActions();    
    createMenus();
    createToolBars();
    createStatusBar();
    createMainToolBox();
    
    // Llegim les configuracions de l'aplicaci, estat de la finestra, posicio, ltims
    // arxius oberts etc amb QSettings
    readSettings();

    setCaption( tr("VisualPork") );
    setIcon( QPixmap::fromMimeSource("icon.png") );

    m_fileFilters = tr("MetaIO Images (*.mhd);;DICOM Images (*.dcm);;All Files (*)");
    m_fileFilters2 = tr("DICOM Images (*.dcm);;MetaIO Images (*.mhd);;All Files (*)");
    m_modified = false;
                                    
}

QApplicationMainWindow::~ QApplicationMainWindow()
{
    m_volumeRepository->removeVolume( m_volumeID );
}

void QApplicationMainWindow::createActions()
{
    m_newAction = new QAction(tr("&New"), tr("Ctrl+N"), this);
    m_newAction->setIconSet(QPixmap::fromMimeSource("new.png"));
    m_newAction->setStatusTip(tr("Create a new working area"));
    connect(m_newAction, SIGNAL(activated()), this, SLOT(newFile()));

    m_openAction = new QAction(tr("&Open..."), tr("Ctrl+O"), this);
    m_openAction->setIconSet(QPixmap::fromMimeSource("open.png"));
    m_openAction->setStatusTip(tr("Open an existing volume file"));
    connect(m_openAction, SIGNAL(activated()), this, SLOT(open()));

    //Biru: Aquesta opci ens permet obrir un directori de fitxers DICOM
    m_openDirectoryAction = new QAction(tr("&Open Directory..."), tr("Ctrl+D"), this);
    m_openDirectoryAction->setIconSet(QPixmap::fromMimeSource("open.png"));
    m_openDirectoryAction->setStatusTip(tr("Open a directory of Dicom files"));
    connect(m_openDirectoryAction, SIGNAL(activated()), this, SLOT(openDirectory()));
    
    m_showToolBoxAction = new QAction(tr("&Show ToolBox"), 0, this);
    m_showToolBoxAction->setIconSet(QPixmap::fromMimeSource("view_left_right.png"));
    m_showToolBoxAction->setToggleAction(true);
    m_showToolBoxAction->setOn(true);
    m_showToolBoxAction->setStatusTip(tr("Show or hide the toolbox"));
    connect(m_showToolBoxAction, SIGNAL(toggled(bool)), this, SLOT(setShowToolBox(bool)));

    m_aboutAction = new QAction(tr("&About"), 0, this);
    m_aboutAction->setStatusTip(tr("Show the application's About box"));
    connect(m_aboutAction, SIGNAL(activated()), this, SLOT(about()));

    m_aboutQtAction = new QAction(tr("About &Qt"), 0, this);
    m_aboutQtAction->setStatusTip(tr("Show the Qt library's About box"));
    connect(m_aboutQtAction, SIGNAL(activated()), qApp, SLOT(aboutQt()));

    m_tileAction = new QAction(tr("&Tile"), tr("F10"), this);
    m_tileAction->setIconSet(QPixmap::fromMimeSource("view_top_bottom.png"));
    m_tileAction->setStatusTip(tr("Arranges all windows in a tile pattern"));
    connect(m_tileAction, SIGNAL(activated()), m_workingArea, SLOT(tile()));
    
    m_cascadeAction = new QAction(tr("&Cascade"), 0, this);    
    m_cascadeAction->setStatusTip(tr("Arranges all the windows in a cascade pattern"));
    connect(m_cascadeAction, SIGNAL(activated()), m_workingArea, SLOT(cascade()));
    
    m_exitAction = new QAction(tr("E&xit"), tr("Ctrl+Q"), this);
    m_exitAction->setIconSet(QPixmap::fromMimeSource("exit.png"));    
    m_exitAction->setStatusTip(tr("Exit the application"));
    connect(m_exitAction, SIGNAL(activated()), qApp, SLOT(closeAllWindows()));

    m_closeAction = new QAction(tr("&Close"), tr("Ctrl+W"), this);
    m_closeAction->setIconSet(QPixmap::fromMimeSource("fileclose.png"));    
    m_closeAction->setStatusTip(tr("Close the current window"));
    connect(m_closeAction, SIGNAL(activated()), this, SLOT(close()));    
        
 
    
}

void QApplicationMainWindow::createMenus()
{
    m_fileMenu = new QPopupMenu(this);
    m_newAction->addTo(m_fileMenu);
    m_openAction->addTo(m_fileMenu);    
    m_openDirectoryAction->addTo(m_fileMenu); //Biru
    m_fileMenu->insertSeparator();
    
    m_recentFilesMenu = new QPopupMenu(this);
    m_fileMenu->insertItem("&Recent files", m_recentFilesMenu);    
    
    m_fileMenu->insertSeparator();
    m_closeAction->addTo(m_fileMenu);
    m_exitAction->addTo(m_fileMenu);

    for (int i = 0; i < MaxRecentFiles; ++i)
    {
        m_recentFileIds[i] = -1;
    }
  
    m_toolsMenu = new QPopupMenu(this);

    m_optionsMenu = new QPopupMenu(this);
    m_showToolBoxAction->addTo(m_optionsMenu);
    
    m_windowMenu = new QPopupMenu(this);
    m_cascadeAction->addTo(m_windowMenu);
    m_tileAction->addTo(m_windowMenu);

    m_helpMenu = new QPopupMenu(this);
    m_aboutAction->addTo(m_helpMenu);
    m_aboutQtAction->addTo(m_helpMenu);

    m_viewMenu = new QPopupMenu(this);

    menuBar()->insertItem(tr("&File"), m_fileMenu);
    menuBar()->insertItem(tr("&View"), m_viewMenu);
    menuBar()->insertItem(tr("&Tools"), m_toolsMenu);
    menuBar()->insertItem(tr("&Options"), m_optionsMenu);
    menuBar()->insertItem(tr("&Windows"), m_windowMenu);
    menuBar()->insertSeparator();
    menuBar()->insertItem(tr("&Help"), m_helpMenu);
}

void QApplicationMainWindow::createToolBars()
{
    m_fileToolBar = new QToolBar(tr("File"), this);
    m_newAction->addTo(m_fileToolBar);
    m_openAction->addTo(m_fileToolBar);
    //m_openDirectoryAction->addTo(m_fileToolBar); //Biru
    
    m_optionsToolBar = new QToolBar( tr("Options"), this );    
    m_showToolBoxAction->addTo( m_optionsToolBar );
    
    m_windowToolBar = new QToolBar( tr("Options"), this );    
    m_tileAction->addTo(m_windowToolBar);

}

void QApplicationMainWindow::createMainToolBox()
{
    m_toolBoxDockWindow = new QDockWindow( QDockWindow::InDock, this );
    m_toolBoxDockWindow->setResizeEnabled( TRUE );
    m_toolBoxDockWindow->setVerticalStretchable( TRUE );
    addDockWindow( m_toolBoxDockWindow, DockLeft );
    setDockEnabled( m_toolBoxDockWindow, DockTop, FALSE );
    setDockEnabled( m_toolBoxDockWindow, DockBottom, FALSE );
    m_toolBoxDockWindow->setCloseMode( QDockWindow::Never );
    
    // creem el nostre toolbox personal
    m_mainToolBox = new udg::QMainToolBox( m_toolBoxDockWindow, "m_toolBox" );
    m_toolBoxDockWindow->setWidget( m_mainToolBox );
    
    // ***************************************************************************
    // Inici zona de codi editable
    // ***************************************************************************
    
    // li afegim els widgets
    m_mainToolBox->addWidget( m_mutualInformationInputParametersForm, QMainToolBox::RegistrationTab, tr("Mutual Information" ) , m_mutualInformationAction );
    m_mainToolBox->addWidget( m_connectedThresholdInputParametersForm, QMainToolBox::SegmentationTab, tr("Connected Threshold" ) , m_connectedThresholdAction );    
    m_mainToolBox->addWidget( m_neighborhoodConnectedInputParametersForm, QMainToolBox::SegmentationTab, tr("Neighborhood Connected" ) , m_neighborhoodConnectedAction );    
    m_mainToolBox->addWidget( m_confidenceConnectedInputParametersForm, QMainToolBox::SegmentationTab, tr("Confidence Connected" ) , m_confidenceConnectedAction );    
    m_mainToolBox->addWidget( m_isolatedConnectedInputParametersForm, QMainToolBox::SegmentationTab, tr("Isolated Connected" ) , m_isolatedConnectedAction );    
    //m_mainToolBox->addWidget( m_watershedInputParametersForm, QMainToolBox::SegmentationTab, tr("Watershed" ) , m_watershedAction );    
    m_mainToolBox->addWidget( m_fastMarchingInputParametersForm, QMainToolBox::SegmentationTab, tr("Fast Marching" ) , m_fastMarchingAction );    
    m_mainToolBox->addWidget( m_shapeDetectionInputParametersForm, QMainToolBox::SegmentationTab, tr("Shape Detection" ) , m_shapeDetectionAction );    
    m_mainToolBox->addWidget( m_geodesicActiveInputParametersForm, QMainToolBox::SegmentationTab, tr("Geodesic Active" ) , m_geodesicActiveAction );    
    //m_mainToolBox->addWidget( m_thresholdLevelSetInputParametersForm, QMainToolBox::SegmentationTab, tr("Threshold Level Set" ) , m_thresholdLevelSetAction );    
    //m_mainToolBox->addWidget( m_cannyEdgeLevelSetInputParametersForm, QMainToolBox::SegmentationTab, tr("Canny Edge Level Set" ) , m_cannyEdgeLevelSetAction );    
    //m_mainToolBox->addWidget( m_laplacianLevelSetInputParametersForm, QMainToolBox::SegmentationTab, tr("Laplacian Level Set" ) , m_laplacianLevelSetAction );    
    
    m_mainToolBox->addWidget( m_rayCastingInputParametersForm, QMainToolBox::VisualizationTab, tr("Ray Casting" ) , m_rayCastingAction );    
    m_mainToolBox->addWidget( m_multiplaneInputParametersForm, QMainToolBox::VisualizationTab, tr("Multiplane" ) , m_multiplaneAction );    
    // ***************************************************************************
    // Fi zona de codi editable
    // ***************************************************************************
    
    
}

void QApplicationMainWindow::addWorkingAreaWidget( QWorkingAreaWidget *widget, const QString & title )
{
    udg::QWorkingAreaWindow* window = new udg::QWorkingAreaWindow( m_workingArea, title );
    
    // \TODO QApplicationMainWindow::addWorkingAreaWidget fer un reparent???    
    window->setWidget( widget );
    window->setCaption( title );
    window->show();
        
    m_workingArea->tile();
}

void QApplicationMainWindow::newFile()
{
    if( getCountQApplicationMainWindow() == 2 )
    {
        QMessageBox::information( this, tr("Alert"), tr("Cannot open more windows. The maximum number of windows that can be opened at once is two.") );
    }
    else
    {
        QApplicationMainWindow *newMainWindow = new QApplicationMainWindow;
        newMainWindow->show();
    }
}

void QApplicationMainWindow::open()
{
    if( getCountQApplicationMainWindow() == 2 )
    {
        QMessageBox::information( this, tr("Alert"), tr("Cannot open more windows. The maximum number of windows that can be opened at once is two.") );
    }
    else
    {
        if (maybeSave()) {
            QString fileName =
                    QFileDialog::getOpenFileName(m_workingDirectory, m_fileFilters, this, "open file dialog",tr("Chose an image filename") );
            if (!fileName.isEmpty())
            {
                loadFile(fileName);
                m_workingDirectory = QFileInfo(fileName).dirPath();
            }
        }
    }
}

//Biru: Slot per obrir el directori de fitxers DICOM
void QApplicationMainWindow::openDirectory()
{
    if( getCountQApplicationMainWindow() == 2 )
    {
        QMessageBox::information( this, tr("Alert"), tr("Cannot open more windows. The maximum number of windows that can be opened at once is two.") );
    }
    else
    {
        if (maybeSave()) {
            QString fileName =
                    QFileDialog::getOpenFileName(m_workingDirectory, m_fileFilters2, this, "open file dialog",tr("Chose an image filename") );
            if (!fileName.isEmpty())
            {
                loadDirectory(fileName);
                m_workingDirectory = QFileInfo(fileName).dirPath();
            }
        }
    }
}


bool QApplicationMainWindow::loadFile(const QString &fileName)
{
    bool ok = true; 
    //Si ja tenim obert un model, obrim una finestra nova
    
    if ( ! m_volumeID.isNull() )
    {
        QApplicationMainWindow* newQApplicationMainWindow = new QApplicationMainWindow();
        newQApplicationMainWindow->show();
        newQApplicationMainWindow->loadFile( fileName );   
    }
    else
    {
        
        statusBar()->message(tr("Loading image %1").arg(strippedName(fileName).latin1()), 2000);
        // indiquem que ens obri el fitxer
        if( m_inputReader->openFile( fileName.latin1() ) ) 
        { 
            
            statusBar()->message(tr("File loaded"), 2000);
            setCurrentFile(fileName);
            // creem el volum
          
            udg::Volume *dummyVolume = m_inputReader->getData();
            // afegim el nou volum al repositori
            
            m_volumeID = m_volumeRepository->addVolume( dummyVolume );
            
            m_mainToolBox->updateImageData( m_volumeID );
            
            //*************************BIRU***********************//
            m_connectedThresholdInputParametersForm->setIdInputImage(m_volumeID);
            m_neighborhoodConnectedInputParametersForm->setIdInputImage(m_volumeID);
            m_confidenceConnectedInputParametersForm->setIdInputImage(m_volumeID);
            m_isolatedConnectedInputParametersForm->setIdInputImage(m_volumeID);
            //m_watershedInputParametersForm->setIdInputImage(m_volumeID);
            m_fastMarchingInputParametersForm->setIdInputImage(m_volumeID);
            m_shapeDetectionInputParametersForm->setIdInputImage(m_volumeID);
            m_geodesicActiveInputParametersForm->setIdInputImage(m_volumeID);
            //m_thresholdLevelSetInputParametersForm->setIdInputImage(m_volumeID);
            //m_cannyEdgeLevelSetInputParametersForm->setIdInputImage(m_volumeID);
            //m_laplacianLevelSetInputParametersForm->setIdInputImage(m_volumeID);
            m_multiplaneDirector->setVolume( m_volumeRepository->getVolume( m_volumeID ) );
            //*************************BIRU***********************//
            
            // Exemple de com carregar finestres en la workarea pel volumerepository
            // obrir una finestra vtk que visualitza el volum amb id = 0
            
            //this->createWindowsExampleWithVolumeRepository( m_volumeID );
            this->openMainVolumeWindows();
        }
        else
        {
            // no s'ha pogut obrir l'arxiu per algun motiu
            statusBar()->message(tr("Error loading %1").arg(strippedName(fileName).latin1()), 2000);
            ok = false;
        }
    } 
    return ok;  
}


bool QApplicationMainWindow::loadDirectory(const QString &fileName)
{
    bool ok = true; 
    //Si ja tenim obert un model, obrim una finestra nova
    
    if ( ! m_volumeID.isNull() )
    {
        QApplicationMainWindow* newQApplicationMainWindow = new QApplicationMainWindow();
        newQApplicationMainWindow->show();
        newQApplicationMainWindow->loadDirectory( fileName );   
    }
    else
    {
        
        statusBar()->message(tr("Loading image %1").arg(strippedName(fileName).latin1()), 2000);
        // indiquem que ens obri el fitxer        
        if( m_inputReader->readSeries((QFileInfo(fileName).dirPath()).latin1() ) )
        { 
            statusBar()->message(tr("File loaded"), 2000);
            setCurrentFile(fileName);
            // creem el volum

            udg::Volume *dummyVolume = m_inputReader->getData();
            // afegim el nou volum al repositori

            m_volumeID = m_volumeRepository->addVolume( dummyVolume );

            m_mainToolBox->updateImageData( m_volumeID );

            //*************************BIRU***********************//
            m_connectedThresholdInputParametersForm->setIdInputImage(m_volumeID);
            m_neighborhoodConnectedInputParametersForm->setIdInputImage(m_volumeID);
            m_confidenceConnectedInputParametersForm->setIdInputImage(m_volumeID);
            m_isolatedConnectedInputParametersForm->setIdInputImage(m_volumeID);
            //m_watershedInputParametersForm->setIdInputImage(m_volumeID);
            m_fastMarchingInputParametersForm->setIdInputImage(m_volumeID);
            m_shapeDetectionInputParametersForm->setIdInputImage(m_volumeID);
            m_geodesicActiveInputParametersForm->setIdInputImage(m_volumeID);
            //m_thresholdLevelSetInputParametersForm->setIdInputImage(m_volumeID);
            //m_cannyEdgeLevelSetInputParametersForm->setIdInputImage(m_volumeID);
            //m_laplacianLevelSetInputParametersForm->setIdInputImage(m_volumeID);
            m_multiplaneDirector->setVolume( m_volumeRepository->getVolume( m_volumeID ) );
            //*************************BIRU***********************//

            // Exemple de com carregar finestres en la workarea pel volumerepository
            // obrir una finestra vtk que visualitza el volum amb id = 0
            
            //this->createWindowsExampleWithVolumeRepository( m_volumeID );
            this->openMainVolumeWindows();
        }
        else
        {
            // no s'ha pogut obrir l'arxiu per algun motiu
            statusBar()->message(tr("Error loading %1").arg(strippedName(fileName).latin1()), 2000);
            ok = false;
        }
    } 
    return ok;  
}





unsigned int QApplicationMainWindow::getCountQApplicationMainWindow()
{
    QWidgetList *list = QApplication::topLevelWidgets();
    QWidgetListIt it(*list);
    QWidget *widget;
    unsigned int count = 0;
    
    QString si("true");
    QString no("false");
    QString h;
    
    while ( (widget = it.current()) != 0 )
    {
        if ( widget->isA("udg::QApplicationMainWindow") )
        {
            count++;
        }
        ++it;
    }
    delete list;
    
    return count;
}

void QApplicationMainWindow::closeEvent(QCloseEvent *event)
{
    if (maybeSave()) {
        writeSettings();
        event->accept();
    } else {
        event->ignore();
    }
}

bool QApplicationMainWindow::maybeSave()
{
    if (m_modified) {
/*        int ret = QMessageBox::warning(this, tr("VisualPork"),
                     tr("The document has been m_modified.\n"
                        "Do you want to save your changes?"),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return save();
        else if (ret == QMessageBox::Cancel)
            return false;
*/
    }
    return true;
}

void QApplicationMainWindow::setCurrentFile(const QString &fileName)
{
    m_currentFile = fileName;
    m_modified = false;

    if (m_currentFile.isEmpty()) {
        setCaption(tr("VisualPork"));
    } else {
        setCaption(tr("%1 - %2").arg(strippedName(m_currentFile))
                                .arg(tr("VisualPork")));
        m_recentFiles.remove(m_currentFile);
        m_recentFiles.push_front(m_currentFile);
        updateRecentFileItems();
    }
}

QString QApplicationMainWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}

/*
    Obre una finestra d'un volum del repositori
    ::encara esta = que l'altre

void QApplicationMainWindow::createWindowsExampleWithVolumeRepository( Identifier id )
{
    
    //Exemple de com s'han de crear les finestres en la working area des dels directors

    QSimpleSliceViewer* simpleSliceViewer = new QSimpleSliceViewer();
    simpleSliceViewer->setVolume( m_volumeRepository->getVolume(id) );
    simpleSliceViewer->setView( QSimpleSliceViewer::Axial );
    this->addWorkingAreaWidget( simpleSliceViewer, tr("2D Visualization"));
        
    
    QWidgetVolume* imagePlanes = new QWidgetVolume();
    imagePlanes->setVolume( m_volumeRepository->getVolume(id) );
    this->addWorkingAreaWidget( imagePlanes, tr("3D Image Planes"));
    
    QSimple3DViewer* simple3DView = new QSimple3DViewer();
    simple3DView->setVolume( m_volumeRepository->getVolume(id) );
    this->addWorkingAreaWidget( simple3DView, tr("Simple 3D View"));
    
}
*/

void QApplicationMainWindow::openMainVolumeWindows( )
{
    
    //Exemple de com s'han de crear les finestres en la working area des dels directors

    if( m_volumeID.isNull() )
    {
        // no s'obrir res perqu no hi ha cap volum
        // aix voldria dir que s'ha cridat aquest mtode en un moment inadequat cosa que
        // no hauria de passar
    }
    else
    {
        // Aqu es s'obren les finestres que desitjem que es vegin a l'obrir el model
        
        // *******************************************************************************
        // Inici de zona de codi editable
        // *******************************************************************************
        
        // en aquest exemple s'obre un visor simple 2D ( QSimpleSliceViewer )
        // un visor de plans en 3D ( QWidgetVolume )
        // i un render de volum 3D simple amb ray casting ( QSimple3DViewer )
        
        /*QSimpleSliceViewer* simpleSliceViewer = new QSimpleSliceViewer();
        simpleSliceViewer->setVolume( m_volumeRepository->getVolume( m_volumeID ) );
        simpleSliceViewer->setView( QSimpleSliceViewer::Axial );
        this->addWorkingAreaWidget( simpleSliceViewer, tr("2D Visualization"));
        */    
        
        /*QWidgetVolume* imagePlanes = new QWidgetVolume();
        imagePlanes->setVolume( m_volumeRepository->getVolume( m_volumeID ) );
        this->addWorkingAreaWidget( imagePlanes, tr("3D Image Planes"));
        */
        /*QSimple3DViewer* simple3DView = new QSimple3DViewer();
        simple3DView->rayCasting(m_volumeRepository->getVolume( m_volumeID ));
        this->addWorkingAreaWidget( simple3DView, tr("3D View"));*/

        /*connect (imagePlanes, SIGNAL( coordenatesChangedSignal(double,double,double) ),
                    m_connectedThresholdInputParametersForm, SLOT( coordenatesChangedSlot(double,double,double) ) );
        */

        // *******************************************************************************
        // Fi de zona de codi editable
        // *******************************************************************************
    }
}

void QApplicationMainWindow::updateRecentFileItems()
{
    while ((int)m_recentFiles.size() > MaxRecentFiles)
        m_recentFiles.pop_back();

    for (int i = 0; i < (int)m_recentFiles.size(); ++i) 
    {
        QString text = tr("&%1 %2").arg(i + 1).arg(m_recentFiles[i]);
        if (m_recentFileIds[i] == -1) 
        {
            m_recentFileIds[i] = m_recentFilesMenu->insertItem(text, this, SLOT(openRecentFile(int)), 0, -1, m_recentFilesMenu->count());
            m_recentFilesMenu->setItemParameter(m_recentFileIds[i], i);
        }
        else 
        {
            m_recentFilesMenu->changeItem(m_recentFileIds[i], text);
        }
    }
    
}

void QApplicationMainWindow::openRecentFile(int param)
{
    if (maybeSave())
        loadFile(m_recentFiles[param]);
}

void QApplicationMainWindow::setShowToolBox(bool visible)
{
    if (visible) m_toolBoxDockWindow->show();
    else m_toolBoxDockWindow->hide();
}

void QApplicationMainWindow::createStatusBar()
{
    QLabel *locationLabel;
    QLabel *formulaLabel;
    QLabel *modLabel;
    
    locationLabel = new QLabel(" W999 ", this);
    locationLabel->setAlignment(AlignHCenter);
    locationLabel->setMinimumSize(locationLabel->sizeHint());

    formulaLabel = new QLabel(this);

    modLabel = new QLabel(tr(" MOD "), this);
    modLabel->setAlignment(AlignHCenter);
    modLabel->setMinimumSize(modLabel->sizeHint());
    modLabel->clear();

    statusBar()->addWidget(locationLabel);
    statusBar()->addWidget(formulaLabel, 1);
    statusBar()->addWidget(modLabel);
}

void QApplicationMainWindow::about()
{
    QMessageBox::about(this, tr("About VisualPork"),
            tr("<h2>VisualPork 1.0</h2>"
               "<p>Copyright &copy; 2004 Universitat de Girona"
               "<p>VisualPork is a small application that "
               "lets you view <b>DICOM</b>, <b>MHD's</b>,... "
               "files and manipulate them."));
}

void QApplicationMainWindow::writeSettings()
{
    QSettings settings;
    settings.setPath("software-inc.com", "VisualPork");
    settings.beginGroup("/VisualPork");
    settings.writeEntry("/geometry/x", x());
    settings.writeEntry("/geometry/y", y());
    settings.writeEntry("/geometry/width", width());
    settings.writeEntry("/geometry/height", height());
    settings.writeEntry("/recentFiles", m_recentFiles);
    settings.writeEntry("/showToolBox", m_showToolBoxAction->isOn());
    settings.writeEntry("/workingDirectory", m_workingDirectory);
    settings.endGroup();
}

void QApplicationMainWindow::readSettings()
{
    QSettings settings;
    settings.setPath("software-inc.com", "VisualPork");
    settings.beginGroup("/VisualPork");

    int x = settings.readNumEntry("/geometry/x", 200);
    int y = settings.readNumEntry("/geometry/y", 200);
    int w = settings.readNumEntry("/geometry/width", 400);
    int h = settings.readNumEntry("/geometry/height", 400);
    move(x, y);
    resize(w, h);

    m_recentFiles = settings.readListEntry("/recentFiles");
    updateRecentFileItems();

    m_showToolBoxAction->setOn(
            settings.readBoolEntry("/showToolBox", true));

    m_workingDirectory = settings.readEntry("/workingDirectory", ".");
    
    settings.endGroup();
}

}; // end namespace udg
