/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
 
#ifndef MAINWINDOW_H
#define MAINWINDOW_H


class QAction;
class QDockWindow;

#include <qmainwindow.h>
#include <qstringlist.h>

#include "identifier.h"

namespace udg{

// Forward declarations

class VolumeRepository;
class Input;
class QMainToolBox;
class QWorkingAreaWidget;
class QWorkingArea;

// ***************************************************************************
// Inici zona de codi editable [Forward Declarations]
// ***************************************************************************

// Registre: Informaci Mtua
class MutualInformationDirector;
class MutualInformationParameters;
class MutualInformationInputParametersForm;

// Segmentacio: Connected Threshold
class ConnectedThresholdDirector;
class ConnectedThresholdParameters;
class ConnectedThresholdInputParametersForm;

// Segmentacio: Neighborhood Connected
class NeighborhoodConnectedDirector;
class NeighborhoodConnectedParameters;
class NeighborhoodConnectedInputParametersForm;

// Segmentacio: Confidence Connected
class ConfidenceConnectedDirector;
class ConfidenceConnectedParameters;
class ConfidenceConnectedInputParametersForm;

// Segmentacio: Isolated Connected
class IsolatedConnectedDirector;
class IsolatedConnectedParameters;
class IsolatedConnectedInputParametersForm;

/*// Segmentacio: Watershed
class WatershedDirector;
class WatershedParameters;
class WatershedInputParametersForm;
*/
// Segmentacio: Fast Marching
class FastMarchingDirector;
class FastMarchingParameters;
class FastMarchingInputParametersForm;

// Segmentacio: Shape Detection
class ShapeDetectionDirector;
class ShapeDetectionParameters;
class ShapeDetectionInputParametersForm;

// Segmentacio: Geodesic Active
class GeodesicActiveDirector;
class GeodesicActiveParameters;
class GeodesicActiveInputParametersForm;

/*// Segmentacio: Threshold Level Set
class ThresholdLevelSetDirector;
class ThresholdLevelSetParameters;
class ThresholdLevelSetInputParametersForm;
*/
/*// Segmentacio: Canny Edge Level Set
class CannyEdgeLevelSetDirector;
class CannyEdgeLevelSetParameters;
class CannyEdgeLevelSetInputParametersForm;

// Segmentacio: Laplacian Level Set
class LaplacianLevelSetDirector;
class LaplacianLevelSetParameters;
class LaplacianLevelSetInputParametersForm;
*/

// Visualitzaci: Ray Casting
class RayCastingDirector;
class RayCastingParameters;
class RayCastingInputParametersForm;

// Visualitzaci: Multiplane
class MultiplaneDirector;
class MultiplaneInputParametersForm;

// ***************************************************************************
// Fi zona de codi editable
// ***************************************************************************

class QApplicationMainWindow : public QMainWindow
{

    Q_OBJECT
public:
    ///El constructor del Mainwindow. Aqu es crea tot l'esquelet de l'interfcie: Mens, Barres d'eines, toolbox, working area, etc..
    QApplicationMainWindow(QWidget *parent = 0, const char *name = 0);
    
    ~QApplicationMainWindow();
    
    /// Ens retorna l'id del volum que ha obert
    Identifier getVolumeID() const { return m_volumeID; };
public slots:   
    
    ///Fa una petici per obrir una finestra en la working area i es mostri el widget
    void addWorkingAreaWidget( QWorkingAreaWidget *widget, const QString & title );


protected:
    void closeEvent(QCloseEvent *event);


private slots:
    void newFile();
    
    /// Slot obrir fitxer
    void open();
    void openDirectory(); //Biru
    
    void about();
    void openRecentFile(int param);
    void setShowToolBox(bool);
    
    
private:
    
    /**
        Crea les accions.        
        Les accions engloben l'accs a una certa funcionalitat de l'aplicaci per
        part de diversos elements de la interfcie. Per exemple "Obrir una imatge"
        es pot fer des d'una icona en la barra d'eines, un men o un acccs rpid
        amb le teclat Ctrl+O.
    */
    void createActions();
    /// Crea els mens.
    void createMenus();
    /// Crea la barra d'eines.
    void createToolBars();
    /// Crea el toolbox principal de l'aplicaci. T codi editable.
    void createMainToolBox();
    void createStatusBar();
    void readSettings();
    void writeSettings();
    bool maybeSave();
    /// \TODO \deprecated Refactoring d'aquest mtode. Aquest mtode s el que fa la verdadera feina d'obrir un fitxer
    bool loadFile(const QString &fileName);
    bool loadDirectory(const QString &fileName);//Biru

    ///Retorna el numero de finestres amb diferents models que tenim obertes, el num. de QApplicationMainWindow
    unsigned int getCountQApplicationMainWindow();
    
    void setCurrentFile(const QString &fileName);
    void updateRecentFileItems();
    QString strippedName(const QString &fullFileName);
    
    //void createWindowsExampleWithVolumeRepository(Identifier id = 0);

    /** 
        S'ocupa d'obrir la/es finestra/es bsica/ques on es visalitzi el volum obert.
        Aquest mtode s editable i es pot adaptar per mostrar les finestres que siguin necessries.
    */
    void openMainVolumeWindows();    
    
    /// Indicar l'id del volum que possem
    udg::Identifier m_volumeID; 
    
    bool m_modified;
    
    QStringList m_recentFiles;
    QString m_currentFile;
    QString m_fileFilters,m_fileFilters2;
    QString m_workingDirectory;
    
    
    QDockWindow* m_toolBoxDockWindow;
    
    
    enum { MaxRecentFiles = 5 };
    int m_recentFileIds[MaxRecentFiles];
    
    // Mens
    QPopupMenu *m_fileMenu;
    QPopupMenu *m_toolsMenu;
    QPopupMenu *m_optionsMenu;
    QPopupMenu *m_viewMenu;
    QPopupMenu *m_helpMenu;
    QPopupMenu *m_windowMenu;
    // Sub mens
    QPopupMenu *m_recentFilesMenu;
    
    // Toolbars
    QToolBar *m_fileToolBar;
    QToolBar *m_optionsToolBar;
    QToolBar *m_windowToolBar;
    
    // Actions
    QAction *m_newAction;
    QAction *m_openAction;
    QAction *m_openDirectoryAction; //Biru
    QAction *m_closeAction;
    QAction *m_exitAction;
    QAction *m_showToolBoxAction;
    QAction *m_aboutAction;
    QAction *m_aboutQtAction;
    QAction *m_tileAction;
    QAction *m_cascadeAction;
    
    QWorkingArea* m_workingArea;
    
            
    udg::VolumeRepository* m_volumeRepository;
    udg::Input* m_inputReader;
    udg::QMainToolBox* m_mainToolBox;
    
// ***************************************************************************
// Inici zona de codi editable
// ***************************************************************************
    
    // A partir d'aqu afegim els nous membres de classse
    MutualInformationDirector* m_mutualInformationDirector;
    MutualInformationParameters* m_mutualInformationParameters;
    MutualInformationInputParametersForm* m_mutualInformationInputParametersForm;
    QAction* m_mutualInformationAction;

    ConnectedThresholdDirector* m_connectedThresholdDirector;
    ConnectedThresholdParameters* m_connectedThresholdParameters;
    ConnectedThresholdInputParametersForm* m_connectedThresholdInputParametersForm;
    QAction* m_connectedThresholdAction;

    NeighborhoodConnectedDirector* m_neighborhoodConnectedDirector;
    NeighborhoodConnectedParameters* m_neighborhoodConnectedParameters;
    NeighborhoodConnectedInputParametersForm* m_neighborhoodConnectedInputParametersForm;
    QAction* m_neighborhoodConnectedAction;

    ConfidenceConnectedDirector* m_confidenceConnectedDirector;
    ConfidenceConnectedParameters* m_confidenceConnectedParameters;
    ConfidenceConnectedInputParametersForm* m_confidenceConnectedInputParametersForm;
    QAction* m_confidenceConnectedAction;

    IsolatedConnectedDirector* m_isolatedConnectedDirector;
    IsolatedConnectedParameters* m_isolatedConnectedParameters;
    IsolatedConnectedInputParametersForm* m_isolatedConnectedInputParametersForm;
    QAction* m_isolatedConnectedAction;

    /*WatershedDirector* m_watershedDirector;
    WatershedParameters* m_watershedParameters;
    WatershedInputParametersForm* m_watershedInputParametersForm;
    QAction* m_watershedAction;
    */
    FastMarchingDirector* m_fastMarchingDirector;
    FastMarchingParameters* m_fastMarchingParameters;
    FastMarchingInputParametersForm* m_fastMarchingInputParametersForm;
    QAction* m_fastMarchingAction;

    ShapeDetectionDirector* m_shapeDetectionDirector;
    ShapeDetectionParameters* m_shapeDetectionParameters;
    ShapeDetectionInputParametersForm* m_shapeDetectionInputParametersForm;
    QAction* m_shapeDetectionAction;

    GeodesicActiveDirector* m_geodesicActiveDirector;
    GeodesicActiveParameters* m_geodesicActiveParameters;
    GeodesicActiveInputParametersForm* m_geodesicActiveInputParametersForm;
    QAction* m_geodesicActiveAction;

    /*ThresholdLevelSetDirector* m_thresholdLevelSetDirector;
    ThresholdLevelSetParameters* m_thresholdLevelSetParameters;
    ThresholdLevelSetInputParametersForm* m_thresholdLevelSetInputParametersForm;
    QAction* m_thresholdLevelSetAction;*/
/*
    CannyEdgeLevelSetDirector* m_cannyEdgeLevelSetDirector;
    CannyEdgeLevelSetParameters* m_cannyEdgeLevelSetParameters;
    CannyEdgeLevelSetInputParametersForm* m_cannyEdgeLevelSetInputParametersForm;
    QAction* m_cannyEdgeLevelSetAction;

    LaplacianLevelSetDirector* m_laplacianLevelSetDirector;
    LaplacianLevelSetParameters* m_laplacianLevelSetParameters;
    LaplacianLevelSetInputParametersForm* m_laplacianLevelSetInputParametersForm;
    QAction* m_laplacianLevelSetAction;
    */

    RayCastingDirector* m_rayCastingDirector;
    RayCastingParameters* m_rayCastingParameters;
    RayCastingInputParametersForm* m_rayCastingInputParametersForm;
    QAction* m_rayCastingAction;

    MultiplaneDirector* m_multiplaneDirector;
    MultiplaneInputParametersForm* m_multiplaneInputParametersForm;
    QAction* m_multiplaneAction;
// ***************************************************************************
// Fi zona de codi editable
// ***************************************************************************

};

}; // fi namespace udg

#endif
