/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/


#ifndef UDGQINPUTPARAMETERS_H
#define UDGQINPUTPARAMETERS_H

#include <qwidget.h>

namespace udg {

/**
    Classe base per als widgets destinats a introdur parmetres.
    Els parmetres es poden introdur ja de forma simple i clssica com pot ser amb 
    caixes de text i similar o de formes ms complexes com amb histogrames o d'altres.
    
    Aquesta ofereix la interfcie comuna i bsica de signals o slots perqu les dades
    es mantinguin en sincronisme amb la resta de la interfcie.

@author Grup de Grfics de Girona  ( GGG )
*/

class QInputParameters : public QWidget{
Q_OBJECT
public:
    QInputParameters(QWidget *parent = 0, const char *name = 0);

    virtual ~QInputParameters();

    /**
        Activa/desactiva l'emissi de la senyal "parameterChanged(int)"
    */
    void disableIndividualSincronization();
    void enableIndividualSincronization();
    bool isIndividualSincronizationEnabled() const { return m_individualSincronization; };
    
public slots:
    /**
        Slot que ens serveix per indicar que hem d'actualitzar el parametre que ens diguin mitjanant 
        un identificador (que, en realitat, ser un enum). Serveix per canviar els valors a partir 
        d'una classe Parameters
    */
    virtual void readParameter(int index) = 0; 
    /**
        Escriu tots els valors de parmetres que t actualment al Parameters associat
    */
    virtual void writeAllParameters() = 0;

    virtual bool correctParameters();

private:
    /// indicar si els parmetres s'actualitzen 'en viu' o d'una tacada
    bool m_individualSincronization;
};

};  

#endif
