/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#include "qmaintoolbox.h"

#include "qmaintoolboxpage.h"

#include <qtoolbox.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qwidgetstack.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qaction.h>

#include <iostream.h>

#include "volumerepository.h"
#include "volume.h"

// widgets propis
#include "qaxisviewer.h"

namespace udg {

QMainToolBox::QMainToolBox(QWidget *parent, const char *name)
  : QWidget(parent, name)
{
    if( !name )
        name = "Main Tool Box";

    QVBoxLayout* toolBoxLayout = new QVBoxLayout( this, 0, 0, "Form1Layout");

    m_toolBox = new QToolBox( this, name );
    m_toolBox->setCurrentIndex( 1 );
    toolBoxLayout->addWidget( m_toolBox );    
    
    // ara construim l'esquelet del toolbox
    QWidget* axisViewpage = new QWidget( m_toolBox, "axisViewpage" );
    axisViewpage->setBackgroundMode( QWidget::PaletteBackground );

    QVBoxLayout* axisViewpageLayout = new QVBoxLayout( axisViewpage, 11, 6, "axisViewpageLayout"); 
    m_axisViewer = new QAxisViewer( axisViewpage, "axisViewer" );
    axisViewpageLayout->addWidget( m_axisViewer );
    
    m_toolBox->insertItem( AxisViewTab, axisViewpage, "Axis View" );
    
    
    // Pgina d'algorismes de registre
    QMainToolBoxPage* registrationPage = new QMainToolBoxPage( m_toolBox, "registrationPage" );
    registrationPage->setBackgroundMode( QWidget::PaletteBackground );
    

    m_toolBox->insertItem( RegistrationTab , registrationPage, "Registration" );

    
    // Pgina d'algorismes de segmentaci
    QMainToolBoxPage* segmentationPage = new QMainToolBoxPage( m_toolBox, "segmentationPage" );
    segmentationPage->setBackgroundMode( QWidget::PaletteBackground );
    
    m_toolBox->insertItem( SegmentationTab , segmentationPage, "Segmentation" );
    
    QMainToolBoxPage *visualizationPage = new QMainToolBoxPage( m_toolBox, "visualizationPage" );
    visualizationPage->setBackgroundMode( QWidget::PaletteBackground );
    
    m_toolBox->insertItem( VisualizationTab , visualizationPage, "Visualization" );
    //Escribim el texte de tot arreu
    
    m_volumeRepository = VolumeRepository::getRepository();

}


QMainToolBox::~QMainToolBox()
{
}

void QMainToolBox::addWidget( QWidget* widget, TabIndex index, QString name, QAction* trigger )
{
    QMainToolBoxPage* item;

    item = dynamic_cast< QMainToolBoxPage* >( m_toolBox->item( index ) );
    if( item )
    {
        item->addPage( widget , name, trigger );
    }

    /*
    switch( index )
    {
        case RegistrationTab:
        break;
        case SegmentationTab:
        break;
        case VisualizationTab:
        break;
        default:
        break;
    }*/

}

void QMainToolBox::updateImageData( Identifier id )
{
    m_axisViewer->setInputImage( m_volumeRepository->getVolume( id ) );        
}

}; // end namespace udg
