/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#include "qseedsliceviewer.h"

//includes qt
#include <qlayout.h>

//includes vtk
#include <vtkImageViewer2.h>
#include <QVTKWidget.h>

#include <vtkImagePermute.h> // per poder permutar els eixos
#include <vtkRenderWindow.h>
#include <vtkRenderer.h>
#include <vtkCamera.h>

//includes propis
#include "volume.h"

//Utilitzat per captar els clicks de l'usuari (llavors)
#include <vtkEventQtSlotConnect.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkPropPicker.h>
#include <vtkCell.h>
#include <vtkPointData.h>

//Per dibuixar les llavors
#include <vtkSphereSource.h>
#include <vtkPolyDataMapper.h>
#include <vtkActor.h>
#include <vtkProperty.h>


namespace udg {

QSeedSliceViewer::QSeedSliceViewer(QWidget *parent, const char *name)
 : QWorkingAreaWidget(parent, name)
{
    if ( !name )
        this->setName( "QSeedSliceViewer" );
    
    m_form1Layout = new QVBoxLayout( this, 0, 0, "Form1Layout"); 

    m_vtkWidget = new QVTKWidget( this, "vTKWidget1" );
    m_form1Layout->addWidget( m_vtkWidget );
      
    m_lastView = Axial; // per defecte la vista es Axial
    
    m_volume = 0;

    //Utilitzat per captar els clicks de l'usuari (llavors)
    m_picker=vtkPropPicker::New();
    m_interactor=vtkRenderWindowInteractor::New();

    //Utilitzat per dibuixar les llavors
    m_numSeeds=0;
}


QSeedSliceViewer::~QSeedSliceViewer()
{
//    m_imageViewer->Delete();
}

void QSeedSliceViewer::setVolume( Volume* inputVolume )
{   
    m_volume = inputVolume;
    
    m_imageViewer = vtkImageViewer2::New();
    m_imageViewer->SetInput( m_volume->getVtkData() );
    
    m_vtkWidget->SetRenderWindow( m_imageViewer->GetRenderWindow() );
    //m_imageViewer->SetupInteractor( m_vtkWidget->GetRenderWindow()->GetInteractor() );
    
    // L'hi donem el window/level correcte
    // aix ens dna un level/level "maco" per defecte
    double * range = m_volume->getVtkData()->GetScalarRange();
    double window = fabs(range[1] - range[0]);
    double level = (range[1] + range[0] )/ 2.0;
    
    //Amb els models de porc hi ha problemes i el window i level calculats sn molt superiors als
    //que haurien de ser. Per aquest motiu les imatges no es veuen correctament. Per solucionar-ho
    //assignarem per defecte un level de 1137 i window de 2275. 
    if((window>3500)&&(level>1900))
    {
        level=1137;
        window=2275;

        //cout << "Window: " << window << endl;
        //cout << "Level: " << level << endl;
        m_imageViewer->SetColorLevel( level );
        m_imageViewer->SetColorWindow( window );
    }

    else
    {
        //cout << "Window: " << window << endl;
        //cout << "Level: " << level << endl;
        m_imageViewer->SetColorLevel( level );
        m_imageViewer->SetColorWindow( window );
    }

    //Utilitzat per captar els clicks de l'usuari (llavors)    
    m_connections = vtkEventQtSlotConnect::New();
    m_connections->Connect(m_vtkWidget->GetRenderWindow()->GetInteractor(),vtkCommand::LeftButtonPressEvent,this,SLOT(addSeed()));    

}


void QSeedSliceViewer::setView( ViewType view )
{    
    vtkImageData* vtkImage = m_volume->getVtkData();
    vtkImagePermute* permuter = vtkImagePermute::New();
    
    int *size = m_volume->getVtkData()->GetDimensions(); //Biru

    permuter->SetInput( vtkImage );
    switch( view )
    {
    case Axial:
        m_imageViewer->SetSliceOrientationToXY();  
        //permuter->SetFilteredAxes( 0, 1 , 2 );
        m_imageViewer->SetSlice( size[0]/2 );
        m_slice=size[0]/2;
        m_view=0;
    /*
        if( m_lastView == Coronal )
            permuter->SetFilteredAxes( 0, 2 , 1 );
        else if( m_lastView == Sagittal )
            permuter->SetFilteredAxes( 1, 2 , 0);
    */
        m_lastView = Axial;
    break;
    case Sagittal:
        m_imageViewer->SetSliceOrientationToYZ();  
        //permuter->SetFilteredAxes( 1, 2 , 0);
        m_imageViewer->SetSlice( size[1]/2 );
        m_slice=size[1]/2;
        m_view=1;
    /*
        if( m_lastView == Axial )
            permuter->SetFilteredAxes( 1, 2 , 0);
        else if( m_lastView == Coronal )
            permuter->SetFilteredAxes( 1, 2 , 0);
    */
        m_lastView = Sagittal;
    break;
    case Coronal:
        m_imageViewer->SetSliceOrientationToXZ();  
        //permuter->SetFilteredAxes( 0, 2 , 1 );
        m_imageViewer->SetSlice( size[2]/2 );
        m_slice=size[2]/2;
        m_view=2;
    /*
        if( m_lastView == Axial )
            permuter->SetFilteredAxes( 0, 2 , 1 );
        else if( m_lastView == Sagittal )
            permuter->SetFilteredAxes( 1, 2 , 0);
    */
        m_lastView = Coronal;
    
    break;
    }

    //POSEM B LA CMERA
        // Aix s necessari perqu la imatge es rescali a les mides de la finestreta
        m_imageViewer->GetRenderer()->ResetCamera();
        //Moc la cmera 180 graus de manera que els models de porc es vegin correctament
        vtkCamera *camera=vtkCamera::New();
        camera=m_imageViewer->GetRenderer()->GetActiveCamera();
        camera->SetRoll(0);
        camera->Roll(180);
        m_imageViewer->GetRenderer()->SetActiveCamera(camera);
    //FI POSEM B LA CMERA

    //m_imageViewer->SetInput( permuter->GetOutput() );
}

void QSeedSliceViewer::ChangeSlice( int value )
{
    int *size = m_volume->getVtkData()->GetDimensions();
    
    switch( m_lastView )
    {
        case Axial:
            if( value <= size[0] && value >= 0 )
            {
                m_imageViewer->SetSlice( value );m_slice=value;
                m_imageViewer->Render();
            }
        break;
        
        case Sagittal:
            if( value <= size[1] && value >= 0 )
            {   
                m_imageViewer->SetSlice( value );m_slice=value;
                m_imageViewer->Render();
            }
        break;
        
        case Coronal:
            if( value <= size[2] && value >= 0 )
            {   
                m_imageViewer->SetSlice( value );m_slice=value;
                m_imageViewer->Render();
            }
        break;
    }
}


void QSeedSliceViewer::addSeed()
{   
    m_interactor = m_vtkWidget->GetRenderWindow()->GetInteractor();
    
    // Agafem el punt que est apuntant el ratol en aquell moment. 
    // Podrem passar-li el 4t parmatre opcional (vtkPropCollection) per indicar que noms agafi 
    // de l'ImageActor, per no sembla que suigui necessari realment i que si fa pick d'un altre 
    // actor 2D no passa res.
    m_picker->PickProp( m_interactor->GetEventPosition()[0], m_interactor->GetEventPosition()[1], m_imageViewer->GetRenderer() );
    
    // Ara tenim el pixel al que ha clicat. Ara hem de calcular el pixel trobat, s a dir, traduir
    // aquest pixel de la pantalla al valor real (posici i valor escalar) de la llesca.
    double q[3], imageValue; //Coordenades i valor escalar
    m_picker->GetPickPosition( q );
    int found = 0;
    
    // Quan dona una posici de (0,0,0) s que estem fora de l'actor.
    if( !( q[0] == 0 && q[1] == 0 && q[2] == 0) )
    {
        double tolerance;
        int subCellId;
        double parametricCoordinates[3], interpolationWeights[8];
        
        vtkPointData *pointData = m_volume->getVtkData()->GetPointData(); // m_volume s una instncia de la classe Volume, s a dir les dades que visualitzem
        vtkPointData* outPointData = vtkPointData::New();
        outPointData->InterpolateAllocate( pointData , 1 , 1 );
    
        // Use tolerance as a function of size of source data
        tolerance = m_volume->getVtkData()->GetLength();
        tolerance = tolerance ? tolerance*tolerance / 1000.0 : 0.001;
    
        // Find the cell that contains q and get it
        vtkCell *cell = m_volume->getVtkData()->FindAndGetCell( q , NULL , -1 , tolerance , subCellId , parametricCoordinates , interpolationWeights );
        if ( cell )
        {
            // Interpolate the point data
            outPointData->InterpolatePoint( pointData , 0 , cell->PointIds , interpolationWeights );
            imageValue = outPointData->GetScalars()->GetTuple1(0);
            found = 1;
        }
        outPointData->Delete();
    }
    if( !found )
    {
        // No s'ha trobat cap punt, no estem a sobre de cap actor que interessi
    }
    
    else //Ok!! Tenim la coordenada ( q[0],q[1],q[2] ) i el valor del voxel (imageValue)
    {
        cout << "Coordenades: " << q[0] << " " << q[1] << " " << q[2] << endl;
        emit signalSeed(m_view,q[0],q[1],q[2],imageValue);
        emit signalSlice(m_slice);
        drawSeed(q[0],q[1],q[2],imageValue);
    }

    //m_vtkWidget->GetRenderWindow()->GetInteractor()->Render(); 
}


void QSeedSliceViewer::drawSeed(double x,double y,double z,double v)
{   
    //Dibuixem la llavor sobre l'imatge
    vtkSphereSource *sphere = vtkSphereSource::New();
    sphere->SetRadius(3);

    vtkPolyDataMapper *sphereMapper = vtkPolyDataMapper::New();
    sphereMapper->SetInputConnection(sphere->GetOutputPort());

    vtkActor *sphere1 = vtkActor::New();
    sphere1->SetMapper(sphereMapper);
    sphere1->SetPosition(x,y,z);
    sphere1->GetProperty()->SetColor(1,0,0);
    sphere1->VisibilityOn();

    m_vectorSeeds[m_numSeeds]=sphere1;
    m_numSeeds=m_numSeeds+1;
    
    m_imageViewer->GetRenderer()->AddActor(sphere1);
    m_imageViewer->Render();
    
    //m_vtkWidget->GetRenderWindow()->GetInteractor()->Render();
}


void QSeedSliceViewer::deleteSeed(int n)
{
    m_imageViewer->GetRenderer()->RemoveActor(m_vectorSeeds[n]);
    m_imageViewer->Render();    
    m_vtkWidget->GetRenderWindow()->GetInteractor()->Render(); 
    
    //Esborro la llavor del vector d'actors    
    for(int i=n;i<m_numSeeds-1;i++)
    {
        m_vectorSeeds[i]=m_vectorSeeds[i+1];
    }
    m_vectorSeeds[m_numSeeds-1]=NULL; 
    m_numSeeds=m_numSeeds-1;
}


void QSeedSliceViewer::deleteAllSeeds()
{
    for(int i=0;i<m_numSeeds;i++)
    {
        m_imageViewer->GetRenderer()->RemoveActor(m_vectorSeeds[i]);
        m_vectorSeeds[i]=NULL;
    }
    m_numSeeds=0;
    m_imageViewer->Render();    
    m_vtkWidget->GetRenderWindow()->GetInteractor()->Render(); 

}

void QSeedSliceViewer::selectSeed(int n,int nAnt)
{
    //Si n'hi havia anteriorment un de seleccionat el poso al color vermell
    if((nAnt!=-1)&&(nAnt<m_numSeeds))
    {
        vtkActor* actorAnt=m_vectorSeeds[nAnt];
        actorAnt->GetProperty()->SetColor(1,0,0);
        m_imageViewer->GetRenderer()->RemoveActor(m_vectorSeeds[nAnt]);
        m_vectorSeeds[nAnt]=actorAnt;
        m_imageViewer->GetRenderer()->AddActor(actorAnt);
    }

    //El que ara seleccionem el posem de color verd
    vtkActor* actor=m_vectorSeeds[n];
    actor->GetProperty()->SetColor(0,1,0);
    m_imageViewer->GetRenderer()->RemoveActor(m_vectorSeeds[n]);
    m_vectorSeeds[n]=actor;
    m_imageViewer->GetRenderer()->AddActor(actor);

    m_imageViewer->Render();    
    m_vtkWidget->GetRenderWindow()->GetInteractor()->Render(); 
}

void QSeedSliceViewer::IntensityMax( int value )
{
}

void QSeedSliceViewer::IntensityMin( int value )
{
}

void QSeedSliceViewer::ZoomIn()
{
}

void QSeedSliceViewer::ZoomOut()
{
}

};