/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/


#ifndef UDGQSEEDSLICEVIEWER_H
#define UDGQSEEDSLICEVIEWER_H

#include "qworkingareawidget.h"
#include <vtkActor.h>

namespace udg
{
    class Volume;
}
class QVBoxLayout;
class QVTKWidget;
class vtkImageViewer2;

//Utilitzats per captar els clicks de l'usuari (llavors)
class vtkEventQtSlotConnect;
class vtkRenderWindowInteractor;
class vtkPropPicker;


namespace udg {
 
/**

    Visor simple de llesques a travs del qual podem seleccionar llavors(seeds). 
    Fa servir la clase Volume com a input. Visualitza amb vtkImageViewer. 
    Podem escollir la vista Axial, Coronal o Sagital
    Hereda de QWorkingAreaWidget, com tota finestra que va en l'rea de treball de
    l'aplicaci.
    
@author Grup de Grfics de Girona (GGG)
*/
class QSeedSliceViewer : public QWorkingAreaWidget
{
Q_OBJECT
public:
    enum ViewType{ Axial, Coronal, Sagittal };
    QSeedSliceViewer(QWidget *parent = 0, const char *name = 0);

    ~QSeedSliceViewer();
    /// Li indiquem el volum a visualitzar
    void setVolume(Volume* inputVolume);
    /// Li indiquem la vista Axial, Coronal o Sagital
    void setView( ViewType view );

    void deleteSeed(int n);
    void deleteAllSeeds();
    void selectSeed(int n,int nAnt);
    

public slots:

    ///Mtodes per dibuixar,esborrar,ressaltar les llavors sobre la llesca
    void drawSeed(double x,double y,double z,double v);    

    /// canvia la llesca que estem mirant
    void ChangeSlice(int value);
    // Aquests estan aqu per cpia de l'anterior classe que feia aix "QGLSliceView" per 
    //estan buits
    void IntensityMax(int value);
    void IntensityMin(int value);
    void ZoomIn();
    void ZoomOut();

    ///Mtode utilitzat per captar els clicks de l'usuari (llavors)
    void addSeed();
    
    

signals:
    ///Signal que s'emetra quan l'usuari introdueixi una nova llavor
    void signalSeed(int,double,double,double,double);

    // Aquests signals estan aqu per cpia de l'anterior classe que feia aix "QGLSliceView"
    // de moment no s'han fet servir enlloc
    void Position(int, int, int, float);    
    void SliceChanged(int);

    void signalSlice(int);

private:
    
    QVBoxLayout* m_form1Layout;
    QVTKWidget* m_vtkWidget;
    Volume* m_volume; // s'hauria de fer cpia local!
    vtkImageViewer2* m_imageViewer;
    ViewType m_lastView;

    ///Atributs utilitzats per captar els clicks de l'usuari (llavors)
    vtkEventQtSlotConnect* m_connections;    
    vtkRenderWindowInteractor* m_interactor;
    vtkPropPicker* m_picker;

    ///Atributs utilitzats per dibuixar les llavors;
    vtkActor* m_vectorSeeds[50];
    int m_numSeeds;

    /**Guarda la llesca que t seleccionada. L'utilitzarem quan l'usuari afegeixi
       una llavor per collocar la resta de vistes en la mateixa llesca.
    */
    int m_slice;
    
    ///Aquest ens guardara la vista de la llesca. X(Axial)=0, Y(Sagittal)=1 i Z(Coronal)=2
    int m_view; 
};

};

#endif
