/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
 
#include "qsimple3dviewer.h"

//includes qt
#include <qlabel.h>
#include <qlayout.h>

//includes vtk
#include <vtkRenderer.h>
#include <vtkActor.h>
#include <vtkProperty.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkImageActor.h>

//includes per fer el casting i passar al tipus correcte
#include <vtkImageCast.h> 

#include <vtkImageData.h>
#include <vtkEventQtSlotConnect.h>
#include <QVTKWidget.h>

//includes propis
#include "volume.h"


//includes contouring
#include <vtkContourFilter.h>
#include <vtkSmoothPolyDataFilter.h>
#include <vtkPolyDataNormals.h>
#include <vtkStripper.h>
#include <vtkPolyDataMapper.h>

//includes ray casting
#include <vtkVolumeRayCastCompositeFunction.h>
#include <vtkVolumeRayCastMapper.h>
#include <vtkVolumeProperty.h>
#include <vtkColorTransferFunction.h>
#include <vtkPiecewiseFunction.h>

//includes vtkBoxWidget
#include <vtkBoxWidget.h>
#include <vtkPlanes.h>


namespace udg {

QSimple3DViewer::QSimple3DViewer(QWidget *parent, const char *name)
 : QWorkingAreaWidget(parent, name)
{
    if ( !name )    this->setName( "Widget 3D Volume" );
    
    m_form1Layout = new QVBoxLayout( this, 11, 6, "Form1Layout"); 

    m_vtkWidget = new QVTKWidget( this, "vTKWidget1" );
    m_form1Layout->addWidget( m_vtkWidget );

    //    m_textLabel2 = new QLabel( this, "textLabel2" );
    //    m_form1Layout->addWidget( m_textLabel2 );
    this->languageChange();
    this->resize( QSize(600, 480).expandedTo(minimumSizeHint()) );
    this->clearWState( WState_Polished );

    //Creem el Renderer de VTK i li assignem al widget que ens associa Qt amb VTK
    m_renderer = vtkRenderer::New();
    m_interactor=vtkRenderWindowInteractor::New();

    m_vtkWidget->GetRenderWindow()->AddRenderer( m_renderer );
    m_vtkWidget->GetRenderWindow()->GetInteractor()->SetRenderWindow(m_vtkWidget->GetRenderWindow());
}


QSimple3DViewer::~QSimple3DViewer()
{
    m_renderer->Delete();
}

void QSimple3DViewer::setVolume( Volume* inputImage )
{

    m_imageData = inputImage;
    // Fem un casting de les dades ja que el ray cast mapper nomes accepta unsigned char/short
    vtkImageCast* caster = vtkImageCast::New();
    caster->SetInput( m_imageData->getVtkData() );
    caster->SetOutputScalarType( VTK_UNSIGNED_CHAR ); // tb seria vlid VTK_UNSIGNED_SHORT
    caster->ClampOverflowOn();
    caster->Update();
    
    // Creem la funci de transferncia de l'opacitat
    vtkPiecewiseFunction* opacityTransferFunction = vtkPiecewiseFunction::New();
    opacityTransferFunction->AddPoint( 20, 0.0 );
    opacityTransferFunction->AddPoint( 255, 0.2 );
    
    // Creem la funci de transferncia de colors
    vtkColorTransferFunction* colorTransferFunction = vtkColorTransferFunction::New();
    colorTransferFunction->AddRGBPoint( 0.0, 0.0, 0.0, 0.0 );
    colorTransferFunction->AddRGBPoint( 64.0, 1.0, 0.0, 0.0 );
    colorTransferFunction->AddRGBPoint( 128.0, 0.0, 0.0, 1.0 );
    colorTransferFunction->AddRGBPoint( 192.0, 0.0, 1.0, 0.0 );
    colorTransferFunction->AddRGBPoint( 255.0, 0.0, 0.2, 0.0 );
    
    // La propietat descriur com es veuran les dades
    vtkVolumeProperty* volumeProperty = vtkVolumeProperty::New();
    volumeProperty->SetColor( colorTransferFunction );
    volumeProperty->SetScalarOpacity( opacityTransferFunction );
    
    // el mapper (funcio de ray cast) sabr com visualitzar les dades
    vtkVolumeRayCastCompositeFunction* compositeFunction = vtkVolumeRayCastCompositeFunction::New();
    vtkVolumeRayCastMapper* volumeMapper = vtkVolumeRayCastMapper::New();
    
    volumeMapper->SetVolumeRayCastFunction( compositeFunction );
    volumeMapper->SetInput( caster->GetOutput()  ); // abans inputImage->getVtkData()
    
    // el volum cont el mapper i la propietat i es pot usar per posicionar/orientar el volum
    vtkVolume* volume = vtkVolume::New();
    volume->SetMapper( volumeMapper );
    volume->SetProperty( volumeProperty );

    
    //vtkImageActor* actor = vtkImageActor::New();
    //actor->SetInput( inputImage->getVtkData() );
    //m_renderer->AddViewProp( actor );
    
    m_renderer->AddViewProp( volume );
    m_renderer->Render();

}





/**Mtode de visualitzaci 3D aplicant la tcnica de contouring 
(visualitzaci de superfcies)*/
void QSimple3DViewer::contouring(Volume* inputImage,int isolevel,QColor rgb)
{
    cout << "QSimple3DViewer Contouring: 1" << endl;
    m_imageData=inputImage;

    //Fem un casting de les dades 
    vtkImageCast* caster = vtkImageCast::New();
    caster->SetInput( m_imageData->getVtkData() );
    caster->SetOutputScalarType( VTK_UNSIGNED_CHAR );
    caster->ClampOverflowOn();
    caster->Update();

    cout << "QSimple3DViewer Contouring: 2" << endl;
    vtkContourFilter *contour = vtkContourFilter::New();
    contour->SetInput(caster->GetOutput());
    contour->SetValue(0,isolevel);
    cout << "QSimple3DViewer Contouring: 3" << endl;
    vtkSmoothPolyDataFilter* smoother = vtkSmoothPolyDataFilter::New();
    smoother->SetInput(contour->GetOutput());
    smoother->SetNumberOfIterations(4);
    smoother->SetRelaxationFactor(0.01);
    smoother->FeatureEdgeSmoothingOff();
    cout << "QSimple3DViewer Contouring: 4" << endl;
    vtkPolyDataNormals* norms = vtkPolyDataNormals::New();
    norms->SetInput(smoother->GetOutput());
    norms->SetFeatureAngle(60);
    cout << "QSimple3DViewer Contouring: 5" << endl;
    vtkStripper* strip = vtkStripper::New();
    strip->SetInput(norms->GetOutput());
    cout << "QSimple3DViewer Contouring: 6" << endl;
    vtkPolyDataMapper* isoMapper = vtkPolyDataMapper::New();
    isoMapper->SetInput(strip->GetOutput());
    isoMapper->ScalarVisibilityOff();
    cout << "QSimple3DViewer Contouring: 7" << endl;
    vtkActor* actor = vtkActor::New();
    actor->SetMapper(isoMapper);
    double red,green,blue;
    red=rgb.red();
    green=rgb.green();
    blue=rgb.blue();            
    red=red/255; green=green/255; blue=blue/255; 
    actor->GetProperty()->SetDiffuseColor(red,green,blue);
    cout << "QSimple3DViewer Contouring: 8" << endl;

    actor->RotateX(180); 
    actor->RotateY(180);

    m_renderer->AddActor(actor); cout << "QSimple3DViewer Contouring: 9" << endl;
    m_renderer->Render();
    cout << "QSimple3DViewer Contouring: 10" << endl;
}



///Representa per pantalla el volum de Ray Casting. Utilitzat per el mtode de visualitzaci.
void QSimple3DViewer::rayCasting(vtkVolume* volume,vtkVolumeRayCastMapper* mapper)
{
    m_volumeMapper =  vtkVolumeRayCastMapper::New();
    m_volumeMapper=mapper;
    m_renderer->AddViewProp( volume );

    m_boxWidget = vtkBoxWidget::New();
    m_boxWidget->SetInteractor(m_vtkWidget->GetRenderWindow()->GetInteractor());
    m_boxWidget->SetPlaceFactor(1.0);

    m_boxWidget->SetProp3D(volume);
    m_boxWidget->PlaceWidget();
    m_boxWidget->InsideOutOn();

    m_connections = vtkEventQtSlotConnect::New();
    m_connections->Connect(m_boxWidget,vtkCommand::StartInteractionEvent,this,SLOT(startInteraction()));    
    m_connections->Connect(m_boxWidget,vtkCommand::InteractionEvent,this,SLOT(clipVolumeRender()));    
    m_connections->Connect(m_boxWidget,vtkCommand::EndInteractionEvent,this,SLOT(endInteraction()));   

    vtkProperty* outlineProperty = m_boxWidget->GetOutlineProperty();
    outlineProperty->SetRepresentationToWireframe();
    outlineProperty->SetAmbient(1.0);
    outlineProperty->SetAmbientColor(1, 1, 1);
    outlineProperty->SetLineWidth(3);

    vtkProperty* selectedOutlineProperty = m_boxWidget->GetSelectedOutlineProperty();
    selectedOutlineProperty->SetRepresentationToWireframe();
    selectedOutlineProperty->SetAmbient(1.0);
    selectedOutlineProperty->SetAmbientColor(1, 0, 0);
    selectedOutlineProperty->SetLineWidth(3);

    m_renderer->Render();

}

void QSimple3DViewer::setBoxOn() { m_boxWidget->On(); }
void QSimple3DViewer::setBoxOff(){ m_boxWidget->Off(); }

void QSimple3DViewer::startInteraction()
{
    m_vtkWidget->GetRenderWindow()->SetDesiredUpdateRate(10);
}

void QSimple3DViewer::clipVolumeRender()
{
    vtkPlanes* planes= vtkPlanes::New();
    m_boxWidget->GetPlanes(planes);
    m_volumeMapper->SetClippingPlanes(planes);
}

void QSimple3DViewer::endInteraction()
{
    m_vtkWidget->GetRenderWindow()->SetDesiredUpdateRate(0.001);
}


///Representa per pantalla el volum de Ray Casting. Utilitzat per la visualitzaci 3D dels resultats.
void QSimple3DViewer::rayCasting(Volume* inputImage,QColor rgb)
{
    m_imageData=inputImage;

    //Fem un casting de les dades 
    vtkImageCast* caster = vtkImageCast::New();
    caster->SetInput( m_imageData->getVtkData() );
    caster->SetOutputScalarType( VTK_UNSIGNED_CHAR );
    caster->ClampOverflowOn();
    caster->Update();

    vtkPiecewiseFunction* m_opacityTransferFunction = vtkPiecewiseFunction::New();
    vtkColorTransferFunction* m_colorTransferFunction = vtkColorTransferFunction::New();
    vtkVolumeProperty* m_volumeProperty = vtkVolumeProperty::New();
    vtkVolumeRayCastCompositeFunction* m_compositeFunction = vtkVolumeRayCastCompositeFunction::New();
    m_volumeMapper =  vtkVolumeRayCastMapper::New();
    vtkVolume* m_volume = vtkVolume::New();   

    m_opacityTransferFunction->ClampingOff();
    m_opacityTransferFunction->RemoveAllPoints();
    m_colorTransferFunction->ClampingOff();
    m_colorTransferFunction->RemoveAllPoints();

    double red,green,blue;
    red=rgb.red();
    green=rgb.green();
    blue=rgb.blue();            
    red=red/255; green=green/255; blue=blue/255; 

    //Representem tot el volum. Obviem els que tenen un valor inferior a 40 (aire,...).
    m_opacityTransferFunction->AddSegment(40,0.3,255,1.0);
    m_colorTransferFunction->AddRGBSegment(40,red,green,blue,255,red,green,blue);

    m_volumeProperty->SetInterpolationTypeToLinear();
    m_volumeMapper->SetSampleDistance(0.2);

    m_volumeProperty->SetColor( m_colorTransferFunction );
    m_volumeProperty->SetScalarOpacity( m_opacityTransferFunction );
    m_volumeProperty->ShadeOn();    

    // el mapper (funcio de ray cast) sabr com visualitzar les dades
    m_volumeMapper->SetVolumeRayCastFunction( m_compositeFunction );
    m_volumeMapper->SetInput( caster->GetOutput() ); // abans inputImage->getVtkData()

    // el volum cont el mapper i la propietat i es pot usar per posicionar/orientar el volum
    m_volume->SetMapper( m_volumeMapper );
    m_volume->SetProperty( m_volumeProperty ); 
    m_volume->RotateX(180);
    m_volume->RotateY(180);

    m_renderer->AddViewProp( m_volume );

    m_boxWidget = vtkBoxWidget::New();
    m_boxWidget->SetInteractor(m_vtkWidget->GetRenderWindow()->GetInteractor());
    m_boxWidget->SetPlaceFactor(1.0);

    m_boxWidget->SetProp3D(m_volume);
    m_boxWidget->PlaceWidget();
    m_boxWidget->InsideOutOn();

    m_connections = vtkEventQtSlotConnect::New();
    m_connections->Connect(m_boxWidget,vtkCommand::StartInteractionEvent,this,SLOT(startInteraction()));    
    m_connections->Connect(m_boxWidget,vtkCommand::InteractionEvent,this,SLOT(clipVolumeRender()));    
    m_connections->Connect(m_boxWidget,vtkCommand::EndInteractionEvent,this,SLOT(endInteraction()));   

    vtkProperty* outlineProperty = m_boxWidget->GetOutlineProperty();
    outlineProperty->SetRepresentationToWireframe();
    outlineProperty->SetAmbient(1.0);
    outlineProperty->SetAmbientColor(1, 1, 1);
    outlineProperty->SetLineWidth(3);

    vtkProperty* selectedOutlineProperty = m_boxWidget->GetSelectedOutlineProperty();
    selectedOutlineProperty->SetRepresentationToWireframe();
    selectedOutlineProperty->SetAmbient(1.0);
    selectedOutlineProperty->SetAmbientColor(1, 0, 0);
    selectedOutlineProperty->SetLineWidth(3);

    m_renderer->Render();
}

/*
void QSimple3DViewer::rayCasting(Volume* inputImage)
{

    m_imageData = inputImage;

    //Fem un casting de les dades ja que el ray cast mapper nomes accepta unsigned char/short
    vtkImageCast* caster = vtkImageCast::New();
    caster->SetInput( m_imageData->getVtkData() );
    caster->SetOutputScalarType( VTK_UNSIGNED_CHAR ); // tamb seria vlid VTK_UNSIGNED_SHORT
    caster->ClampOverflowOn();
    caster->Update();

    // Creem la funci de transferncia de l'opacitat
    vtkPiecewiseFunction* opacityTransferFunction = vtkPiecewiseFunction::New();
    opacityTransferFunction->ClampingOff();

    //opacityTransferFunction->AddPoint( 950, 0.5 );
    //opacityTransferFunction->AddPoint( 430, 0.5 );
    //opacityTransferFunction->AddPoint( 500, 0.0 );

    //opacityTransferFunction->AddSegment( 30, 0.1, 50, 0.3 );
    //opacityTransferFunction->AddSegment( 60, 0.35, 100, 0.5 );
    //opacityTransferFunction->AddSegment( 110, 0.55, 150, 0.7 );
    //opacityTransferFunction->AddSegment( 1010, 0.75, 1200, 0.9 );
    //opacityTransferFunction->AddSegment( 30, 0.0, 50, 0.0 );
    //opacityTransferFunction->AddSegment( 60, 0.0, 100, 0.0 );
    //opacityTransferFunction->AddSegment( 100, 0.4, 300, 0.8 );
    //opacityTransferFunction->AddSegment( 950, 0.5, 1000, 0.55 );

    // Creem la funci de transferncia de colors
    vtkColorTransferFunction* colorTransferFunction = vtkColorTransferFunction::New();
    colorTransferFunction->ClampingOff();

    //colorTransferFunction->AddRGBPoint( 90, 0.8, 0.5, 0.5 );//color pell 
    //colorTransferFunction->AddRGBPoint( 430.0, 1.0, 0.0, 0.0 );
    //colorTransferFunction->AddRGBPoint( 950.0, 0.8, 0.5, 0.5 );//color pell
    //colorTransferFunction->AddRGBPoint( 1010.0, 0.7, 0.6, 0.4 );//marr
    //colorTransferFunction->AddRGBPoint( 1300.0, 1.0, 1.0, 0.8 );//blanquinos

    //colorTransferFunction->AddRGBSegment( 30, 0.8, 0.5, 0.5, 50, 0.8, 0.5, 0.5 ); //color pell
    //colorTransferFunction->AddRGBSegment( 60, 0.0, 1.0, 0.0, 100, 0.0, 1.0, 0.0 ); //color pell
    //colorTransferFunction->AddRGBSegment( 100, 0.8, 0.5, 0.5, 300, 0.8, 0.5, 0.5 ); //color pell
    //colorTransferFunction->AddRGBSegment( 1020, 0.7, 0.6, 0.4, 1200, 0.7, 0.6, 0.4 ); //marronos
    //colorTransferFunction->AddRGBSegment( 1020, 0.0, 1.0, 0.0, 1200, 0.0, 1.0, 0.0 ); //marronos
    //colorTransferFunction->AddRGBSegment( 1300, 1.0, 1.0, 0.8, 1600, 1.0, 1.0, 0.8 ); //blanquins
    //colorTransferFunction->AddRGBSegment( 1300, 1.0, 1.0, 0.8, 1600, 1.0, 1.0, 0.8 ); //blanquins

    // La propietat descriur com es veuran les dades
    vtkVolumeProperty* volumeProperty = vtkVolumeProperty::New();
    volumeProperty->SetColor( colorTransferFunction );
    volumeProperty->SetScalarOpacity( opacityTransferFunction );
    volumeProperty->ShadeOn();
    volumeProperty->SetInterpolationTypeToLinear();

    // el mapper (funcio de ray cast) sabr com visualitzar les dades
    vtkVolumeRayCastCompositeFunction* compositeFunction = vtkVolumeRayCastCompositeFunction::New();
    vtkVolumeRayCastMapper* volumeMapper = vtkVolumeRayCastMapper::New();
    volumeMapper->SetVolumeRayCastFunction( compositeFunction );
    volumeMapper->SetInput( caster->GetOutput() ); // abans inputImage->getVtkData()
    //volumeMapper->SetSampleDistance(0.2); 

    // el volum cont el mapper i la propietat i es pot usar per posicionar/orientar el volum
    vtkVolume* volume = vtkVolume::New();
    volume->SetMapper( volumeMapper );
    volume->SetProperty( volumeProperty );
    volume->RotateX(180);
    volume->RotateY(180);

    m_renderer->AddViewProp( volume );
    m_renderer->Render();

}
*/




void QSimple3DViewer::languageChange()
{
    this->setCaption( tr( "Widget Volume 3D" ) );
    // m_textLabel2->setText( tr( "textLabel2" ) );
}

void QSimple3DViewer::updateCoords( vtkObject* obj )
{
    // Obtenim l'interactor
    vtkRenderWindowInteractor* iren = vtkRenderWindowInteractor::SafeDownCast(obj);
    // Obtenim la posici de l'event (moure el mouse, en aquest cas)
    int event_pos[2];
    iren->GetEventPosition(event_pos);
    /*
    /// \TODO QSimple3DViewer::updateCoords Actualitzem el textLabel amb aquesta informaci
    QString str;
    str.sprintf("x=%d : y=%d", event_pos[0], event_pos[1]);
    m_textLabel2->setText(str);
    */
}

}; // end namespace udg
