/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#ifndef UDGQSIMPLE3DVIEWER_H
#define UDGQSIMPLE3DVIEWER_H

#include "qworkingareawidget.h"
#include "vtkActor.h"
#include "vtkVolume.h"
#include "vtkVolumeRayCastMapper.h"
#include "vtkBoxWidget.h"

class QVBoxLayout;
class QVTKWidget;

class vtkRenderer;
class vtkRenderWindowInteractor;
class vtkEventQtSlotConnect;
class vtkObject;


namespace udg {

class Volume;

/**
    Aquesta classe fa el render d'un 'Volume' amb vtk a travs d'un algorisme ray casting

@author Grup de Grfics de Girona  ( GGG )
*/
class QSimple3DViewer : public QWorkingAreaWidget{
    Q_OBJECT
public:
    QSimple3DViewer(QWidget *parent = 0, const char *name = 0);

    ~QSimple3DViewer();
    
    /**
        Li indiquem el volum a visualitzar
    */
    void setVolume( Volume* inputImage );

    /**Mtode de visualitzaci 3D aplicant la tcnica de contouring (visualitzaci de superfcies)*/
    void contouring(Volume* inputImage,int isolevel,QColor rgb);

    /*///Mtode de visualitzaci 3D aplicant la tcnica de ray-casting (visualitzaci de volums)
    void rayCasting(Volume* inputImage);
    */
    ///Representa per pantalla el volum de Ray Casting. Utilitzat per el mtode
    ///de visualitzaci.
    void rayCasting(vtkVolume* volume,vtkVolumeRayCastMapper* mapper);

    ///Representa per pantalla el volum de Ray Casting. Utilitzat per la visualitzaci 3D dels resultats.
    void rayCasting(Volume* inputImage,QColor rgb);

    void setBoxOn();
    void setBoxOff();


protected slots:
    ///Posa les cadenes de texte als subwidgets utilitzant l'idioma actual
    virtual void languageChange();
    void updateCoords(vtkObject*);

    //Slots utilitzats per el vtkBoxWidget
    void startInteraction();
    void clipVolumeRender();
    void endInteraction();
    
private:
    QVBoxLayout* m_form1Layout;
    QVTKWidget* m_vtkWidget;

    vtkRenderer* m_renderer;
    vtkRenderWindowInteractor* m_interactor;
    vtkEventQtSlotConnect* m_connections;
    
    Volume* m_imageData;

    vtkBoxWidget* m_boxWidget;
    vtkVolumeRayCastMapper* m_volumeMapper;

};

};  // end namespace udg

#endif
