/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#include "qsimplesliceviewer.h"

//includes qt
#include <qlayout.h>

//includes vtk
#include <vtkImageViewer2.h>
#include <QVTKWidget.h>

#include <vtkImagePermute.h> // per poder permutar els eixos
#include <vtkRenderWindow.h>
#include <vtkRenderer.h>
#include <vtkCamera.h>

//includes propis
#include "volume.h"

#include <vtkImageActor.h>

namespace udg {

QSimpleSliceViewer::QSimpleSliceViewer(QWidget *parent, const char *name)
 : QWorkingAreaWidget(parent, name)
{
    if ( !name )
        this->setName( "QSimpleSliceViewer" );
    
    m_form1Layout = new QVBoxLayout( this, 0, 0, "Form1Layout"); 

    m_vtkWidget = new QVTKWidget( this, "vTKWidget1" );
    m_form1Layout->addWidget( m_vtkWidget );
      
    m_lastView = Axial; // per defecte la vista es Axial
    
    m_volume = 0;
}


QSimpleSliceViewer::~QSimpleSliceViewer()
{
//    m_imageViewer->Delete();
}

void QSimpleSliceViewer::setVolume( Volume* inputVolume )
{
    m_volume = inputVolume;
    m_imageViewer = vtkImageViewer2::New();
    m_imageViewer->SetInput( m_volume->getVtkData() );

    m_vtkWidget->SetRenderWindow( m_imageViewer->GetRenderWindow() );
    m_imageViewer->SetupInteractor( m_vtkWidget->GetRenderWindow()->GetInteractor() );

    int *size = m_volume->getVtkData()->GetDimensions();
    m_imageViewer->SetSlice( size[2]/2 );

    // li donem el window/level correcte
    // aix ens dna un level/level "maco" per defecte
    double * range = m_volume->getVtkData()->GetScalarRange();
    double window = fabs(range[1] - range[0]);
    double level = (range[1] + range[0] )/ 2.0;

    //Amb els models de porc hi ha problemes i el window i level calculats sn molt superiors als
    //que haurien de ser. Per aquest motiu les imatges no es veuen correctament. Per solucionar-ho
    //assignarem per defecte un level de 1137 i window de 2275. 
    if((window>3500)&&(level>1900))
    {
        level=1137;
        window=2275;

        //cout << "Window: " << window << endl;
        //cout << "Level: " << level << endl;
        m_imageViewer->SetColorLevel( level );
        m_imageViewer->SetColorWindow( window );
    }

    else
    {
        //cout << "Window: " << window << endl;
        //cout << "Level: " << level << endl;
        m_imageViewer->SetColorLevel( level );
        m_imageViewer->SetColorWindow( window );
    }

}

void QSimpleSliceViewer::setView( ViewType view )
{    
    vtkImageData* vtkImage = m_volume->getVtkData();
    vtkImagePermute* permuter = vtkImagePermute::New();
    
    permuter->SetInput( vtkImage );
    switch( view )
    {
    case Axial:
       m_imageViewer->SetSliceOrientationToXY();  
      //permuter->SetFilteredAxes( 0, 1 , 2 );
    /*
        if( m_lastView == Coronal )
            permuter->SetFilteredAxes( 0, 2 , 1 );
        else if( m_lastView == Sagittal )
            permuter->SetFilteredAxes( 1, 2 , 0);
    */
        m_lastView = Axial;
    break;
    case Sagittal:
        m_imageViewer->SetSliceOrientationToYZ();
        //permuter->SetFilteredAxes( 1, 2 , 0);
    /*
        if( m_lastView == Axial )
            permuter->SetFilteredAxes( 1, 2 , 0);
        else if( m_lastView == Coronal )
            permuter->SetFilteredAxes( 1, 2 , 0);
    */
        m_lastView = Sagittal;
    break;
    case Coronal:
        m_imageViewer->SetSliceOrientationToXZ();
        //permuter->SetFilteredAxes( 0, 2 , 1 );
    /*
        if( m_lastView == Axial )
            permuter->SetFilteredAxes( 0, 2 , 1 );
        else if( m_lastView == Sagittal )
            permuter->SetFilteredAxes( 1, 2 , 0);
    */
        m_lastView = Coronal;
    break;
    }

    //POSEM B LA CMERA
        // Aix s necessari perqu la imatge es rescali a les mides de la finestreta
        m_imageViewer->GetRenderer()->ResetCamera();
        //Moc la cmera 180 graus de manera que els models de porc es vegin correctament
        vtkCamera *camera=vtkCamera::New();
        camera=m_imageViewer->GetRenderer()->GetActiveCamera();
        camera->SetRoll(0);
        camera->Roll(180);
        m_imageViewer->GetRenderer()->SetActiveCamera(camera);
    //FI POSEM B LA CMERA

    //m_imageViewer->SetInput( permuter->GetOutput() );
}

void QSimpleSliceViewer::ChangeSlice( int value )
{
    // sempre ho farem sobre l'eix de les Z
    int *size = m_volume->getVtkData()->GetDimensions();
    // aqu caldria verificar les tres dims?, se suposa que no que tenim volums
    
    if( value <= size[2] && value >= 0 )
    {
        m_imageViewer->SetSlice( value );
        m_imageViewer->Render();
    }
}


void QSimpleSliceViewer::IntensityMax( int value )
{
}

void QSimpleSliceViewer::IntensityMin( int value )
{
}

void QSimpleSliceViewer::ZoomIn()
{
}

void QSimpleSliceViewer::ZoomOut()
{
}

};
