#include "qwcolorbutton.h"
#include <qimage.h>
#include <qcolordialog.h>

//!!!!!!!!!!!!!!!!!!!!!!!!!
#include <iostream>

QwColorButton::QwColorButton(QWidget* parent, const char* name): QPushButton(parent,name)
{
  m_color.setRgb( qRgba(255,255,240,255) );
  m_porcentatgePuntRodo = 10;
  m_amplada = m_alcada = 25;
  m_difuminat = false;

  this->dibuixaQuadrat();

  emit colorCanviat ( m_color );

  connect( this, SIGNAL(clicked()),
           this, SLOT(mostraMenuColor()) );
}

QwColorButton::QwColorButton( QColor c, unsigned int w, unsigned int h,QWidget* parent, const char* name)
  :QPushButton(parent,name)
{
  m_color = c;
  m_amplada = w>0?w:1;
  m_alcada = h>0?h:1;
  m_difuminat = false;

  this->dibuixaQuadrat();

  emit colorCanviat ( m_color );

  connect( this, SIGNAL(clicked()),
           this, SLOT(mostraMenuColor()) );
}

QwColorButton::QwColorButton(QRgb c, unsigned int w, unsigned int h,QWidget* parent, const char* name)
  :QPushButton(parent,name)
{
  m_color.setRgb(c);
  m_amplada = w>0?w:1;
  m_alcada = h>0?h:1;
  m_difuminat = false;
  m_porcentatgePuntRodo = 10;

  this->dibuixaQuadrat();

  emit colorCanviat ( m_color );

  connect( this, SIGNAL(clicked()),
           this, SLOT(mostraMenuColor()) );
//  this->QwColorButton(QColor(c), w, h, parent, name);
}

QwColorButton::~QwColorButton()
{
}

QColor QwColorButton::getColor()
{
  return m_color;
}

void QwColorButton::setPuntRodo( unsigned int perCent)
{
  if (m_porcentatgePuntRodo != perCent)
  {
    //Fem que vagi de 0..50, mapejant el 0..100
    m_porcentatgePuntRodo = perCent > 100 ? 50 : perCent/2;
    this->dibuixaQuadrat();
  }
}

void QwColorButton::setDifuminat( bool d)
{
  m_difuminat = d;
  this->dibuixaQuadrat();
}

void QwColorButton::setTamanyQuadreColor(unsigned int w, unsigned int h)
{
  m_amplada = w>0?w:1;
  m_alcada = h>0?w:1;
  this->dibuixaQuadrat();
}

void QwColorButton::mostraMenuColor()
{
  QRgb rgba = m_color.rgb();
  bool colorCorrecte = false;
  QColor c;

  rgba = QColorDialog::getRgba(rgba,&colorCorrecte);
  c.setRgb( rgba );

  if (colorCorrecte && c != m_color)
  {
    m_color = c;
    this->dibuixaQuadrat();

    emit colorCanviat ( c );
    emit colorSeleccionat( c );
  }
}

void QwColorButton::setColor( const QColor &c )
{
  if (c != m_color)
  {
    m_color = c;
    this->dibuixaQuadrat();

    emit colorCanviat ( c );
  }
}

void QwColorButton::dibuixaQuadrat()
{
  int r, g, b;
  const double k_alpha = static_cast<double>(qAlpha( m_color.rgb() )) / 255.0;
  double alpha;

  QImage img ( m_amplada, m_alcada, 32 );
  img.fill ( 0 );

  m_color.rgb ( &r, &g, &b );

  int w = img.width ( );
  int h = img.height ( );

  int dx = w * m_porcentatgePuntRodo / 100; // Per arrodonir els cantos del quadrat
  int dy = h * m_porcentatgePuntRodo / 100;

  for ( int y = 0; y < h; y++ )
  {
      for ( int x = 0; x < w; x++ )
      {
      alpha = k_alpha;

      if ( x < dx )
            alpha *= ( double ( x + 1 ) / dx );
        else if ( x >= w - dx )
            alpha *= ( double ( w - x ) / dx );
        if ( y < dy )
            alpha *= ( double ( y + 1 ) / dy );
        else if ( y >= h - dy )
            alpha *= ( double ( h - y ) / dy );

        if ( alpha != k_alpha  &&  !m_difuminat )
        {
          if (alpha < 0.5) alpha = 0.0;
          else alpha = 1.0;
        }

        int a = static_cast<int>(alpha*255.0);
        if ( a < 0 ) a = 0;
        else if ( a > 255 ) a = 255;

        img.setPixel ( x, y, qRgba ( r, g, b, a ));
      }
  }
  img.setAlphaBuffer ( true );

  QPixmap pix;
  pix.convertFromImage ( img );
  this->setPixmap ( pix );
}
