#ifndef QWCOLORBUTTON_H
#define QWCOLORBUTTON_H

#include <qwidget.h>
#include <qpushbutton.h>

/**Qt Widget ( Qw ) que serveix per tenir un bot que ens permet triar un color i "recordar-lo". El bot quedar pintat del color que triem. Accepta colors amb Alpha i la tria es fa a partir d'un QColorDialog
  *@author Joaquim Rodrguez i Guerrero
  */

class QwColorButton : public QPushButton  {
   Q_OBJECT
public: 
  QwColorButton(QWidget *parent=0, const char *name=0);
  ~QwColorButton();
  QwColorButton(QColor, unsigned int w=25, unsigned int h=25,QWidget* parent=0, const char* name=0);
  QwColorButton(QRgb, unsigned int w=25, unsigned int h=25,QWidget* parent=0, const char* name=0);  
  QColor getColor();

  QRgb getColorRgba() { return m_color.rgb(); }
  int getRed() { return m_color.red(); }
  int getGreen() { return m_color.green(); }
  int getBlue() { return m_color.blue(); }  
  int getAlpha() { return qAlpha(m_color.rgb()); }

  /**
    Serveix per indicar quin % s'ha d'aplicar per arrodonir els cantos
    del quadrat de color.
    */
  void setPuntRodo( unsigned int );

  /**
    Indica si els cantos del quadrat fan un difuminat amb el fons
    */
  void setDifuminat( bool );
  void setTamanyQuadreColor(unsigned int, unsigned int);

public slots:
  void setColor( const QColor & );
  void setPuntRodo( int i ){ this->setPuntRodo((unsigned int)i); }

signals:

  /** Senyal que s'emet noms quan es selecciona un color amb el men
      per no quan s canviat mitjanant setColor o altres mtodes
    */
  void colorSeleccionat ( const QColor & );

  /** Senyal que s'emet cada cop que el color del bot canvia, sigui
      quin sigui el mtode que s'hagi fet servir per canviar-ho.
    */
  void colorCanviat (const QColor & );


protected:
  void dibuixaQuadrat();


private slots:
  void mostraMenuColor();

private:
  QColor m_color;
  unsigned int m_porcentatgePuntRodo;
  int m_amplada, m_alcada;
  bool m_difuminat;  
};

#endif
