/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

 
 #include "qwidgetvolume.h"

//includes qt
#include <qlabel.h>
#include <qlayout.h>

//includes vtk
#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include <vtkCellPicker.h>
#include <vtkProperty.h>
#include <vtkImagePlaneWidget.h>

#include <vtkImageData.h>
#include <vtkEventQtSlotConnect.h>
#include <QVTKWidget.h>

//includes propis
#include "volume.h"

#include <vtkCamera.h>

namespace udg{

QWidgetVolume::QWidgetVolume(QWidget *parent, const char *name)
 : QWorkingAreaWidget(parent, name)
{
    if ( !name )    this->setName( "Widget 3D" );
    
    m_form1Layout = new QVBoxLayout( this, 11, 6, "Form1Layout"); 

    m_vtkWidget = new QVTKWidget( this, "vTKWidget1" );
    m_form1Layout->addWidget( m_vtkWidget );

    this->languageChange();
    this->resize( QSize(600, 480).expandedTo(minimumSizeHint()) );
    this->clearWState( WState_Polished );

    //Creem el Renderer de VTK i li assignem al widget que ens associa Qt amb VTK
    m_ren = vtkRenderer::New();
    m_vtkWidget->GetRenderWindow()->AddRenderer(m_ren);
}


QWidgetVolume::~QWidgetVolume()
{
    m_ren->Delete();
}



void QWidgetVolume::setVolume( Volume* inputImage )
{
    m_imageData = inputImage;
    //vtkImageData* image = inputImage->getVtkData();

    int *size = m_imageData->getVtkData()->GetDimensions();
    //int *size = image->GetDimensions();

    //------------------------------------------------------------------------
    // VTK pipeline.
    //------------------------------------------------------------------------

    // Utilitzem un "cell picker" per interactuar amb les vistes ortogonals.
    vtkCellPicker * picker = vtkCellPicker::New();
    picker->SetTolerance(0.005);

    //Assignem les propietats per defecte
    vtkProperty * ipwProp = vtkProperty::New();

    // Creem tres vistes ortogonals utilitzant la classe ImagePlaneWidget
    xImagePlaneWidget =  vtkImagePlaneWidget::New();
    yImagePlaneWidget =  vtkImagePlaneWidget::New();
    zImagePlaneWidget =  vtkImagePlaneWidget::New();

    // Els 3 widgets s'utilizen per visualizar el model
    // (mostra imatges en 2D amb 3 orientacions diferents)
    xImagePlaneWidget->DisplayTextOn();
    xImagePlaneWidget->SetInput( m_imageData->getVtkData() ); // atenci estem posant aqu le dades vtk
    //xImagePlaneWidget->SetInput( image ); // atenci estem posant aki le dades
    xImagePlaneWidget->SetPlaneOrientationToXAxes();
    xImagePlaneWidget->SetSliceIndex(size[0]/2);
    xImagePlaneWidget->SetPicker(picker);
    xImagePlaneWidget->RestrictPlaneToVolumeOn();
    xImagePlaneWidget->SetKeyPressActivationValue('x');
    xImagePlaneWidget->GetPlaneProperty()->SetColor(1, 0, 0);
    xImagePlaneWidget->SetTexturePlaneProperty(ipwProp);
    xImagePlaneWidget->SetResliceInterpolateToNearestNeighbour();

    yImagePlaneWidget->DisplayTextOn();
    yImagePlaneWidget->SetInput( m_imageData->getVtkData() );
    yImagePlaneWidget->SetPlaneOrientationToYAxes();
    yImagePlaneWidget->SetSliceIndex(size[1]/2);
    yImagePlaneWidget->SetPicker(picker);
    yImagePlaneWidget->RestrictPlaneToVolumeOn();
    yImagePlaneWidget->SetKeyPressActivationValue('y');
    yImagePlaneWidget->GetPlaneProperty()->SetColor(1, 1, 0);
    yImagePlaneWidget->SetTexturePlaneProperty(ipwProp);
    yImagePlaneWidget->SetLookupTable(xImagePlaneWidget->GetLookupTable());

    zImagePlaneWidget->DisplayTextOn();
    zImagePlaneWidget->SetInput( m_imageData->getVtkData() );
    zImagePlaneWidget->SetPlaneOrientationToZAxes();
    zImagePlaneWidget->SetSliceIndex(size[2]/2);
    zImagePlaneWidget->SetPicker(picker);
    zImagePlaneWidget->SetKeyPressActivationValue('z');
    zImagePlaneWidget->GetPlaneProperty()->SetColor(0, 0, 1);
    zImagePlaneWidget->SetTexturePlaneProperty(ipwProp);
    zImagePlaneWidget->SetLookupTable(xImagePlaneWidget->GetLookupTable());

    xImagePlaneWidget->SetInteractor( m_vtkWidget->GetRenderWindow()->GetInteractor() );    
    xImagePlaneWidget->On();

    yImagePlaneWidget->SetInteractor( m_vtkWidget->GetRenderWindow()->GetInteractor() );
    yImagePlaneWidget->On();

    zImagePlaneWidget->SetInteractor( m_vtkWidget->GetRenderWindow()->GetInteractor() );
    zImagePlaneWidget->On();

    // Indiquem el color de fons
    m_ren->SetBackground(0.4392, 0.5020, 0.5647);


    //Connectem, com a exemple, una senyal de la interacci de l'usuari amb un element de Qt
    //En aquest exemple fem que cada cop que l'usuari mogui el mouse per la pantalla en 3D
    //es vagi mostrant la coordenada en un textLabel.

    connections = vtkEventQtSlotConnect::New();
    //connections->Connect(m_vtkWidget->GetRenderWindow()->GetInteractor(),vtkCommand::MouseMoveEvent,this,SLOT(updateCoords(vtkObject*)));
    connections->Connect(xImagePlaneWidget,vtkCommand::InteractionEvent,this,SLOT(updateCoords()));
    connections->Connect(yImagePlaneWidget,vtkCommand::InteractionEvent,this,SLOT(updateCoords()));
    connections->Connect(zImagePlaneWidget,vtkCommand::InteractionEvent,this,SLOT(updateCoords()));

    m_ren->ResetCamera();
    //Moc la cmera 180 graus de manera que els models de porc es vegin correctament
    vtkCamera *camera=vtkCamera::New();
    camera=m_ren->GetActiveCamera();
    camera->Roll(180);
    m_ren->SetActiveCamera(camera);

    // Executem el Renderer perqu mostri les imatges
    m_ren->Render();
}

void QWidgetVolume::languageChange()
{
    this->setCaption( tr( "Widget 3D" ) );
}

bool QWidgetVolume::isValidCursorStatus()
{
    return xImagePlaneWidget->GetCursorDataStatus() == 1 || yImagePlaneWidget->GetCursorDataStatus() == 1 || zImagePlaneWidget->GetCursorDataStatus() == 1;
}


void QWidgetVolume::updateCoords( )
{ 
    //if ( this->isValidCursorStatus() )
    //{
    double coordenates[3];

    //std::cout<<"l'estat del cursor es "<<xImagePlaneWidget->GetCursorDataStatus()<<std::endl;

    if ( xImagePlaneWidget->GetCursorDataStatus()==1)
    {
        xImagePlaneWidget->GetCurrentCursorPosition(coordenates);
    }
    else if ( yImagePlaneWidget->GetCursorDataStatus() == 1 )
    {
        yImagePlaneWidget->GetCurrentCursorPosition(coordenates);
    }
    else if ( zImagePlaneWidget->GetCursorDataStatus() == 1 )
    {
        zImagePlaneWidget->GetCurrentCursorPosition(coordenates);
    }

    // Actualitzem el textLabel amb aquesta informaci
    QString str;
    str.sprintf("(x=%g, y=%g, z=%g)", coordenates[0], coordenates[1], coordenates[2]);

    emit coordenatesChangedSignal(coordenates[0], coordenates[1], coordenates[2]);
    //}
    /*else
    {
        std::cout << "QWidgetVolume::updateCoords Invalid Current Coordinates" << std::endl;
    }*/


    // Obtenim l'interactor
    /*vtkRenderWindowInteractor* iren = vtkRenderWindowInteractor::SafeDownCast(obj);
    // Obtenim la posici de l'event (moure el mouse, en aquest cas)
    int event_pos[2];
    iren->GetEventPosition(event_pos);
    // Actualitzem el textLabel amb aquesta informaci
    QString str;
    str.sprintf("x=%d : y=%d", event_pos[0], event_pos[1]);
    m_textLabel2->setText(str);
    */
}

}; //end namespace udg
