#include "raycastingdirector.h"
#include "raycastingparameters.h"

#include "volumerepository.h"

// per la finestra que obrim com a resultat
#include "qwidgetvolume.h"

// per tenir accs a l'aplication window 
#include "qapplicationmainwindow.h"

// per debug
#include <iostream.h>

namespace udg {

RayCastingDirector::RayCastingDirector(udg::QApplicationMainWindow* app, QObject *parent, const char *name):Director(app, parent, name)
{
    m_applicationWindow = app;
    m_parameters = 0;
    m_method = new MethodType;    
}


RayCastingDirector::~RayCastingDirector()
{
    delete m_method;    
}



void RayCastingDirector::setParameters( RayCastingParameters* parameters )
{
	m_parameters = parameters;
}


void RayCastingDirector::execute()
{
 
    if( !m_parameters )
    {
        // Si no hi ha parmetres establerts no podem fer res
        // \TODO exception, cerr,... ?
        std::cerr << "RayCastingDirector::No hi ha parametres establerts" << std::endl;
    }
    else
    {	
        // obtenim el repositori
        udg::VolumeRepository* repository = udg::VolumeRepository::getRepository();

        // obtenim el volum a treballar
        Volume* volum = repository->getVolume( m_parameters->getVolumeId() );

        // li indiquem al mtode els parmetres necessaris
        m_method->setInputVolume( volum );
        m_method->setTransferFunction(m_parameters->getTransferFunction());
        //m_method->setParameters( m_parameters->getInterpolation(),m_parameters->getDistance());
        m_method->setParameters( );

        if( m_method->applyMethod() )
        {
            ViewResultsRayCastingWidget* viewResults = new ViewResultsRayCastingWidget(m_applicationWindow, "RayCastingWidget");
            viewResults->rayCasting(this->getVolume(),this->getMapper());
            m_applicationWindow->addWorkingAreaWidget(viewResults, tr("Ray Casting"));
            viewResults->show();
	}

        else
        {
            std::cerr << " No s'ha pogut aplicar el mtode de visualitzaci" << std::endl;
        }
    }
}



vtkVolume* RayCastingDirector::getVolume()
{
	return m_method->getVolume();
}

vtkVolumeRayCastMapper* RayCastingDirector::getMapper()
{
	return m_method->getMapper();
}


}; //end namespace udg
