#ifndef UDGRAYCASTINGDIRECTOR_H
#define UDGRAYCASTINGDIRECTOR_H

#include "director.h"
#include "volume.h"
#include "raycasting.h"
#include "viewresultsraycastingwidget.h"

namespace udg {

//Fordware declaration
class RayCastingParameters;


/**
    Sub-classe de Director. En aquest cas fa de pont amb un mtode de visualitzaci anomenat
    Ray Casting.
    
    Aquesta inclou un punter a la classe de parmetres especfics del seu mtode, la classe que 
    encapsula el mtode de segmentaci ( RayCasting )
    
    Totes les classes heredades de Director, inclouran un mtode setParameters on l'indicarem el 
    punter a la classe de parmetres corresponent. Tamb implementaran l'slot virtual apply, 
    on es far la feina ms important de l'execuci d'un mtode
    
*/


class RayCastingDirector : public Director
{
Q_OBJECT
public:

    typedef udg::Volume::ItkImageType ImageType;
    typedef udg::RayCasting   MethodType;    

    RayCastingDirector(udg::QApplicationMainWindow* app, QObject *parent = 0, const char *name = 0);
    ~RayCastingDirector();

    void setParameters( RayCastingParameters* parameters );
    
    
public slots:
    /**
        Aquest mtode aplica el mtode de visualitzaci. Comprova primer si t els parmetres
        necessaris, sin retornar fals. Slot pblic heredat de Director.
    */
    void execute();    
 
private:

    MethodType *m_method;    
    
    RayCastingParameters *m_parameters;    
    udg::QApplicationMainWindow* m_applicationWindow;

    ///Obt el volum 
    vtkVolume* getVolume(); 

    ///Obt el mapper del ray casting (utilitzat per el vtkWidgetBox)
    vtkVolumeRayCastMapper* getMapper();

};

};

#endif
