/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#ifndef UDGRAYCASTINGPARAMETERS_H
#define UDGRAYCASTINGPARAMETERS_H

#include "parameters.h"
#include "identifier.h"
#include "qtable.h"

namespace udg {

/**
    Sub-classe de Parameters. En aquest cas encapsula els parmetres del mtode de visualitzaci
    de Ray Casting.
    
    Tindrem un mtode set/get per tots i cadascun dels parmetres del mtode.
    Els mtodes set emetran el signal changed amb l'id de parmetre que hem creat amb el tipus
    enumerat
*/

class RayCastingParameters : public Parameters
{
Q_OBJECT
public:
    
    /**
        Aquest tipus enumerat ens serveix per crear els identificadors de cada parmetre.
    */
     enum RayCastingParametersNames { VolumeId, Interpolation, Distance, TransferFunction };

    RayCastingParameters(QObject *parent = 0, const char *name = 0);

    ~RayCastingParameters();

    
    void setVolumeId( udg::Identifier id)
    {   m_volumeId= id;	emit changed(VolumeId); }
    
    void setInterpolation(int interpolation)
    {  	m_interpolation=interpolation;	emit changed(Interpolation); }
    
    void setDistance(double distance)
    {	m_distance=distance; emit changed(Distance); }

    void setTransferFunction(QTable* t)
    {	m_transferFunction=t; emit changed(TransferFunction); }

    udg::Identifier getVolumeId() const
    { 	return m_volumeId;}
    
    int getInterpolation() const
    {  	return m_interpolation; }
    
    double getDistance() const
    { 	return m_distance; }

    QTable* getTransferFunction() const
    {	return m_transferFunction; }


 private:

    //Identificador del volum de dades amb el que treballem
    udg::Identifier m_volumeId;
    
    //Parmetres del mtode
    int m_interpolation; 
    double m_distance;
    QTable* m_transferFunction;
};

};

#endif
