#include "shapedetectiondirector.h"
#include "shapedetectionparameters.h"

#include "volumerepository.h"

// per la finestra que obrim com a resultat
#include "qwidgetvolume.h"

// per tenir accs a l'aplication window 
#include "qapplicationmainwindow.h"

#include "viewresultswidget.h"

// per debug
#include <iostream.h>

#include <time.h>

namespace udg {

ShapeDetectionDirector::ShapeDetectionDirector(udg::QApplicationMainWindow* app, QObject *parent, const char *name):Director(app, parent, name)
{
    m_applicationWindow = app;
    m_parameters = 0;
    m_method = new MethodType;    
}


ShapeDetectionDirector::~ShapeDetectionDirector()
{
    delete m_method;    
}



void ShapeDetectionDirector::setParameters( ShapeDetectionParameters* parameters )
{
	m_parameters = parameters;
}


void ShapeDetectionDirector::execute()
{
 
    if( !m_parameters )
    {
        // Si no hi ha parmetres establerts no podem fer res
        // \TODO exception, cerr,... ?
        std::cerr << "ShapeDetectionDirector::No hi ha parametres establerts" << std::endl;
    }
    else
    {	
        // obtenim el repositori
        udg::VolumeRepository* repository = udg::VolumeRepository::getRepository();

        // obtenim el volum a treballar
        Volume* volum = repository->getVolume( m_parameters->getVolumeId() );

        //Ccul del temps d'execuci
        time_t start,end;
        double dif;
        time (&start);
        cout << "Shape Director: 1" << endl;
        // li indiquem al mtode els parmetres necessaris
        m_method->setInputVolume( volum );  cout << "Shape Director: 2" << endl;      
        m_method->setSmoothingParameters(); cout << "Shape Director: 3" << endl;        
        m_method->addSeeds(m_parameters->getSeedList(), m_parameters->getDistance());   cout << "Shape Director: 4" << endl;
        m_method->setParameters( m_parameters->getSigma(), m_parameters->getAlpha(), m_parameters->getBeta(), m_parameters->getCurvature(), m_parameters->getPropagation());
        cout << "Shape Director: 5" << endl;
        if( m_method->applyMethod() )
        {   cout << "Shape Director: 6" << endl;
            ViewResultsWidget* viewResults = new ViewResultsWidget(m_applicationWindow, "SegmentedVolumeWidget"); cout << "Shape Director: 7" << endl;           
            viewResults->setInputImage( this->getSegmentedVolume(),this->getBinaryVolume() );  cout << "Shape Director: 8" << endl;
            m_applicationWindow->addWorkingAreaWidget(viewResults, tr("Segmented Volume")); cout << "Shape Director: 9" << endl;           
            viewResults->show(); cout << "Shape Director: 10" << endl;

            time (&end);
            dif = difftime (end,start);
            int minuts=dif/60;
            int segons=dif-(minuts*60);
            cout <<"----------------------------------------" << endl;
            cout <<"Temps Execucio Shape Detection: " << minuts << " min. " << segons << " seg. " << endl;
            cout <<"----------------------------------------" << endl;
	}

        else
        {
            std::cerr << " No s'ha pogut aplicar el mtode de segmentaci" << std::endl;
        }
    }
}



udg::Volume* ShapeDetectionDirector::getSegmentedVolume()
{
	return m_method->getSegmentedVolume();
}

udg::Volume* ShapeDetectionDirector::getBinaryVolume()
{
	return m_method->getBinaryVolume();
}


}; //end namespace udg
