#ifndef UDGSHAPEDETECTIONDIRECTOR_H
#define UDGSHAPEDETECTIONDIRECTOR_H

#include "director.h"
#include "volume.h"
#include "shapedetection.h"

namespace udg {

//Fordware declaration
class ShapeDetectionParameters;


/**
    Sub-classe de Director. En aquest cas fa de pont amb un mtode de segmentaci anomenat
    Shape Detection.
    
    Aquesta inclou un punter a la classe de parmetres especfics del seu mtode, la classe que 
    encapsula el mtode de segmentaci ( ShapeDetection )
    
    Totes les classes heredades de Director, inclouran un mtode setParameters on l'indicarem el 
    punter a la classe de parmetres corresponent. Tamb implementaran l'slot virtual apply, 
    on es far la feina ms important de l'execuci d'un mtode
    
*/


class ShapeDetectionDirector : public Director
{
Q_OBJECT
public:

    typedef udg::Volume::ItkImageType ImageType;
    typedef udg::ShapeDetection   MethodType;    

    ShapeDetectionDirector(udg::QApplicationMainWindow* app, QObject *parent = 0, const char *name = 0);
    ~ShapeDetectionDirector();

    void setParameters( ShapeDetectionParameters* parameters );
    
    
public slots:
    /**
        Aquest mtode aplica el mtode de segmentaci. Comprova primer si t els parmetres
        necessaris, sin retornar fals. Slot pblic heredat de Director.
    */
    void execute();    
 
private:

    MethodType *m_method;    
    
    ShapeDetectionParameters *m_parameters;    
    udg::QApplicationMainWindow* m_applicationWindow;

    ///Obt el volum segmentat
    udg::Volume* getSegmentedVolume();

    ///Obt el volum binari segmentat
    udg::Volume* getBinaryVolume();  

};

};

#endif
