/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#ifndef UDGSHAPEDETECTIONPARAMETERS_H
#define UDGSHAPEDETECTIONPARAMETERS_H

#include "parameters.h"
#include "identifier.h"
#include "qlistview.h"

namespace udg {

/**
    Sub-classe de Parameters. En aquest cas encapsula els parmetres del mtode de segmentaci
    ShapeDetection.
    
    Tindrem un mtode set/get per tots i cadascun dels parmetres del mtode.
    Els mtodes set emetran el signal changed amb l'id de parmetre que hem creat amb el tipus
    enumerat
*/

class ShapeDetectionParameters : public Parameters
{
Q_OBJECT
public:
    
    /**
        Aquest tipus enumerat ens serveix per crear els identificadors de cada parmetre.
    */
     enum ShapeDetectionParametersNames { VolumeId, Distance, Sigma, Alpha, Beta, Curvature, Propagation, SeedList };

    ShapeDetectionParameters(QObject *parent = 0, const char *name = 0);

    ~ShapeDetectionParameters();

    
    void setVolumeId( udg::Identifier id)
    {   m_volumeId= id;	emit changed(VolumeId); }
    
    void setDistance(double distance)
    {  	m_distance=distance; emit changed(Distance); }

    void setSigma(double sigma)
    {  	m_sigma=sigma;	emit changed(Sigma); }
    
    void setAlpha(double alpha)
    {	m_alpha=alpha;	emit changed(Alpha); }

    void setBeta(double beta)
    {  	m_beta=beta;	emit changed(Beta); }
    
    void setCurvature(double curvature)
    {	m_curvature=curvature;	emit changed(Curvature); }

    void setPropagation(double propagation)
    {  	m_propagation=propagation;  emit changed(Propagation); }

    void setSeedList(QListView* l)
    {	m_seedList=l; emit changed(SeedList); }

    udg::Identifier getVolumeId() const
    { 	return m_volumeId;}
    
    double getDistance() const
    {  	return m_distance; }

    double getSigma() const
    {  	return m_sigma; }
    
    double getAlpha() const
    { 	return m_alpha; }
    
    double getBeta() const
    {  	return m_beta; }
    
    double getCurvature() const
    { 	return m_curvature; }

    double getPropagation() const
    {  	return m_propagation; }

    QListView* getSeedList() const
    {	return m_seedList; }


 private:

    //Identificador del volum de dades amb el que treballem
    udg::Identifier m_volumeId;
    
    //Parmetres del mtode
    double m_distance, m_sigma, m_alpha, m_beta, m_curvature, m_propagation;
    QListView* m_seedList;
};

};

#endif
