#include "viewresultswidget.h"
#include <qslider.h>
#include <qspinbox.h>
#include <qlineedit.h> 

#include "volume.h"
#include "volumecalculator.h"

#include "qcursor.h"

namespace udg {

ViewResultsWidget::ViewResultsWidget(QWidget *parent, const char *name)
 : ViewResultsWidgetBase(parent, name)
{
    m_numView=0; //per defecte la vista s la X
    m_applicationWindow=(QApplicationMainWindow*)parent;
}


ViewResultsWidget::~ViewResultsWidget()
{
}


void ViewResultsWidget::setInputImage(Volume* segmentedVolume,Volume* binaryVolume)
{
    m_volume=segmentedVolume; //Aquest s el volum amb els valors originals
    m_binaryVolume=binaryVolume; //Aquest s el volum amb binari (1 els segmentats i 0 els no)

    //****Utilitzat per calcular el volum total i el volum de la llesca****
    char* tempchar = new char[20];
    char* percentatge = new char[20];

    m_spacing[0] = ((ImageType*)(m_binaryVolume->getItkData()))->GetSpacing()[0];//x 
    m_spacing[1] = ((ImageType*)(m_binaryVolume->getItkData()))->GetSpacing()[1];//y
    m_spacing[2] = ((ImageType*)(m_binaryVolume->getItkData()))->GetSpacing()[2];//z

    //cout << "m_spacing: " << m_spacing[0] << " " << m_spacing[1] << " " << m_spacing[2] << endl;

    m_xVolumeCalculator = new VolumeCalculator((ImageType*)(m_binaryVolume->getItkData()),0);
    //m_yVolumeCalculator = new VolumeCalculator((ImageType*)(m_volume->getItkData()),1);
    //m_zVolumeCalculator = new VolumeCalculator((ImageType*)(m_volume->getItkData()),2);

    //el volum total en mm cubics. El volum total tenen tots 3 el mateix i agafarem per exemple
    //el m_xVolumeCalculator. El que variar entre ells sn les reas de les llesques.
    sprintf(tempchar,"%.2f",m_xVolumeCalculator->getTotalVolume()*(m_spacing[0]*m_spacing[1]*m_spacing[2]));

    sprintf(percentatge,"%.4f",((m_xVolumeCalculator->getTotalVolume()*(m_spacing[0]*m_spacing[1]*m_spacing[2]))/6278385)*100);

    this->m_txtTotalVolume->setText(tr(tempchar));
    this->m_txtPercentatge->setText(tr(percentatge));
    //****Fi clculs volums****

    this->m_view->setVolume(m_volume);
    this->showView();
}



void ViewResultsWidget::slotSliceArea(int slice)
{ 
    char* tempchar = new char[20];
    float res;
    switch(m_numView)
    {
        case 0:
            res = m_spacing[1]*m_spacing[2];
            //el volum de la llesca en mm quadrats    
            sprintf(tempchar,"%.2f",m_xVolumeCalculator->getSliceArea(slice)*res);
        break;
        
        case 1:
            res = m_spacing[0]*m_spacing[2];
            //el volum de la llesca en mm quadrats    
            //sprintf(tempchar,"%.2f",m_yVolumeCalculator->getSliceArea(slice)*res);
            tempchar="";
        break;
    
        case 2:
            res = m_spacing[0]*m_spacing[1];
            //el volum de la llesca en mm quadrats    
            //sprintf(tempchar,"%.2f",m_zVolumeCalculator->getSliceArea(slice)*res);
            tempchar="";
        break;
    }

    this->m_txtSliceArea->setText(tr(tempchar));
}


void ViewResultsWidget::showView()
{   
    int *size = m_binaryVolume->getVtkData()->GetDimensions();    
    switch(m_numView)
    {
        case 0:
             this->m_txtSliceArea->setEnabled(true);
             this->m_view->setView( QSimpleSliceViewer::Axial ); 
             this->m_axisSlider->setMaxValue( size[2]-1 );
             this->m_axisSpinBox->setMaxValue( size[2]-1 ); 
             this->m_axisSpinBox->setValue( size[2]/2 );
        break;

        case 1:
            this->m_txtSliceArea->setEnabled(false);
            this->m_view->setView( QSimpleSliceViewer::Sagittal );
            this->m_axisSlider->setMaxValue( size[1]-1 );
            this->m_axisSpinBox->setMaxValue( size[1]-1 );
            this->m_axisSpinBox->setValue( size[1]/2 );
        break;

        case 2:
            this->m_txtSliceArea->setEnabled(false);
            this->m_view->setView( QSimpleSliceViewer::Coronal );
            this->m_axisSlider->setMaxValue( size[0]-1 );
            this->m_axisSpinBox->setMaxValue( size[0]-1 );
            this->m_axisSpinBox->setValue( size[0]/2 );
        break;
    }


}


void ViewResultsWidget::slotChangeView(int i)
{
    m_numView=i;
    this->showView();
}


void ViewResultsWidget::slot3DView()
{ 
    //Posem el cursor en mode espera (indicant execuci)
    QCursor cursor(WaitCursor);
    setCursor(cursor);

    ViewResultsRayCastingWidget* viewResults = new ViewResultsRayCastingWidget(m_applicationWindow, "3DViewWidget");
    QColor rgb=m_qwColorButton->getColor();
    viewResults->rayCasting(this->m_volume,rgb);
    m_applicationWindow->addWorkingAreaWidget(viewResults, tr("3D View"));
    viewResults->show();

    //Posem el cursor en mode normal
    QCursor cursor2(ArrowCursor);
    setCursor(cursor2);
}


void ViewResultsWidget::slotMultiplane()
{
    QWidgetVolume* imagePlanes = new QWidgetVolume();
    imagePlanes->setVolume( this->m_volume );
    m_applicationWindow->addWorkingAreaWidget( imagePlanes, tr("3D Image Planes"));
    imagePlanes->show();
}



};
