/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/


#ifndef UDGVOLUME_H
#define UDGVOLUME_H


#include <itkImage.h>

// Els filtres per passar itk<=>vtk: InsightApplications/auxiliary/vtk --> ho tenim a /tools
#include "itkImageToVTKImageFilter.h" //Converts an ITK image into a VTK image and plugs a itk data pipeline to a VTK datapipeline.   
#include "itkVTKImageToImageFilter.h" // Converts a VTK image into an ITK image and plugs a vtk data pipeline to an ITK datapipeline.   

class vtkImageData;

namespace udg {

/**
    Aquesta classe respresenta un volum de dades. Aquesta ser la classe on es guardaran les dades que voldrem tractar. Ens donar mtodes
    per poder interrogar-la i obtenir les dades en el format que volguem: ITK, VTK, etc.
    
    @author Grup de Grfics de Girona  ( GGG )
*/

class Volume
{

public:

    // Tipus d'imatge intern per defecte d'itk
    typedef signed int ItkPixelType;    
    static const signed int VDimension = 3;
    
    typedef itk::Image<ItkPixelType, VDimension >    ItkImageType;
    typedef ItkImageType::Pointer  ItkImageTypePointer;
    typedef ItkImageType::RegionType   RegionType;
    typedef ItkImageType::SizeType     SizeType;
    typedef ItkImageType::IndexType    IndexType;

    
    // tipus de punter de dades vtk
    typedef vtkImageData* VtkImageTypePointer;

    // Filtres per importar/exportar
    typedef itk::ImageToVTKImageFilter< ItkImageType > ItkToVtkFilterType;
    typedef itk::VTKImageToImageFilter< ItkImageType > VtkToItkFilterType;
    
    //typedef vtkMetaImageReader* ImageReaderType;
    
    Volume();
    ~Volume();

    /**
       Li donem les dades en format ITK
       \sa getVtkData(), getItkData()
    */
    void setData( ItkImageTypePointer itkImage );
 
    /**
       Li donem les dades en format VTK
       \sa getVtkData(), getItkData()
    */
    void setData( VtkImageTypePointer vtkImage );
    
    /**
        Ens retorna la dades en format VTK
        \sa setData()
    */
    VtkImageTypePointer getVtkData();

    /**
        Ens retorna les dades en format ITK
        \sa setData()
    */
    ItkImageTypePointer getItkData();
    
    //Volume & operator = ( const Volume & volume );
    
private:
    
    ItkImageTypePointer m_imageDataITK;
    VtkImageTypePointer m_imageDataVTK;
    
    ItkToVtkFilterType::Pointer m_itkToVtkFilter;
    VtkToItkFilterType::Pointer m_vtkToItkFilter;
};

};  // end namespace udg



#endif
