/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/

#ifndef UDGVOLUMEREPOSITORY_H
#define UDGVOLUMEREPOSITORY_H

#include "identifier.h"

namespace udg {

class Volume;

/**
    Aquesta classe s el repositori de volums. En aquesta classe es guarden tots els volums que hi ha oberts durant
    l'execuci del programa. Noms hi haura una sola instncia en tota la vida del programa d'aquesta classe. 
    Per fer-ho s'aplica el patr Singleton.
    
    Per poder obtenir una instncia del repositori hem de fer un include del fitxer volumerepository.h i fer una crida 
    al mtode VolumeRepository::getRepository(). Aquest ens retornar un punter al repositori de volums.
    
    Exemple:
    
    \code
    #include "volumerepository.h"
    ...
    udg::VolumeRepository* m_volumeRepository;
    m_volumeRepository = VolumeRepository::getRepository();
    ...
    Volume* m_volume = m_volumeRepository->getVolume(id);    
    \endcode
    
    @author GGG
*/

class VolumeRepository
{
public:

    /// El destructor allibera l'espai ocupat pels volums
    ~VolumeRepository();
    
    /**
        Afegeix un volum al repositori.
        Ens retorna l'id del volum afegit per poder-lo obtenir ms endavant.
        \TODO \warning En aquests moments el repositori noms pot contenir dos volums.
    */    
    Identifier addVolume( Volume *model ); 
    
    /// Ens retorna un volum del repositori amb l'identificador que especifiquem.
    Volume *getVolume( Identifier id );
    /// Elimina un model del repositori
    void removeVolume( Identifier id );
    /// Retorna el nombre de volums que hi ha al repositori
    int count();
    
    /// Ens retorna l'nica instncia del repositori.
    static VolumeRepository* getRepository()
    {   
        static VolumeRepository repository;
        return &repository;
    }

private:
    // \TODO Provisionalment el repositori tan sols tindra 2 volums com a molt Ms endavant tindrem una llista ordenada, llista hash o algo per l'estil
    Volume *m_volume1, *m_volume2; 
            
    VolumeRepository(); // no implementat expressament
};

};  



#endif
