/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#include "confidenceconnected.h"
#include "volume.h"
#include <iostream.h>

namespace udg {

ConfidenceConnected::ConfidenceConnected()
{
    m_inputVolume       = new udg::Volume();
    m_segmentedImage    = VolumeImageType::New();

    m_smoothing         = CurvatureFlowImageFilterType::New();
    m_confidenceConnected= ConfidenceConnectedImageFilterType::New();
}

//destructor
ConfidenceConnected::~ConfidenceConnected()
{
}

/* En aquest mtode es pot treballar amb ms d'una llavor, per aquest motiu afegirem 
   les llavors de la llista al vector de llavors, l'atribut m_seedsVector.    
*/
void ConfidenceConnected::addSeeds( QListView* l )
{
    double x,y,z;
    QListViewItemIterator it(l);
    m_seedsVector.clear();

    while  ( it.current() ) //Mentre hi hagin elements
    {   
        QListViewItem *item = it.current();
        x = (item->text(0)).toDouble();
	y = (item->text(1)).toDouble();
        z = (item->text(2)).toDouble();

        std::cout<<"[ConfidenceConnected] addSeeds: "<< x <<" " << y << " " << z <<std::endl;
        VolumeImageIndexType seed;
        seed[0] = x;
        seed[1] = y;
        seed[2] = z;
        m_seedsVector.push_back( seed );

        ++it;
    }
}


// Carrega el volum a tractar
void ConfidenceConnected::setInputVolume( Volume* v )
{
    m_inputVolume = v;
}


/* Carrega els parmetres necessaris per configurar el filtre de suavitzaci
   ( iteracions i TimeStep )*/
void ConfidenceConnected::setSmoothingParameters( )
{
    m_smoothing->SetInput( m_inputVolume->getItkData() );
    m_smoothing->SetNumberOfIterations( 2 );
    m_smoothing->SetTimeStep( 0.125 );
}


/* Carrega els parmetres necessaris per configurar el filtre de segmentaci
   ( llavors, finestra inicial, iteracions, marge d'acceptaci )*/
void ConfidenceConnected::setParameters( double factor, int iterations, double radius )
{
    std::cout << "[ConfidenceConnected] setConnectedParameters: factor: " << factor << std::endl;
    m_confidenceConnected->SetMultiplier(factor);
    
    std::cout << "[ConfidenceConnected] setConnectedParameters: iterations: " << iterations << std::endl;
    m_confidenceConnected->SetNumberOfIterations(iterations);

    m_confidenceConnected->SetReplaceValue( 255 );
    m_confidenceConnected->SetSeed( m_seedsVector[0] );
    std::cout<<"[ConfidenceConnected] setConnectedParameters llavor 0: "<<m_seedsVector[0]<< std::endl;
    for (unsigned int i=1; i<m_seedsVector.size(); i++)
    {
        m_confidenceConnected->AddSeed( m_seedsVector[i] );
        std::cout<<"[ConfidenceConnected] setConnectedParameters llavor " << i << ": " <<m_seedsVector[i]<< std::endl;
    }

    m_confidenceConnected->SetInitialNeighborhoodRadius(radius);
}


// Aplica el metode de segmentaci
bool ConfidenceConnected::applyMethod()
{
    // Crea la PIPELINE
    m_confidenceConnected->SetInput( m_smoothing->GetOutput() );
    m_segmentedImage = m_confidenceConnected->GetOutput();

    // Executa la segmentaci
    try
    {
        m_segmentedImage->Update();
    }
    catch( itk::ExceptionObject & e )
    {
        std::cerr << "ConfidenceConnected: Error en la segmentaci!" << std::endl;
        std::cerr << "Exception caught! " << std::endl;
        std::cerr << e << std::endl;
        return false;
    }
 
    //-------------------- Resultats ------------------

    return true; 
}


// Retorna el volum segmentat
udg::Volume* ConfidenceConnected::getSegmentedVolume()
{
    m_mask=MaskFilterType::New();
    m_mask->SetInput1(m_inputVolume->getItkData());
    m_mask->SetInput2(m_segmentedImage);

    udg::Volume* volume = new udg::Volume();
    volume->setData( m_mask->GetOutput() );
    return volume; 
}

udg::Volume* ConfidenceConnected::getBinaryVolume()
{
    udg::Volume* volume = new udg::Volume();
    volume->setData( m_segmentedImage );
    return volume; 
}

};
