/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#include "connectedthreshold.h"
#include "volume.h"
#include <iostream.h>

namespace udg {

ConnectedThreshold::ConnectedThreshold()
{
    m_inputVolume       = new udg::Volume();
    m_segmentedImage    = VolumeImageType::New();

    m_smoothing         = CurvatureFlowImageFilterType::New();
    m_connectedThreshold= ConnectedThresholdImageFilterType::New();
}

//destructor
ConnectedThreshold::~ConnectedThreshold()
{
}

/* En aquest mtode es pot treballar amb ms d'una llavor, per aquest motiu afegirem 
   les llavors de la llista al vector de llavors, l'atribut m_seedsVector.    
*/
void ConnectedThreshold::addSeeds( QListView* l )
{
    double x,y,z;
    QListViewItemIterator it(l);
    m_seedsVector.clear();

    while  ( it.current() ) //Mentre hi hagin elements
    {   
        QListViewItem *item = it.current();
        x = (item->text(0)).toDouble();
	y = (item->text(1)).toDouble();
        z = (item->text(2)).toDouble();

        std::cout<<"[ConnectedThreshold] addSeeds: "<< x <<" " << y << " " << z <<std::endl;
        VolumeImageIndexType seed;
        seed[0] = x;
        seed[1] = y;
        seed[2] = z;
        m_seedsVector.push_back( seed );

        ++it;
    }

    
}


// Carrega el volum a tractar
void ConnectedThreshold::setInputVolume( Volume* v )
{
    m_inputVolume = v;
}


/* Carrega els parmetres necessaris per configurar el filtre de suavitzaci
   ( iteracions i TimeStep )*/
void ConnectedThreshold::setSmoothingParameters( )
{
    m_smoothing->SetInput( m_inputVolume->getItkData() );
    m_smoothing->SetNumberOfIterations( 2 );
    m_smoothing->SetTimeStep( 0.125 );

}


/* Carrega els parmetres necessaris per configurar el filtre de segmentaci
  */
void ConnectedThreshold::setParameters( int lower, int upper )
{
    std::cout << "[ConnectedThreshold] setParameters: lower: " << lower << " upper: " << upper << std::endl;
    m_connectedThreshold->SetLower( lower );
    m_connectedThreshold->SetUpper( upper );
    m_connectedThreshold->SetReplaceValue( 255 );
   
    m_connectedThreshold->SetSeed( m_seedsVector[0] );
    std::cout<<"[ConnectedThreshold] setParameters llavor 0: "<<m_seedsVector[0]<< std::endl;
    for (unsigned int i=1; i<m_seedsVector.size(); i++)
    {
        m_connectedThreshold->AddSeed( m_seedsVector[i] );
        std::cout<<"[ConnectedThreshold] setParameters llavor " << i << ": " <<m_seedsVector[i]<< std::endl;
    }
}


// Aplica el metode de segmentaci
bool ConnectedThreshold::applyMethod()
{
    // Crea la PIPELINE
    m_connectedThreshold->SetInput( m_smoothing->GetOutput() );
    m_segmentedImage = m_connectedThreshold->GetOutput();

    // Executa la segmentaci
    try
    {
        m_segmentedImage->Update();
    }
    catch( itk::ExceptionObject & e )
    {
        std::cerr << "ConnectedThreshold: Error en la segmentaci!" << std::endl;
        std::cerr << "Exception caught! " << std::endl;
        std::cerr << e << std::endl;
        return false;
    }
    //-------------------- Resultats ------------------

    return true;  
}


// Retorna el volum segmentat
udg::Volume* ConnectedThreshold::getSegmentedVolume()
{
    m_mask=MaskFilterType::New();
    m_mask->SetInput1(m_inputVolume->getItkData());
    m_mask->SetInput2(m_segmentedImage);

    udg::Volume* volume = new udg::Volume();
    volume->setData( m_mask->GetOutput() );

    return volume; 
}


udg::Volume* ConnectedThreshold::getBinaryVolume()
{
    udg::Volume* volume = new udg::Volume();
    volume->setData( m_segmentedImage );
    return volume; 
}

};
