/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#include "fastmarching.h"
#include "volume.h"
#include <iostream.h>

namespace udg {

FastMarching::FastMarching()
{
    m_inputVolume       = new udg::Volume();
    m_segmentedImage    = VolumeImageType::New();
    //m_casterInput       = CastingFilterTypeIn::New();

    m_smoothing         = SmoothingFilterType::New();
    m_gradientMagnitude = GradientFilterType::New();
    m_sigmoid           = SigmoidFilterType::New();
    m_fastMarching      = FastMarchingFilterType::New();
    m_thresholder       = ThresholdingFilterType::New();

    m_seeds             = NodeContainer::New();
}

//destructor
FastMarching::~FastMarching()
{
}

/* En aquest mtode es pot treballar amb ms d'una llavor, per aquest motiu afegirem 
   les llavors de la llista al vector de llavors. 
*/
void FastMarching::addSeeds( QListView* l )
{
    double x,y,z,value;
    QListViewItemIterator it(l);
    m_seeds->Initialize();
    int aux = 0;

    while  ( it.current() ) //Mentre hi hagin elements
    {   
        QListViewItem *item = it.current();
        x = (item->text(0)).toDouble();
	y = (item->text(1)).toDouble();
        z = (item->text(2)).toDouble();
        value = (item->text(3)).toDouble();
        std::cout<<"[FastMarching] addSeeds: "<< x <<" " << y << " " << z <<std::endl;

        m_seedPosition[0] = x;
        m_seedPosition[1] = y;
        m_seedPosition[2] = z;        
        m_node.SetValue( 0.0 );
        m_node.SetIndex( m_seedPosition );
        m_seeds->InsertElement( aux, m_node );

        ++it;
        aux++;
    }
}


// Carrega el volum a tractar
void FastMarching::setInputVolume( Volume* v )
{
    m_inputVolume = v;
}


/* Carrega els parmetres necessaris per configurar el filtre de suavitzaci
   ( iteracions i TimeStep )*/
void FastMarching::setSmoothingParameters( )
{
    //m_casterInput->SetInput( m_inputVolume->getItkData() );
    //m_smoothing->SetInput( m_casterInput->GetOutput() );
    m_smoothing->SetInput( m_inputVolume->getItkData() );
    m_smoothing->SetTimeStep( 0.0625 );
    m_smoothing->SetNumberOfIterations(  5 );
    m_smoothing->SetConductanceParameter( 9.0 );
}


/* Carrega els parmetres necessaris per configurar el filtre de segmentaci
*/
void FastMarching::setParameters( double sigma, double alpha, double beta, int threshold, int stop )
{
   m_gradientMagnitude->SetInput( m_smoothing->GetOutput() );
   m_gradientMagnitude->SetSigma(  sigma  );
   std::cout << "[FastMarching] setParameters: sigma: " << sigma << std::endl;

   m_sigmoid->SetInput( m_gradientMagnitude->GetOutput() );
   m_sigmoid->SetOutputMinimum(  0.0  );
   m_sigmoid->SetOutputMaximum(  1.0  );
   m_sigmoid->SetAlpha( alpha );
   m_sigmoid->SetBeta(  beta  );
   std::cout << "[FastMarching] setParameters: alpha: " << alpha << std::endl;
   std::cout << "[FastMarching] setParameters: beta: " << beta << std::endl;

   m_fastMarching->SetInput( m_sigmoid->GetOutput() );
   m_fastMarching->SetTrialPoints(  m_seeds  );
   m_fastMarching->SetOutputSize( m_inputVolume->getItkData()->GetBufferedRegion().GetSize() );
   m_fastMarching->SetStoppingValue(  stop  );
   std::cout << "[FastMarching] setParameters: stop: " << stop << std::endl;

   m_thresholder->SetInput( m_fastMarching->GetOutput() );
   m_thresholder->SetLowerThreshold( 0.0  );
   m_thresholder->SetUpperThreshold( threshold  );
   m_thresholder->SetOutsideValue(  0  );
   m_thresholder->SetInsideValue(  255 );
   std::cout << "[FastMarching] setParameters: threshold: " << threshold << std::endl;
}


// Aplica el metode de segmentaci
bool FastMarching::applyMethod()
{
    // Crea la PIPELINE
    m_segmentedImage = m_thresholder->GetOutput();

    // Executa la segmentaci
    try
    {
        m_segmentedImage->Update();
    }
    catch( itk::ExceptionObject & e )
    {
        std::cerr << "FastMarching: Error en la segmentaci!" << std::endl;
        std::cerr << "Exception caught! " << std::endl;
        std::cerr << e << std::endl;
        return false;
    }
 
    //-------------------- Resultats ------------------

    return true; 
}


// Retorna el volum segmentat
udg::Volume* FastMarching::getSegmentedVolume()
{
    m_mask=MaskFilterType::New();
    m_mask->SetInput1(m_inputVolume->getItkData());
    m_mask->SetInput2(m_segmentedImage);

    udg::Volume* volume = new udg::Volume();
    volume->setData( m_mask->GetOutput() );
    return volume; 
}

udg::Volume* FastMarching::getBinaryVolume()
{
    udg::Volume* volume = new udg::Volume();
    volume->setData( m_segmentedImage );
    return volume; 
}
 
};
