/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#include "geodesicactive.h"
#include "volume.h"
#include <iostream.h>

namespace udg {

GeodesicActive::GeodesicActive()
{
    m_inputVolume       = new udg::Volume();
    m_segmentedImage    = VolumeImageType::New();
    //m_casterInput       = CastingFilterTypeIn::New();

    m_smoothing         = SmoothingFilterType::New();
    m_gradientMagnitude = GradientFilterType::New();
    m_sigmoid           = SigmoidFilterType::New();
    m_fastMarching      = FastMarchingFilterType::New();
    m_geodesicActiveContour = GeodesicActiveContourFilterType::New();
    m_thresholder       = ThresholdingFilterType::New();

    //m_caster            = CastingFilterType::New();

    m_seeds             = NodeContainer::New();
}

//destructor
GeodesicActive::~GeodesicActive()
{
}

/* En aquest mtode es pot treballar amb ms d'una llavor, per aquest motiu afegirem 
   les llavors de la llista al vector de llavors. 
*/
void GeodesicActive::addSeeds( QListView* l, double distance )
{
    double x,y,z;
    QListViewItemIterator it(l);
    m_seeds->Initialize();
    int aux = 0;

    while  ( it.current() ) //Mentre hi hagin elements
    {   
        QListViewItem *item = it.current();
        x = (item->text(0)).toDouble();
	y = (item->text(1)).toDouble();
        z = (item->text(2)).toDouble();
        std::cout<<"[GeodesicActive] addSeeds: "<< x <<" " << y << " " << z <<std::endl;

        m_seedPosition[0] = x;
        m_seedPosition[1] = y;
        m_seedPosition[2] = z;
        m_node.SetValue( - distance );
        m_node.SetIndex( m_seedPosition );
        m_seeds->InsertElement( aux, m_node );

        ++it;
        aux++;
    }
}


// Carrega el volum a tractar
void GeodesicActive::setInputVolume( Volume* v )
{
    m_inputVolume = v;
}


/* Carrega els parmetres necessaris per configurar el filtre de suavitzaci
   ( iteracions i TimeStep )*/
void GeodesicActive::setSmoothingParameters( )
{
    //m_casterInput->SetInput( m_inputVolume->getItkData() );
    //m_smoothing->SetInput( m_casterInput->GetOutput() );
    m_smoothing->SetInput( m_inputVolume->getItkData() );
    m_smoothing->SetTimeStep( 0.0625 );
    m_smoothing->SetNumberOfIterations(  5 );
    m_smoothing->SetConductanceParameter( 9.0 );


    /*CastFilterType::Pointer caster1 = CastFilterType::New();
    WriterType::Pointer writer1 = WriterType::New();
    caster1->SetInput( m_smoothing->GetOutput() );
    writer1->SetInput( caster1->GetOutput() );
    writer1->SetFileName("/home/biru/Desktop/VisualPork/visualpork-qt3-2006.0/src/segmentation/GeodesicActiveContourImageFilterOutput1.png");
    caster1->SetOutputMinimum(   0 );
    caster1->SetOutputMaximum( 255 );
    writer1->Update();*/
}


/* Carrega els parmetres necessaris per configurar el filtre de segmentaci
*/
void GeodesicActive::setParameters( double sigma, double alpha, double beta, double propagation )
{
   m_gradientMagnitude->SetInput( m_smoothing->GetOutput() );
   m_gradientMagnitude->SetSigma(  sigma  );
   std::cout << "[GeodesicActive] setParameters: sigma: " << sigma << std::endl;

        /*CastFilterType::Pointer caster2 = CastFilterType::New();
        WriterType::Pointer writer2 = WriterType::New();
        caster2->SetInput( m_gradientMagnitude->GetOutput() );
        writer2->SetInput( caster2->GetOutput() );
        writer2->SetFileName("/home/biru/Desktop/VisualPork/visualpork-qt3-2006.0/src/segmentation/GeodesicActiveContourImageFilterOutput2.png");
        caster2->SetOutputMinimum(   0 );
        caster2->SetOutputMaximum( 255 );
        writer2->Update();*/

   m_sigmoid->SetInput( m_gradientMagnitude->GetOutput() );
   m_sigmoid->SetOutputMinimum(  0.0  );
   m_sigmoid->SetOutputMaximum(  1.0  );
   m_sigmoid->SetAlpha( alpha );
   m_sigmoid->SetBeta(  beta  );
   std::cout << "[GeodesicActive] setParameters: alpha: " << alpha << std::endl;
   std::cout << "[GeodesicActive] setParameters: beta: " << beta << std::endl;

        /*CastFilterType::Pointer caster3 = CastFilterType::New();
        WriterType::Pointer writer3 = WriterType::New();
        caster3->SetInput( m_sigmoid->GetOutput() );
        writer3->SetInput( caster3->GetOutput() );
        writer3->SetFileName("/home/biru/Desktop/VisualPork/visualpork-qt3-2006.0/src/segmentation/GeodesicActiveContourImageFilterOutput3.png");
        caster3->SetOutputMinimum(   0 );
        caster3->SetOutputMaximum( 255 );
        writer3->Update();*/

   m_fastMarching->SetInput( m_sigmoid->GetOutput() );
   m_fastMarching->SetTrialPoints(  m_seeds  );
   m_fastMarching->SetSpeedConstant( 1.0 );
   m_fastMarching->SetOutputSize( m_inputVolume->getItkData()->GetBufferedRegion().GetSize() );   
    
        /*CastFilterType::Pointer caster4 = CastFilterType::New();
        WriterType::Pointer writer4 = WriterType::New();
        caster4->SetInput( m_fastMarching->GetOutput() );
        writer4->SetInput( caster3->GetOutput() );
        writer4->SetFileName("/home/biru/Desktop/VisualPork/visualpork-qt3-2006.0/src/segmentation/GeodesicActiveContourImageFilterOutput4.png");
        caster4->SetOutputMinimum(   0 );
        caster4->SetOutputMaximum( 255 );
        writer4->Update();*/

   m_geodesicActiveContour->SetInput(  m_fastMarching->GetOutput() );
   m_geodesicActiveContour->SetFeatureImage( m_sigmoid->GetOutput() );
   m_geodesicActiveContour->SetPropagationScaling( propagation );
   m_geodesicActiveContour->SetCurvatureScaling( 1.0 );
   m_geodesicActiveContour->SetAdvectionScaling( 1.0 );
   m_geodesicActiveContour->SetMaximumRMSError( 0.02 );
   m_geodesicActiveContour->SetNumberOfIterations( 800 );
   std::cout << "[GeodesicActive] setParameters: propagation: " << propagation << std::endl;

   //m_caster->SetInput(m_geodesicActiveContour->GetOutput());

   m_thresholder->SetInput( m_geodesicActiveContour->GetOutput() );
   //m_thresholder->SetInput( m_caster->GetOutput() );
   m_thresholder->SetLowerThreshold( -1000.0  );
   m_thresholder->SetUpperThreshold( 0.0  );
   m_thresholder->SetOutsideValue(  0  );
   m_thresholder->SetInsideValue(  255 );
}


// Aplica el metode de segmentaci
bool GeodesicActive::applyMethod()
{
    // Crea la PIPELINE
    m_segmentedImage = m_thresholder->GetOutput();

    // Executa la segmentaci
    try
    {
        m_segmentedImage->Update();
    }
    catch( itk::ExceptionObject & e )
    {
        std::cerr << "GeodesicActive: Error en la segmentaci!" << std::endl;
        std::cerr << "Exception caught! " << std::endl;
        std::cerr << e << std::endl;
        return false;
    }
 
    //-------------------- Resultats ------------------

    return true; 
}


// Retorna el volum segmentat
udg::Volume* GeodesicActive::getSegmentedVolume()
{
    m_mask=MaskFilterType::New();
    m_mask->SetInput1(m_inputVolume->getItkData());
    m_mask->SetInput2(m_segmentedImage);

    udg::Volume* volume = new udg::Volume();
    volume->setData( m_mask->GetOutput() );
    return volume;  
} 

udg::Volume* GeodesicActive::getBinaryVolume()
{
    udg::Volume* volume = new udg::Volume();
    volume->setData( m_segmentedImage );
    return volume; 
}

};
