/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#ifndef UDGGEODESICACTIVE_H
#define UDGGEODESICACTIVE_H

#include <qobject.h>

#include "itkImage.h"
#include <itkCastImageFilter.h>
#include "itkGeodesicActiveContourLevelSetImageFilter.h"
#include "itkCurvatureAnisotropicDiffusionImageFilter.h"
#include "itkGradientMagnitudeRecursiveGaussianImageFilter.h"
#include "itkSigmoidImageFilter.h"
#include "itkFastMarchingImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "itkBinaryThresholdImageFilter.h"
#include <itkMaskImageFilter.h>

//Per tenir accs als tipus d'imatges de volum
#include <vector.h>
#include "volume.h"

#include "qlistview.h"

//class Volume;


#include "itkImageFileWriter.h"

using namespace std;

namespace udg {

/**
@author Grup de Grfics de Girona  ( GGG )
*/

/**
    Aquesta classe, GeodesicActive s un mtode de segmentaci de Level Set. 
    
    Aquesta classe prepara tot el tractament de la imatge, rep tots els parmetres necessaris
    per executar el mtode de segmentaci (imatge a segmentar inclosa) i executa tot el procs de 
    segmentaci.
*/



class GeodesicActive{
private:
 
// *************** ATRIBUTS ***************

    // ----- Definim els tipus d'objectes -----
    
    // Definim un tipus d'imatge, el tipus d'imateg del volum.
    typedef udg::Volume::ItkImageType VolumeImageType;
    // I el tipus dels seus ndexs.
    typedef VolumeImageType::IndexType    VolumeImageIndexType;
    
    typedef float InternalPixelType;
    typedef itk::Image< InternalPixelType,udg::Volume::VDimension > InternalImageType;

     /* Definim tipus de Castings si els filtres necessiten algn tipus d'imatge determinat
       (InternalImageType) han de treballar amb aquest.*/
    /*typedef itk::CastImageFilter< VolumeImageType, InternalImageType >    CastingFilterTypeIn;

    typedef itk::CurvatureAnisotropicDiffusionImageFilter< InternalImageType, InternalImageType > SmoothingFilterType;
    typedef itk::GradientMagnitudeRecursiveGaussianImageFilter< InternalImageType,InternalImageType > GradientFilterType;
    typedef itk::SigmoidImageFilter< InternalImageType,InternalImageType > SigmoidFilterType;
    typedef itk::FastMarchingImageFilter< InternalImageType,InternalImageType >  FastMarchingFilterType;
    typedef itk::GeodesicActiveContourLevelSetImageFilter< InternalImageType,InternalImageType > GeodesicActiveContourFilterType;

    typedef itk::BinaryThresholdImageFilter< InternalImageType, VolumeImageType > ThresholdingFilterType;    
    */

    typedef itk::CurvatureAnisotropicDiffusionImageFilter< VolumeImageType, VolumeImageType > SmoothingFilterType;
    typedef itk::GradientMagnitudeRecursiveGaussianImageFilter< VolumeImageType,VolumeImageType > GradientFilterType;
    typedef itk::SigmoidImageFilter< VolumeImageType,VolumeImageType > SigmoidFilterType;
    typedef itk::FastMarchingImageFilter< VolumeImageType,VolumeImageType >  FastMarchingFilterType;
    typedef itk::GeodesicActiveContourLevelSetImageFilter< VolumeImageType,VolumeImageType > GeodesicActiveContourFilterType;


    //typedef itk::CastImageFilter< InternalImageType, VolumeImageType >    CastingFilterType;

    typedef itk::BinaryThresholdImageFilter< InternalImageType, VolumeImageType > ThresholdingFilterType;    

    typedef FastMarchingFilterType::NodeContainer           NodeContainer;
    typedef FastMarchingFilterType::NodeType                NodeType;
  
    typedef itk::MaskImageFilter< VolumeImageType, VolumeImageType, VolumeImageType > MaskFilterType;

    
    //Castings per obtenir imatges de les diferents etapes del procs
    typedef unsigned char OutputPixelType;
    typedef itk::Image< OutputPixelType,2> OutputImageType;
    
    typedef itk::RescaleIntensityImageFilter< VolumeImageType,OutputImageType >  CastFilterType;
    typedef  itk::ImageFileWriter<  OutputImageType  > WriterType;


    // ----------------------------------------

    // --------- Definim els atributs ---------

    udg::Volume*                                 m_inputVolume;
    VolumeImageType::Pointer                     m_segmentedImage;
    //CastingFilterTypeIn::Pointer                 m_casterInput;
    SmoothingFilterType::Pointer                 m_smoothing;
    GradientFilterType::Pointer                  m_gradientMagnitude;
    SigmoidFilterType::Pointer                   m_sigmoid;
    FastMarchingFilterType::Pointer              m_fastMarching;
    GeodesicActiveContourFilterType::Pointer     m_geodesicActiveContour;
    //CastingFilterType::Pointer                   m_caster;
    ThresholdingFilterType::Pointer              m_thresholder;
    NodeContainer::Pointer                       m_seeds;
    VolumeImageType::IndexType                   m_seedPosition;
    NodeType                                     m_node; 
    MaskFilterType::Pointer                      m_mask;

    // ----------------------------------------

    // ****************************************

public:
    GeodesicActive();

    ~GeodesicActive();

    /// Carrega el volum a tractar
    void setInputVolume( Volume* v );

     /** Carrega els parmetres necessaris per configurar el filtre de suavitzaci
        */
    void setSmoothingParameters();

    /** En aquest mtode es pot treballar amb ms d'una llavor, per aquest motiu afegirem 
        les llavors de la llista al vector de llavors
    */
    void addSeeds( QListView* l, double distance );
    
    /** Carrega els parmetres necessaris per configurar el filtre de segmentaci
        ( sigma, alpha, beta, propagation )*/
    void setParameters( double sigma, double alpha, double beta, double propagation );
    
    /** Aplica el metode de segmentaci a la imatge passada amb els parmetres tamb
        passats.*/
    bool applyMethod();
    
    /// Retorna el volum segmentat
    udg::Volume* getSegmentedVolume();

    /// Retorna el volum binari segmentat
    udg::Volume* getBinaryVolume(); 

};

};

#endif
