/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#include "isolatedconnected.h"
#include "volume.h"
#include <iostream.h>

namespace udg {

IsolatedConnected::IsolatedConnected()
{
    m_inputVolume       = new udg::Volume();
    m_segmentedImage    = VolumeImageType::New();

    m_smoothing         = CurvatureFlowImageFilterType::New();
    m_isolatedConnected = IsolatedConnectedImageFilterType::New();
}

//destructor
IsolatedConnected::~IsolatedConnected()
{
}

/* En aquest mtode es pot treballar amb una llavor inside, per aquest motiu afegirem 
   la primera llavor de la llista al vector de llavors, l'atribut m_insideSeedsVector.
    */
void IsolatedConnected::addInsideSeeds( QListView* l )
{   
    double x,y,z;
    QListViewItemIterator it(l);
    m_insideSeedsVector.clear();

    if  ( it.current() ) //Nomes agafem la primera
    {   
        QListViewItem *item = it.current();
        x = (item->text(0)).toDouble();
	y = (item->text(1)).toDouble();
        z = (item->text(2)).toDouble();

        std::cout<<"[IsolatedConnected] addSeeds: "<< x <<" " << y << " " << z <<std::endl;
        VolumeImageIndexType seed;
        seed[0] = x;
        seed[1] = y;
        seed[2] = z;
        m_insideSeedsVector.push_back( seed );

        ++it;
    }

    
}

/* En aquest mtode es pot treballar amb una llavor outside, per aquest motiu afegirem 
   la primera llavor de la llista al vector de llavors, l'atribut m_outsideSeedsVector.
    */
void IsolatedConnected::addOutsideSeeds( QListView* l )
{   
    double x,y,z;
    QListViewItemIterator it(l);
    m_outsideSeedsVector.clear();

    if  ( it.current() ) //Nomes agafem la primera
    {   
        QListViewItem *item = it.current();
        x = (item->text(0)).toDouble();
	y = (item->text(1)).toDouble();
        z = (item->text(2)).toDouble();

        std::cout<<"[IsolatedConnected] addSeeds: "<< x <<" " << y << " " << z <<std::endl;
        VolumeImageIndexType seed;
        seed[0] = x;
        seed[1] = y;
        seed[2] = z;
        m_outsideSeedsVector.push_back( seed );

        ++it;
    }

    
}


// Carrega el volum a tractar
void IsolatedConnected::setInputVolume( Volume* v )
{
    m_inputVolume = v;
}


/* Carrega els parmetres necessaris per configurar el filtre de suavitzaci
   ( iteracions i TimeStep )*/
void IsolatedConnected::setSmoothingParameters( )
{
    m_smoothing->SetInput( m_inputVolume->getItkData() );
    m_smoothing->SetNumberOfIterations( 2 );
    m_smoothing->SetTimeStep( 0.125 );
}


/* Carrega els parmetres necessaris per configurar el filtre de segmentaci
   ( llavors, finestra inicial, iteracions, marge d'acceptaci )*/
void IsolatedConnected::setParameters( int lower )
{
    std::cout << "[IsolatedConnected] setParameters: lower: " << lower << std::endl;
    m_isolatedConnected->SetLower( lower );
    
    m_isolatedConnected->SetSeed1( m_insideSeedsVector[0] );
    std::cout<<"[IsolatedConnected] setParameters llavor Inside: "<<m_insideSeedsVector[0]<< std::endl;
    
    m_isolatedConnected->SetSeed2( m_outsideSeedsVector[0] );
    std::cout<<"[IsolatedConnected] setParameters llavor Outside: "<<m_outsideSeedsVector[0]<< std::endl;

    m_isolatedConnected->SetReplaceValue( 255 );
}


// Aplica el metode de segmentaci
bool IsolatedConnected::applyMethod()
{
    // Crea la PIPELINE
    m_isolatedConnected->SetInput( m_smoothing->GetOutput() );
    m_segmentedImage = m_isolatedConnected->GetOutput();

    // Executa la segmentaci
    try
    {   
        m_segmentedImage->Update();
    }
    catch( itk::ExceptionObject & e )
    {
        std::cerr << "IsolatedConnected: Error en la segmentaci!" << std::endl;
        std::cerr << "Exception caught! " << std::endl;
        std::cerr << e << std::endl;
        return false;
    }
 
    //-------------------- Resultats ------------------

    return true; 
}


// Retorna el volum segmentat
udg::Volume* IsolatedConnected::getSegmentedVolume()
{
    m_mask=MaskFilterType::New();
    m_mask->SetInput1(m_inputVolume->getItkData());
    m_mask->SetInput2(m_segmentedImage);

    udg::Volume* volume = new udg::Volume();
    volume->setData( m_mask->GetOutput() );
    return volume; 
}

udg::Volume* IsolatedConnected::getBinaryVolume()
{
    udg::Volume* volume = new udg::Volume();
    volume->setData( m_segmentedImage );
    return volume; 
}
 
};
