/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#ifndef UDGISOLATEDCONNECTED_H
#define UDGISOLATEDCONNECTED_H

#include <qobject.h>
#include <itkIsolatedConnectedImageFilter.h>
#include <itkImage.h>
#include <itkCastImageFilter.h>
#include <itkCurvatureFlowImageFilter.h>
#include <itkMaskImageFilter.h>

//Per tenir accs als tipus d'imatges de volum
#include <vector.h>
#include "volume.h"

#include "qlistview.h"

//class Volume;

using namespace std;

namespace udg {

/**
@author Grup de Grfics de Girona  ( GGG )
*/

/**
    Aquesta classe, IsolatedConnected s un mtode de segmentaci usant tcniques de
    Region Growing. Est pensat per trobar la  regi que volem segmentar utilitzant
    una llavor i avaluant la intensitat dels pxels vens segons un criteri que nosaltres
    determinarem.
    
    Aquesta classe prepara tot el tractament de la imatge, rep tots els parmetres necessaris
    per executar el mtode de segmentaci (imatge a segmentar inclosa) i executa tot el procs de 
    segmentaci.
*/



class IsolatedConnected{
private:

// *************** ATRIBUTS ***************

    // ----- Definim els tipus d'objectes -----
    
    // Definim un tipus d'imatge, el tipus d'imateg del volum.
    typedef udg::Volume::ItkImageType VolumeImageType;
    // I el tipus dels seus ndexs.
    typedef VolumeImageType::IndexType    VolumeImageIndexType;

    // Definim filtre de suavitzaci
    typedef itk::CurvatureFlowImageFilter< VolumeImageType, VolumeImageType > CurvatureFlowImageFilterType;

    // Definim filtre de segmentaci Region Growing
    typedef itk::IsolatedConnectedImageFilter< VolumeImageType, VolumeImageType >IsolatedConnectedImageFilterType;

    typedef itk::MaskImageFilter< VolumeImageType, VolumeImageType, VolumeImageType > MaskFilterType;
    // ----------------------------------------

    // --------- Definim els atributs ---------

    udg::Volume*                                 m_inputVolume;
    VolumeImageType::Pointer                     m_segmentedImage;
    vector<VolumeImageIndexType>                 m_insideSeedsVector;
    vector<VolumeImageIndexType>                 m_outsideSeedsVector;
    CurvatureFlowImageFilterType::Pointer        m_smoothing;
    IsolatedConnectedImageFilterType::Pointer    m_isolatedConnected;
    MaskFilterType::Pointer                      m_mask;

    // ----------------------------------------

    // ****************************************

public:
    IsolatedConnected();

    ~IsolatedConnected();

    /// Carrega el volum a tractar
    void setInputVolume( Volume* v );

     /** Carrega els parmetres necessaris per configurar el filtre de suavitzaci
        ( iteracions i TimeStep )*/
    void setSmoothingParameters();

    /** En aquest mtode es pot treballar amb una llavor inside, per aquest motiu afegirem 
        la primera llavor de la llista al vector de llavors, l'atribut m_insideSeedsVector.
    */
    void addInsideSeeds( QListView* l );

    /** En aquest mtode es pot treballar amb una llavor outside, per aquest motiu afegirem 
        la primera llavor de la llista al vector de llavors, l'atribut m_outsideSeedsVector.
    */
    void addOutsideSeeds( QListView* l );
    
    /** Carrega els parmetres necessaris per configurar el filtre de segmentaci
        ( lower threshold, upper threshold )*/
    void setParameters( int lower );
    
    /** Aplica el metode de segmentaci a la imatge passada amb els parmetres tamb
        passats.*/
    bool applyMethod();
    
    /// Retorna el volum segmentat
    udg::Volume* getSegmentedVolume(); 

    /// Retorna el volum binari segmentat
    udg::Volume* getBinaryVolume();

};

};

#endif
