/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#include "neighborhoodconnected.h"
#include "volume.h"
#include <iostream.h>

namespace udg {

NeighborhoodConnected::NeighborhoodConnected()
{
    m_inputVolume       = new udg::Volume();
    m_segmentedImage    = VolumeImageType::New();

    m_smoothing         = CurvatureFlowImageFilterType::New();
    m_neighborhoodConnected= NeighborhoodConnectedImageFilterType::New();
}

//destructor
NeighborhoodConnected::~NeighborhoodConnected()
{
}

/* En aquest mtode es pot treballar amb ms d'una llavor, per aquest motiu afegirem 
   les llavors de la llista al vector de llavors, l'atribut m_seedsVector.    
*/
void NeighborhoodConnected::addSeeds( QListView* l )
{   
    double x,y,z;
    QListViewItemIterator it(l);
    m_seedsVector.clear();

    while  ( it.current() ) //Mentre hi hagin elements
    {   
        QListViewItem *item = it.current();
        x = (item->text(0)).toDouble();
	y = (item->text(1)).toDouble();
        z = (item->text(2)).toDouble();

        std::cout<<"[NeighborhoodConnected] addSeeds: "<< x <<" " << y << " " << z <<std::endl;
        VolumeImageIndexType seed;
        seed[0] = x;
        seed[1] = y;
        seed[2] = z;
        m_seedsVector.push_back( seed );

        ++it;
    }

    
}


// Carrega el volum a tractar
void NeighborhoodConnected::setInputVolume( Volume* v )
{
    m_inputVolume = v;
}


/* Carrega els parmetres necessaris per configurar el filtre de suavitzaci
   ( iteracions i TimeStep )*/
void NeighborhoodConnected::setSmoothingParameters( )
{
    m_smoothing->SetInput( m_inputVolume->getItkData() );
    m_smoothing->SetNumberOfIterations( 2 );
    m_smoothing->SetTimeStep( 0.125 );
}


/* Carrega els parmetres necessaris per configurar el filtre de segmentaci
   ( llavors, finestra inicial, iteracions, marge d'acceptaci )*/
void NeighborhoodConnected::setParameters( int lower, int upper, double radi )
{
    std::cout << "[NeighborhoodConnected] setParameters: lower: " << lower << " upper: " << upper << std::endl;
    m_neighborhoodConnected->SetLower( lower );
    m_neighborhoodConnected->SetUpper( upper );    
    
    //InternalImageType::SizeType   radius;
    VolumeImageType::SizeType   radius;
    radius[0] = radi; // eix X
    radius[1] = radi; // eix Y
    radius[2] = radi; // eix Z
    m_neighborhoodConnected->SetRadius( radius );
    std::cout<<"[NeighborhoodConnected] setParameters radius: "<< radi << std::endl;
    
    m_neighborhoodConnected->SetSeed( m_seedsVector[0] );
    std::cout<<"[NeighborhoodConnected] setParameters llavor 0: "<<m_seedsVector[0]<< std::endl;
    
    for (unsigned int i=1; i<m_seedsVector.size(); i++)
    {
        m_neighborhoodConnected->AddSeed( m_seedsVector[i] );
        std::cout<<"[NeighborhoodConnected] setParameters llavor " << i << ": " <<m_seedsVector[i]<< std::endl;
    }
    m_neighborhoodConnected->SetReplaceValue( 255 );
}


// Aplica el metode de segmentaci
bool NeighborhoodConnected::applyMethod()
{   
    // Crea la PIPELINE
    m_neighborhoodConnected->SetInput( m_smoothing->GetOutput() );
    m_segmentedImage = m_neighborhoodConnected->GetOutput();

    // Executa la segmentaci
    try
    {
        m_segmentedImage->Update();
    }
    catch( itk::ExceptionObject & e )
    {   
        std::cerr << "NeighborhoodConnected: Error en la segmentaci!" << std::endl;
        std::cerr << "Exception caught! " << std::endl;
        std::cerr << e << std::endl;
        return false;
    }

    //-------------------- Resultats ------------------

    return true; 
}


// Retorna el volum segmentat
udg::Volume* NeighborhoodConnected::getSegmentedVolume()
{   
    m_mask=MaskFilterType::New();
    m_mask->SetInput1(m_inputVolume->getItkData());
    m_mask->SetInput2(m_segmentedImage);

    udg::Volume* volume = new udg::Volume();
    volume->setData( m_mask->GetOutput() );
    return volume;   
}

udg::Volume* NeighborhoodConnected::getBinaryVolume()
{
    udg::Volume* volume = new udg::Volume();
    volume->setData( m_segmentedImage );
    return volume; 
}

};
