/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#ifndef UDGNEIGHBORHOODCONNECTED_H
#define UDGNEIGHBORHOODCONNECTED_H

#include <qobject.h>
#include "itkNeighborhoodConnectedImageFilter.h"
#include <itkImage.h>
#include <itkCastImageFilter.h>
#include <itkCurvatureFlowImageFilter.h>
#include <itkMaskImageFilter.h>

//Per tenir accs als tipus d'imatges de volum
#include <vector.h>
#include "volume.h"

#include "qlistview.h"

//class Volume;

using namespace std;

namespace udg {

/**
@author Grup de Grfics de Girona  ( GGG )
*/

/**
    Aquesta classe, NeighborhoodConnected s un mtode de segmentaci usant tcniques de
    Region Growing. Est pensat per trobar la  regi que volem segmentar utilitzant
    una llavor i avaluant la intensitat dels pxels vens segons un criteri que nosaltres
    determinarem.
    
    Aquesta classe prepara tot el tractament de la imatge, rep tots els parmetres necessaris
    per executar el mtode de segmentaci (imatge a segmentar inclosa) i executa tot el procs de 
    segmentaci.
*/



class NeighborhoodConnected{
private:

// *************** ATRIBUTS ***************

    // ----- Definim els tipus d'objectes -----
    
    // Definim un tipus d'imatge, el tipus d'imateg del volum.
    typedef udg::Volume::ItkImageType VolumeImageType;
    // I el tipus dels seus ndexs.
    typedef VolumeImageType::IndexType    VolumeImageIndexType;

    // Definim filtre de suavitzaci
    typedef itk::CurvatureFlowImageFilter< VolumeImageType, VolumeImageType > CurvatureFlowImageFilterType;

    // Definim filtre de segmentaci Region Growing
    typedef itk::NeighborhoodConnectedImageFilter< VolumeImageType, VolumeImageType >NeighborhoodConnectedImageFilterType;

    typedef itk::MaskImageFilter< VolumeImageType, VolumeImageType, VolumeImageType > MaskFilterType;
    // ----------------------------------------

    // --------- Definim els atributs ---------

    udg::Volume*                                 m_inputVolume;
    VolumeImageType::Pointer                     m_segmentedImage;
    vector<VolumeImageIndexType>                 m_seedsVector;
    CurvatureFlowImageFilterType::Pointer        m_smoothing;
    NeighborhoodConnectedImageFilterType::Pointer   m_neighborhoodConnected;
    MaskFilterType::Pointer                      m_mask;

    // ----------------------------------------

    // ****************************************

public:
    NeighborhoodConnected();

    ~NeighborhoodConnected();

    /// Carrega el volum a tractar
    void setInputVolume( Volume* v );

     /** Carrega els parmetres necessaris per configurar el filtre de suavitzaci
        ( iteracions i TimeStep )*/
    void setSmoothingParameters();

    /** En aquest mtode es pot treballar amb ms d'una llavor, per aquest motiu afegirem 
        les llavors de la llista al vector de llavors, l'atribut m_seedsVector.
    */
    void addSeeds( QListView* l );
    
    /** Carrega els parmetres necessaris per configurar el filtre de segmentaci
        ( lower threshold, upper threshold i radius )*/
    void setParameters( int lower, int upper, double radius );
    
    /** Aplica el metode de segmentaci a la imatge passada amb els parmetres tamb
        passats.*/
    bool applyMethod();
    
    /// Retorna el volum segmentat
    udg::Volume* getSegmentedVolume(); 

    /// Retorna el volum binari segmentat
    udg::Volume* getBinaryVolume();

};

};

#endif
