/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#include "shapedetection.h"
#include "volume.h"
#include <iostream.h>

namespace udg {

ShapeDetection::ShapeDetection()
{
    m_inputVolume       = new udg::Volume();
    m_segmentedImage    = VolumeImageType::New();
    //m_casterInput       = CastingFilterTypeIn::New();

    m_smoothing         = SmoothingFilterType::New();
    m_gradientMagnitude = GradientFilterType::New();
    m_sigmoid           = SigmoidFilterType::New();
    m_fastMarching      = FastMarchingFilterType::New();
    m_shapeDetection    = ShapeDetectionFilterType::New();
    m_thresholder       = ThresholdingFilterType::New();

    m_seeds             = NodeContainer::New();
}

//destructor
ShapeDetection::~ShapeDetection()
{
}

/* En aquest mtode es pot treballar amb ms d'una llavor, per aquest motiu afegirem 
   les llavors de la llista al vector de llavors. 
*/
void ShapeDetection::addSeeds( QListView* l, double distance )
{
    double x,y,z;
    QListViewItemIterator it(l);
    m_seeds->Initialize();
    int aux = 0;

    while  ( it.current() ) //Mentre hi hagin elements
    {   
        QListViewItem *item = it.current();
        x = (item->text(0)).toDouble();
	y = (item->text(1)).toDouble();
        z = (item->text(2)).toDouble();
        std::cout<<"[ShapeDetection] addSeeds: "<< x <<" " << y << " " << z <<std::endl;

        m_seedPosition[0] = x;
        m_seedPosition[1] = y;
        m_seedPosition[2] = z;
        m_node.SetValue( - distance );
        m_node.SetIndex( m_seedPosition );
        m_seeds->InsertElement( aux, m_node );

        ++it;
        aux++;
    }
}


// Carrega el volum a tractar
void ShapeDetection::setInputVolume( Volume* v )
{
    m_inputVolume = v;
}


/* Carrega els parmetres necessaris per configurar el filtre de suavitzaci
   ( iteracions i TimeStep )*/
void ShapeDetection::setSmoothingParameters( )
{
    //m_casterInput->SetInput( m_inputVolume->getItkData() );
    //m_smoothing->SetInput( m_casterInput->GetOutput() );
    m_smoothing->SetInput( m_inputVolume->getItkData() );
    m_smoothing->SetTimeStep( 0.0625 );
    m_smoothing->SetNumberOfIterations(  5 );
    m_smoothing->SetConductanceParameter( 9.0 );
}


/* Carrega els parmetres necessaris per configurar el filtre de segmentaci
*/
void ShapeDetection::setParameters( double sigma, double alpha, double beta, double curvature, double propagation )
{
   m_gradientMagnitude->SetInput( m_smoothing->GetOutput() );
   m_gradientMagnitude->SetSigma(  sigma  );
   std::cout << "[ShapeDetection] setParameters: sigma: " << sigma << std::endl;

   m_sigmoid->SetInput( m_gradientMagnitude->GetOutput() );
   m_sigmoid->SetOutputMinimum(  0.0  );
   m_sigmoid->SetOutputMaximum(  1.0  );
   m_sigmoid->SetAlpha( alpha );
   m_sigmoid->SetBeta(  beta  );
   std::cout << "[ShapeDetection] setParameters: alpha: " << alpha << std::endl;
   std::cout << "[ShapeDetection] setParameters: beta: " << beta << std::endl;

   m_fastMarching->SetInput( m_sigmoid->GetOutput() );
   m_fastMarching->SetTrialPoints(  m_seeds  );
   m_fastMarching->SetSpeedConstant( 1.0 );
   m_fastMarching->SetOutputSize( m_inputVolume->getItkData()->GetBufferedRegion().GetSize() );   
   m_fastMarching->SetStoppingValue( 250  );

   m_shapeDetection->SetInput( m_fastMarching->GetOutput() );
   m_shapeDetection->SetFeatureImage( m_sigmoid->GetOutput() );
   m_shapeDetection->SetPropagationScaling(  propagation );
   m_shapeDetection->SetCurvatureScaling( curvature ); 
   m_shapeDetection->SetMaximumRMSError( 0.02 );
   m_shapeDetection->SetNumberOfIterations( 800 );
   std::cout << "[ShapeDetection] setParameters: curvature: " << curvature << std::endl;
   std::cout << "[ShapeDetection] setParameters: propagation: " << propagation << std::endl;

   m_thresholder->SetInput( m_shapeDetection->GetOutput() );
   m_thresholder->SetLowerThreshold( -1000.0  );
   m_thresholder->SetUpperThreshold( 0.0  );
   m_thresholder->SetOutsideValue(  0  );
   m_thresholder->SetInsideValue(  255 );
   //cout << "1" << endl;
}


// Aplica el metode de segmentaci
bool ShapeDetection::applyMethod()
{   //cout << "2" << endl;
    // Crea la PIPELINE
    m_segmentedImage = m_thresholder->GetOutput();

    // Executa la segmentaci
    try
    {   //cout << "3" << endl;
        m_segmentedImage->Update(); //cout << "4" << endl;
    }
    catch( itk::ExceptionObject & e )
    {
        std::cerr << "ShapeDetection: Error en la segmentaci!" << std::endl;
        std::cerr << "Exception caught! " << std::endl;
        std::cerr << e << std::endl;
        return false;
    }
 
    //-------------------- Resultats ------------------

    return true; 
}


// Retorna el volum segmentat
udg::Volume* ShapeDetection::getSegmentedVolume()
{   //cout << "5" << endl;
    m_mask=MaskFilterType::New();
    m_mask->SetInput1(m_inputVolume->getItkData());
    m_mask->SetInput2(m_segmentedImage);

    udg::Volume* volume = new udg::Volume();
    volume->setData( m_mask->GetOutput() );
    return volume;   //cout << "6" << endl;
} 

udg::Volume* ShapeDetection::getBinaryVolume()
{
    udg::Volume* volume = new udg::Volume();
    volume->setData( m_segmentedImage );
    return volume; 
}

};
