/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#include "raycasting.h"
#include <iostream.h>
#include "volume.h"
#include <vtkVolume.h>

namespace udg {

RayCasting::RayCasting()
{
    m_inputVolume = new udg::Volume();

    m_caster = vtkImageCast::New();
    m_opacityTransferFunction = vtkPiecewiseFunction::New();
    m_colorTransferFunction = vtkColorTransferFunction::New();
    m_volumeProperty = vtkVolumeProperty::New();
    m_compositeFunction = vtkVolumeRayCastCompositeFunction::New();
    m_volumeMapper =  vtkVolumeRayCastMapper::New();   
    m_volume = vtkVolume::New();
}

//destructor
RayCasting::~RayCasting()
{
}


void RayCasting::setTransferFunction( QTable* l )
{
    double scalar1,scalar2,opacity1,opacity2;
    double auxscalar1,auxscalar2;
    double* range;
    double red,green,blue;
    QColor rgb;

    m_opacityTransferFunction->ClampingOff();
    m_opacityTransferFunction->RemoveAllPoints();
    m_colorTransferFunction->ClampingOff();
    m_colorTransferFunction->RemoveAllPoints();

   for (int i = 0; i < l->numRows(); i++)
   {
        scalar1=l->text(i,0).toDouble();
        opacity1=l->text(i,1).toDouble();
        scalar2=l->text(i,2).toDouble();
        opacity2=l->text(i,3).toDouble();
        rgb=((ColorTableItem *) (l->item(i,4)))->getColor();

        red=rgb.red();
        green=rgb.green();
        blue=rgb.blue();            
        red=red/255; green=green/255; blue=blue/255; 

        //Traducci del rang del model al rang 0-255
        range=m_inputVolume->getVtkData()->GetScalarRange();
        //cout << "Scalar Min: " << range[0] << endl;
        //cout << "Scalar Max: " << range[1] << endl;

        auxscalar1=scalar1+(range[0]*-1);
        auxscalar2=scalar2+(range[0]*-1);

        auxscalar1=auxscalar1/(range[1]+(range[0]*-1));
        auxscalar2=auxscalar2/(range[1]+(range[0]*-1));

        auxscalar1=auxscalar1*255;
        auxscalar2=auxscalar2*255;

        //cout << "AuxScalar1: " << auxscalar1 << " AuxScalar2: " << auxscalar2 << endl;

        m_opacityTransferFunction->AddSegment(auxscalar1,opacity1,auxscalar2,opacity2);
        m_colorTransferFunction->AddRGBSegment(auxscalar1,red,green,blue,auxscalar2,red,green,blue);

        /*
        m_opacityTransferFunction->AddPoint(40,0.0);
        m_opacityTransferFunction->AddPoint(255,1.0);
        m_colorTransferFunction->AddRGBPoint (0, 0.0, 0.0, 0.0);
        m_colorTransferFunction->AddRGBPoint (255, 1.0, 1.0, 1.0);
        */
    }
}



// Carrega el volum a tractar
void RayCasting::setInputVolume( Volume* v )
{
    m_inputVolume = v;
}


/// Carrega els parmetres necessaris per configurar el filtre de visualitzaci
void RayCasting::setParameters( )
{
     /*if (interpolation==0) { m_volumeProperty->SetInterpolationTypeToLinear();}
     else if (interpolation==1) { m_volumeProperty->SetInterpolationTypeToNearest();}*/

    m_volumeProperty->SetInterpolationTypeToLinear();
    m_volumeMapper->SetSampleDistance(0.5);
}


// Aplica el metode de segmentaci
bool RayCasting::applyMethod()
{
    m_caster->SetInput( m_inputVolume->getVtkData() );
    m_caster->SetOutputScalarType( VTK_UNSIGNED_CHAR ); // tamb seria vlid VTK_UNSIGNED_SHORT
    m_caster->ClampOverflowOn();
    m_caster->Update();

    m_volumeProperty->SetColor( m_colorTransferFunction );
    m_volumeProperty->SetScalarOpacity( m_opacityTransferFunction );
    m_volumeProperty->ShadeOn();    

    // el mapper (funcio de ray cast) sabr com visualitzar les dades
    m_volumeMapper->SetVolumeRayCastFunction( m_compositeFunction );
    m_volumeMapper->SetInput( m_caster->GetOutput() ); // abans inputImage->getVtkData()

    // el volum cont el mapper i la propietat i es pot usar per posicionar/orientar el volum
    m_volume->SetMapper( m_volumeMapper );
    m_volume->SetProperty( m_volumeProperty );

    m_volume->SetOrientation(0,0,0);

    m_volume->RotateX(180);
    m_volume->RotateY(180);

    return true;  
}


// Retorna el volum
vtkVolume* RayCasting::getVolume()
{
    return m_volume;
}

// Retorna el mapper
vtkVolumeRayCastMapper* RayCasting::getMapper()
{
    return m_volumeMapper;
}

};
