/***************************************************************************
 *   Copyright (C) 2005 by Grup de Grfics de Girona                       *
 *   http://iiia.udg.es/GGG/index.html?langu=uk                            *
 *                                                                         *
 *   Universitat de Girona                                                 *
 ***************************************************************************/
#ifndef UDGRAYCASTING_H
#define UDGRAYCASTING_H

//includes vtk

#include <vtkProperty.h>
#include <vtkImageActor.h>
#include <vtkVolume.h>

//includes Ray Casting
#include <vtkVolumeRayCastCompositeFunction.h>
#include <vtkVolumeRayCastMapper.h>
#include <vtkVolumeProperty.h>
#include <vtkColorTransferFunction.h>
#include <vtkPiecewiseFunction.h>

//includes per fer el casting i passar al tipus correcte
#include <vtkImageCast.h> 
#include <vtkImageData.h>

#include "qtable.h"
#include "ColorTableItem.h"

//class Volume;

using namespace std;

namespace udg {

class Volume;

/**
@author Grup de Grfics de Girona  ( GGG )
*/

/**
    Aquesta classe, RayCasting s un mtode de visualitzaci.
    
    Aquesta classe prepara tot el tractament de la imatge, rep tots els parmetres necessaris
    per executar el mtode de visualitzaci (imatge a segmentar inclosa) i executa tot el procs de 
    visualitzaci.
*/



class RayCasting{
private:

    // *************** ATRIBUTS ***************

    udg::Volume* m_inputVolume;
    vtkImageCast* m_caster;    
    vtkPiecewiseFunction* m_opacityTransferFunction;
    vtkColorTransferFunction* m_colorTransferFunction;
    vtkVolumeProperty* m_volumeProperty;
    vtkVolumeRayCastCompositeFunction* m_compositeFunction;
    vtkVolumeRayCastMapper* m_volumeMapper;   
    vtkVolume* m_volume;

    // ****************************************

public:
    RayCasting();

    ~RayCasting();

    /// Carrega el volum a tractar
    void setInputVolume( Volume* v );

    void setTransferFunction( QTable* l );

    /** Carrega els parmetres necessaris per configurar el filtre de visualitzaci
        ( interpolation, distance )*/
    void setParameters( );

    /** Aplica el metode de visualitzaci a la imatge passada amb els parmetres tamb
        passats.*/
    bool applyMethod();
    
    /// Retorna el volum segmentat
    vtkVolume* getVolume(); 

    /// Retorna el mapper
    vtkVolumeRayCastMapper* getMapper(); 

};

};

#endif
